@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -15,7 +15,7 @@
 #define ADIF_INT(iname,item) sprintf(s,"%d",item); g_string_sprintfa(gs,"<%s:%d>%s  ",iname, (int)strlen(s),s)
 
 
-gchar *convert_lt(gchar *s){
+static gchar *convert_lt(gchar *s){
     char *c;
 
     for (c=s; *c!='\0'; c++)
@@ -23,7 +23,7 @@ gchar *convert_lt(gchar *s){
     return s;
 }
 
-int write_adif_header(struct band *b, struct config_band *confb, FILE *f){
+static int write_adif_header(struct band *b, struct config_band *confb, FILE *f){
     char s[1024],dummy[10], *psect;
     GString *gs;
     int ret;
@@ -35,7 +35,7 @@ int write_adif_header(struct band *b, struct config_band *confb, FILE *f){
     }
     
     gs = g_string_sized_new(1024);
-    g_string_sprintfa(gs, "Created by TUCNAK ver. "VERSION_STRING"\r\n");
+    g_string_sprintfa(gs, "Created by TUCNAK ver. %s\r\n", VERSION_STRING);
     g_string_sprintfa(gs, "TName=%s\r\n", safe_strncpy0(s, ctest->tname, 69));
     if (!b->qsos->len)
         g_string_sprintfa(gs, "TDate=%s;%s\r\n", ctest->cdate, ctest->cdate); 
@@ -86,7 +86,7 @@ int write_adif_header(struct band *b, struct config_band *confb, FILE *f){
     return ret;
 }
 
-int write_adif_qsos(struct band *b, struct config_band *confb, FILE *f){
+static int write_adif_qsos(struct band *b, struct config_band *confb, FILE *f){
     struct qso *q;
     GString *gs;
     int i;
@@ -123,15 +123,23 @@ int write_adif_qsos(struct band *b, struct config_band *confb, FILE *f){
         ADIF_STR   ("MODE",adif_modes[abs(q->mode)%10]); /* TODO sent vs rcvd */
         ADIF_STR_UC("RST_SENT",q->rsts);
         ADIF_STR_UC("NO_SENT",q->qsonrs);
-        ADIF_STR_UC("STX",q->qsonrs);
+        if (ctest->excused > 0 && ctest->qsoused == 0){
+            ADIF_STR_UC("STX",ctest->pexch);
+        }else{
+            ADIF_STR_UC("STX",q->qsonrs);
+        }
         ADIF_STR_UC("RST_RCVD",q->rstr);
-        ADIF_STR_UC("SRX",q->qsonrr);
+        if (ctest->excused > 0 && ctest->qsoused == 0){
+            ADIF_STR_UC("SRX",q->exc);
+        }else{
+            ADIF_STR_UC("SRX",q->qsonrr);
+        }
         ADIF_STR_UC("NO_RCVD",q->qsonrr); 
         ADIF_STR_UC("GRIDSQUARE",q->locator);
         ADIF_INT   ("QRB", (int)q->qrb);
         ADIF_INT   ("QTF", q->qtf);
         ADIF_INT   ("POINTS", q->qsop);
-        ADIF_STR_UC("OPERATOR",q->operator);
+        ADIF_STR_UC("OPERATOR",q->operator_);
         ADIF_STR   ("COMMENT",q->remark);
         
         /* ES,MS,AUR,TR,RS,AUE,BS,EME,FAI,F2,ION,LOS,SAT,TEP,TRS,TRD */
@@ -144,6 +152,11 @@ int write_adif_qsos(struct band *b, struct config_band *confb, FILE *f){
         }else{
             ADIF_STR   ("PROP_MODE","TR");
         }
+        
+        if (q->qrg){
+            sprintf(s,"%d.%06d",(int)(q->qrg/1000000.0), (int)fmod(q->qrg, 1000000.0)); 
+            g_string_sprintfa(gs,"<%s:%d>%s  ", "FREQ", (int)strlen(s),s);
+        }
             
         
         g_string_sprintfa(gs, "<eor>\r\n");
@@ -211,7 +224,7 @@ int export_all_bands_adif(void){
 }
 
 
-int adif_ignore(FILE *f){
+static int adif_ignore(FILE *f){
     int cc;
 
     while ((cc=fgetc(f))!=EOF){
@@ -220,7 +233,7 @@ int adif_ignore(FILE *f){
     return EOF;
 }
 
-int adif_read_tag(FILE *f, GString *gs, int *taglen){
+static int adif_read_tag(FILE *f, GString *gs, int *taglen){
     int cc;
     gchar **items;
     g_string_truncate(gs, 0);
@@ -244,19 +257,19 @@ int adif_read_tag(FILE *f, GString *gs, int *taglen){
     return EOF;
 }
 
-gchar *adif_item(GHashTable *hash, gchar *key) {
+static gchar *adif_item(GHashTable *hash, gchar *key) {
     return g_hash_table_lookup(hash,key);
 }
 #define ADIF_ITEM(key) adif_item(hash,(key))
 
-gchar *safe_adif_item(GHashTable *hash, gchar *key) {
+static gchar *safe_adif_item(GHashTable *hash, gchar *key) {
     gchar *tmp;
     tmp=g_hash_table_lookup(hash,key);
     return (tmp?tmp:"");
 }
 #define SAFE_ADIF_ITEM(key) safe_adif_item(hash,(key))
 
-int adif_add_qso(struct band *band, GHashTable *hash){
+static int adif_add_qso(struct band *band, GHashTable *hash){
     struct qso *q;
     struct band *b;
     int i;
@@ -357,8 +370,9 @@ int adif_add_qso(struct band *band, GHashTable *hash){
 
     q->source=g_strdup(net->myid);
 
-    q->operator=fixsemi(g_strdup(SAFE_ADIF_ITEM("OPERATOR")));
+    q->operator_=fixsemi(g_strdup(SAFE_ADIF_ITEM("OPERATOR")));
     
+    q->qrg = 0;
 
     q->stamp=time(NULL);
 
@@ -376,7 +390,7 @@ int adif_add_qso(struct band *band, GHashTable *hash){
 
 void import_adif(void *xxx, char *filename){
     FILE *f;
-    int cc, taglen;
+    int cc, taglen = 0;
     GString *gs, *gs2;
     GHashTable *hash;
     int gs2size;
@@ -423,15 +437,16 @@ void import_adif(void *xxx, char *filename){
     g_string_free(gs2, TRUE);
     g_hash_table_foreach_remove(hash, free_gstr, NULL);
     
-    g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
-/*    g_hash_table_foreach(ctest->bystamp, foreach_source_print, NULL);*/
+    
+	g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
+    g_index_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
 
 /*   don't needed   DIRTY_BAND(); */
 
     for (i=0;i<ctest->bands->len;i++){
         struct band *b=(struct band *)g_ptr_array_index(ctest->bands,i);
         recalc_stats(b);
-        clear_tmpqsos(b);
+        clear_tmpqsos(b, 1);
         
         if (ctest->qsoused && b->qsos->len+1 != atoi(b->tmpqsos[0].qsonrs)){
             g_free(b->tmpqsos[0].qsonrs);
