@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -59,7 +59,7 @@ void freeml(struct memory_list *ml)
 char m_bar = 0;
 
 
-void do_menu_selected(struct menu_item *items, union cba_t data, int selected)
+void do_menu_selected(struct menu_item *items, cba_t data, int selected)
 {
     struct menu *menu;
     if ((menu = mem_alloc(sizeof(struct menu)))) {
@@ -78,19 +78,19 @@ void do_menu_selected(struct menu_item *items, union cba_t data, int selected)
     }
 }
 
-void do_menu(struct menu_item *items, union cba_t data)
+void do_menu(struct menu_item *items, cba_t data)
 {
     do_menu_selected(items, data, 0);
 }
 
-void select_menu(struct menu *menu)
+static void select_menu(struct menu *menu)
 {
     /*int x = menu->x + 4;
     int y = menu->y + menu->selected - menu->view + 2;*/
     struct menu_item *it = &menu->items[menu->selected];
-    void (*func)(union cba_t, union cba_t) = it->func;
-    union cba_t data1 = it->data;
-    union cba_t data2 = menu->data;
+    void (*func)(cba_t, cba_t) = it->func;
+    cba_t data1 = it->data;
+    cba_t data2 = menu->data;
     
     if (menu->selected < 0 || menu->selected >= menu->ni || it->hotkey == M_BAR) return;
     if (!it->in_m) {
@@ -101,7 +101,7 @@ void select_menu(struct menu *menu)
     func(data1, data2);
 }
 
-void count_menu_size(struct terminal *term, struct menu *menu)
+static void count_menu_size(struct terminal *term, struct menu *menu)
 {
     int sx = term->x;
     int sy = term->y;
@@ -123,7 +123,7 @@ void count_menu_size(struct terminal *term, struct menu *menu)
     if (menu->y + my > sy) menu->y = sy - my;
 }
 
-void scroll_menu(struct menu *menu, int d)
+static void scroll_menu(struct menu *menu, int d)
 {
     int c = 0;
     int w = menu->yw - 2;
@@ -152,7 +152,7 @@ void scroll_menu(struct menu *menu, int d)
     /*debug("2:%d %d %d %d", menu->ni, w, menu->view, menu->selected);*/
 }
 
-void display_menu(struct menu *menu)
+static void display_menu(struct menu *menu)
 {
     int p, s;
     fill_area(menu->x+1, menu->y+1, menu->xw-2, menu->yw-2, COLOR_MENU1);
@@ -187,7 +187,7 @@ void display_menu(struct menu *menu)
 void menu_func(struct window *win, struct event *ev, int fwd)
 {
     int s = 0;
-    struct menu *menu = win->data;
+    struct menu *menu = (struct menu*)win->data;
     struct window *w1;
     menu->win = win;
     switch (ev->ev) {
@@ -212,7 +212,7 @@ void menu_func(struct window *win, struct event *ev, int fwd)
                         break;
                     }
                     if (w1->handler != menu_func) break;
-                    m1 = w1->data;
+                    m1 = (struct menu*)w1->data;
                     if (ev->x > m1->x && ev->x < m1->x+m1->xw-1 && ev->y > m1->y && ev->y < m1->y+m1->yw-1) goto del;
                 }
             } else {
@@ -294,7 +294,7 @@ void menu_func(struct window *win, struct event *ev, int fwd)
     }
 }
 
-void do_mainmenu(struct menu_item *items, union cba_t data, int sel)
+void do_mainmenu(struct menu_item *items, cba_t data, int sel)
 {
     struct mainmenu *menu;
     if ((menu = mem_alloc(sizeof(struct mainmenu)))) {
@@ -310,7 +310,7 @@ void do_mainmenu(struct menu_item *items, union cba_t data, int sel)
     }
 }
 
-void display_mainmenu(struct mainmenu *menu)
+static void display_mainmenu(struct mainmenu *menu)
 {
     int i;
     int p = 2;
@@ -337,7 +337,7 @@ void display_mainmenu(struct mainmenu *menu)
     redraw_from_window(menu->win);
 }
 
-void select_mainmenu(struct mainmenu *menu)
+static void select_mainmenu(struct mainmenu *menu)
 {
     struct menu_item *it = &menu->items[menu->selected];
     if (menu->selected < 0 || menu->selected >= menu->ni || it->hotkey == M_BAR) return;
@@ -351,7 +351,7 @@ void select_mainmenu(struct mainmenu *menu)
 void mainmenu_func(struct window *win, struct event *ev, int fwd)
 {
     int s = 0;
-    struct mainmenu *menu = win->data;
+    struct mainmenu *menu = (struct mainmenu *)win->data;
     menu->win = win;
     switch(ev->ev) {
         case EV_INIT:
@@ -419,18 +419,18 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
 struct menu_item *new_menu(int free_i)
 {
     struct menu_item *mi;
-    if (!(mi = mem_alloc(sizeof(struct menu_item)))) return NULL;
+    if (!(mi = (struct menu_item*)mem_alloc(sizeof(struct menu_item)))) return NULL;
     memset(mi, 0, sizeof(struct menu_item));
     mi->free_i = free_i;
     return mi;
 }
 
-void add_to_menu(struct menu_item **mi, char *text, char *rtext, char *hotkey, void (*func)(union cba_t, union cba_t), union cba_t data, int in_m)
+void add_to_menu(struct menu_item **mi, char *text, char *rtext, char *hotkey, void (*func)(cba_t, cba_t), cba_t data, int in_m)
 {
     struct menu_item *mii;
     int n;
     for (n = 0; (*mi)[n].text; n++) ;
-    if (!(mii = mem_realloc(*mi, (n + 2) * sizeof(struct menu_item)))) return;
+    if (!(mii = (struct menu_item*)mem_realloc(*mi, (n + 2) * sizeof(struct menu_item)))) return;
     *mi = mii;
     memcpy(mii + n + 1, mii + n, sizeof(struct menu_item));
     mii[n].text = text;
@@ -447,7 +447,7 @@ void do_dialog(struct dialog *dlg, struct memory_list *ml)
     struct dialog_item *d;
     int n = 0;
     for (d = dlg->items; d->type != D_END; d++) n++;
-    if (!(dd = mem_alloc(sizeof(struct dialog_data) + sizeof(struct dialog_item_data) * n))) return;
+    if (!(dd = (struct dialog_data *)mem_alloc(sizeof(struct dialog_data) + sizeof(struct dialog_item_data) * n))) return;
     dd->dlg = dlg;
     dd->n = n;
     dd->ml = ml;
@@ -467,6 +467,18 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
                 set_window_ptr(dlg->win, di->x, di->y);
             }
             break;
+        case D_CHECKBOX3:
+            if (di->checked==2) 
+                print_text(di->x, di->y, 3, "[O]", COLOR_DIALOG_CHECKBOX);
+            else if (di->checked==1) 
+                print_text(di->x, di->y, 3, "[X]", COLOR_DIALOG_CHECKBOX);
+            else 
+                print_text(di->x, di->y, 3, "[ ]", COLOR_DIALOG_CHECKBOX);
+            if (sel) {
+                set_cursor(di->x + 1, di->y, di->x + 1, di->y);
+                set_window_ptr(dlg->win, di->x, di->y);
+            }
+            break;
         case D_FIELD_PASS:
         case D_FIELD:
             if (di->vpos + di->l <= di->cpos) di->vpos = di->cpos - di->l + 1;
@@ -502,7 +514,7 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
     }
 }
 
-void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
+static void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     if (di->item->type == D_CHECKBOX) {
         if (!di->item->gid) di -> checked = *(int *)di->cdata = !*(int *)di->cdata;
@@ -519,10 +531,28 @@ void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
         }
         display_dlg_item(dlg, di, 1);
     }
+    else if (di->item->type == D_CHECKBOX3) {
+        if (!di->item->gid) {
+            (*(int *)di->cdata)++;
+            if (*(int *)di->cdata == 3) *(int *)di->cdata = 0;
+            di -> checked = *(int *)di->cdata;
+        }else{
+            int i;
+            for (i = 0; i < dlg->n; i++) {
+                if (dlg->items[i].item->type == D_CHECKBOX3 && dlg->items[i].item->gid == di->item->gid) {
+                    *(int *)dlg->items[i].cdata = di->item->gnum;
+                    dlg->items[i].checked = 0;
+                    display_dlg_item(dlg, &dlg->items[i], 0);
+                }
+            }
+            di->checked = 1;
+        }
+        display_dlg_item(dlg, di, 1);
+    }
     else if (di->item->type == D_BUTTON) di->item->fn(dlg, di);
 }
 
-void dlg_set_history(struct dialog_item_data *di)
+static void dlg_set_history(struct dialog_item_data *di)
 {
     char *s = "";
     int l;
@@ -533,7 +563,7 @@ void dlg_set_history(struct dialog_item_data *di)
     di->cpos = l;
 }
 
-int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event *ev)
+static int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event *ev)
 {
     switch (di->item->type) {
         case D_BUTTON:
@@ -552,6 +582,7 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
             display_dlg_item(dlg, di, 1);
             return 1;
         case D_CHECKBOX:
+        case D_CHECKBOX3:
             if (ev->y != di->y || ev->x < di->x || ev->x >= di->x + 3) return 0;
             display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
             dlg->selected = di - dlg->items;
@@ -575,7 +606,7 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
     return 0;
 }
 
-void redraw_dialog(struct dialog_data *dlg)
+static void redraw_dialog(struct dialog_data *dlg)
 {
     int i;
     draw_frame(dlg->x + DIALOG_LEFT_BORDER, dlg->y + DIALOG_TOP_BORDER, dlg->xw - 2 * DIALOG_LEFT_BORDER, dlg->yw - 2 * DIALOG_TOP_BORDER, COLOR_DIALOG_FRAME, DIALOG_FRAME);
@@ -591,7 +622,7 @@ void redraw_dialog(struct dialog_data *dlg)
 void dialog_func(struct window *win, struct event *ev, int fwd)
 {
     int i;
-    struct dialog_data *dlg = win->data;
+    struct dialog_data *dlg = (struct dialog_data *)win->data;
     struct dialog_item_data *di;
 
     dlg->win = win;
@@ -607,13 +638,16 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                 struct dialog_item_data *di = &dlg->items[i];
                 memset(di, 0, sizeof(struct dialog_item_data));
                 di->item = &dlg->dlg->items[i];
-                if ((di->cdata = mem_alloc(di->item->dlen)))
+                if ((di->cdata = (char *)mem_alloc(di->item->dlen)))
                     memcpy(di->cdata, di->item->data, di->item->dlen);
                 if (di->item->type == D_CHECKBOX) {
                     if (di->item->gid) {
                         if (*(int *)di->cdata == di->item->gnum) di->checked = 1;
                     } else if (*(int *)di->cdata) di->checked = 1;
                 } 
+                if (di->item->type == D_CHECKBOX3) {
+                    di->checked = *(int *)di->cdata;
+                } 
                 if (di->item->type == D_BOX) {
                     /* Freed in bookmark_dialog_abort_handler() */
                     di->cdata = mem_alloc( sizeof(struct dlg_data_item_data_box) );
@@ -624,7 +658,6 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                     init_list(((struct dlg_data_item_data_box*)di->cdata)->items);
                 }
                 init_list(di->history);
-                di->cur_hist = (struct history_item *)&di->history;
                 if (di->item->type == D_FIELD || di->item->type == D_FIELD_PASS) {
                     if (di->item->history) {
                         struct history_item *j;
@@ -652,6 +685,15 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
         case EV_KBD:
             di = &dlg->items[dlg->selected];
             if (di->item->type == D_FIELD || di->item->type == D_FIELD_PASS) {
+                
+                if (di->item->tabcompl && ev->x == KBD_F2 && ev->y == 0){
+                    di->item->tabi++;
+                    do_tab_compl(di, di->item->tabi);
+                    if (di->item->tabi == 2) di->item->tabi = 0;
+                    goto dsp_f;
+                }else{
+                    di->item->tabi = 0;
+                }
                 switch (kbd_action(KM_EDIT, ev)) {
                     case ACT_UP:
                         if ((void *)di->cur_hist->prev != &di->history) {
@@ -794,6 +836,21 @@ int check_number(struct dialog_data *dlg, struct dialog_item_data *di)
     return 0;
 }
 
+int check_qrg(struct dialog_data *dlg, struct dialog_item_data *di)
+{
+    char *end;
+    double d = strtod(di->cdata, (char **)(void *)&end);
+    if (!*di->cdata || *end) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_EXPECTED), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    if (d < 0.0 || d > 300000000000.0) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_OUT_OF_RANGE), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    return 0;
+}
+
 int check_nonempty(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     char *p;
@@ -810,11 +867,12 @@ int cancel_dialog(struct dialog_data *dlg, struct dialog_item_data *di)
     return 0;
 }
 
-int check_dialog(struct dialog_data *dlg)
+static int check_dialog(struct dialog_data *dlg)
 {
     int i;
     for (i = 0; i < dlg->n; i++)
-        if (dlg->dlg->items[i].type == D_CHECKBOX || dlg->dlg->items[i].type == D_FIELD || dlg->dlg->items[i].type == D_FIELD_PASS)
+        if (dlg->dlg->items[i].type == D_CHECKBOX || dlg->dlg->items[i].type == D_CHECKBOX3 || 
+            dlg->dlg->items[i].type == D_FIELD || dlg->dlg->items[i].type == D_FIELD_PASS)
             if (dlg->dlg->items[i].fn && dlg->dlg->items[i].fn(dlg, &dlg->items[i])) {
                 dlg->selected = i;
                 redraw_dialog(dlg);
@@ -1027,7 +1085,8 @@ void max_group_width(struct terminal *term, char **texts, struct dialog_item_dat
 {
     int ww = 0;
     while (n--) {
-        int wx = item->item->type == D_CHECKBOX ? 4 : item->item->type == D_BUTTON ? strlen(_(item->item->text)) + 5 : item->item->dlen + 1;
+        int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
+            item->item->type == D_BUTTON ?  strlen(_(item->item->text)) + 5 : item->item->dlen + 1;
         wx += strlen(_(texts[0]));
         if (n) wx++;
         ww += wx;
@@ -1040,7 +1099,8 @@ void max_group_width(struct terminal *term, char **texts, struct dialog_item_dat
 void min_group_width(struct terminal *term, char **texts, struct dialog_item_data *item, int n, int *w)
 {
     while (n--) {
-        int wx = item->item->type == D_CHECKBOX ? 4 : item->item->type == D_BUTTON ? strlen(_(item->item->text)) + 5 : item->item->dlen + 1;
+        int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
+            item->item->type == D_BUTTON ? strlen(_(item->item->text)) + 5 : item->item->dlen + 1;
         wx += strlen(_(texts[0]));
         if (wx > *w) *w = wx;
         texts++;
@@ -1054,10 +1114,11 @@ void dlg_format_group(struct terminal *term1, struct terminal *t2,
 {
     int nx = 0; /* now X coord */
 /*    dbg("\ndlg_format_group (term=%p,n=%d,x=%d,*y=%d,w=%d,rw=%p)\n",term,n,x,*y,w,rw);*/
+    //dbg("dlg_format_group item x=%d y=%d l=%d, vpos=%d, cpos=%d\n", item->x, item->y, item->l, item->vpos, item->cpos);
         
     while (n--) {
         int sl;
-        int wx = item->item->type == D_CHECKBOX ? 4 : 
+        int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
             item->item->type == D_BUTTON ? 
             strlen(_(item->item->text)) + 5 : 
               item->item->maxl ? item->item->maxl+1 : item->item->dlen + 1;        /* widget width */
@@ -1071,8 +1132,8 @@ void dlg_format_group(struct terminal *term1, struct terminal *t2,
 /*        dbg("sl=%d nx=%d wx=%d w=%d\n",sl,nx,wx,w);*/
         if (term1) {
             int stringlen;
-            print_text(x + nx + 4 * (item->item->type == D_CHECKBOX), *y, strlen(_(texts[0])), _(texts[0]), COLOR_DIALOG_TEXT);
-            item->x = x + nx + (sl + 1) * (item->item->type != D_CHECKBOX);
+            print_text(x + nx + 4 * (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3), *y, strlen(_(texts[0])), _(texts[0]), COLOR_DIALOG_TEXT);
+            item->x = x + nx + (sl + 1) * (item->item->type != D_CHECKBOX && item->item->type != D_CHECKBOX3);
             item->y = *y;
 /*            dbg("item->x=%d  item->y=%d\n",item->x,item->y);*/
             if (item->item->type == D_FIELD || item->item->type == D_FIELD_PASS) {
@@ -1102,7 +1163,7 @@ void dlg_format_group1(struct terminal *term1, struct terminal *t2,
         
     while (n--) {
         int sl;
-        int wx = item->item->type == D_CHECKBOX ? 4 : 
+        int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
             item->item->type == D_BUTTON ? 
             strlen(_(item->item->text)) + 5 : 
               item->item->maxl ? item->item->maxl+1 : item->item->dlen + 1;        /* widget width */
@@ -1116,8 +1177,8 @@ void dlg_format_group1(struct terminal *term1, struct terminal *t2,
 /*        dbg("sl=%d nx=%d wx=%d w=%d\n",sl,nx,wx,w);*/
         if (term1) {
             int stringlen;
-            print_text(x + nx + 4 * (item->item->type == D_CHECKBOX), *y, strlen(_(texts[0])), _(texts[0]), COLOR_DIALOG_TEXT);
-            item->x = x + nx + (sl + 1) * (item->item->type != D_CHECKBOX);
+            print_text(x + nx + 4 * (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3), *y, strlen(_(texts[0])), _(texts[0]), COLOR_DIALOG_TEXT);
+            item->x = x + nx + (sl + 1) * (item->item->type != D_CHECKBOX && item->item->type != D_CHECKBOX3);
             item->y = *y;
 /*            dbg("item->x=%d  item->y=%d\n",item->x,item->y);*/
             if (item->item->type == D_FIELD || item->item->type == D_FIELD_PASS) {
@@ -1196,7 +1257,7 @@ void group_fn(struct dialog_data *dlg)
     dlg_format_buttons(term, term, dlg->items + dlg->n - 2, 2, dlg->x + DIALOG_LB, &y, w, &rw, AL_CENTER);
 }
 
-void msg_box_fn(struct dialog_data *dlg)
+static void msg_box_fn(struct dialog_data *dlg)
 {
     int max = 0, min = 0;
     int w, rw;
@@ -1230,7 +1291,7 @@ void msg_box_fn(struct dialog_data *dlg)
     mem_free(text);
 }
 
-int msg_box_button(struct dialog_data *dlg, struct dialog_item_data *di)
+static int msg_box_button(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     void (*fn)(void *) = (void (*)(void *))di->item->udata;
     void *data = dlg->dlg->udata2;
@@ -1324,7 +1385,7 @@ void add_to_history(struct history *h, char *t)
     while (h->n > MAX_HISTORY_ITEMS) {
         struct history_item *hd = h->items.prev;
         if ((void *)hd == &h->items) {
-            internal("history is empty");
+            internal_("history is empty");
             h->n = 0;
             return;
         }
@@ -1334,7 +1395,7 @@ void add_to_history(struct history *h, char *t)
     }
 }
 
-int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
+static int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     void (*fn)(void *) = di->item->udata;
     void *data = dlg->dlg->udata2;
@@ -1343,7 +1404,7 @@ int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
     return 0;
 }
 
-int input_field_ok(struct dialog_data *dlg, struct dialog_item_data *di)
+static int input_field_ok(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     void (*fn)(void *, char *) = di->item->udata;
     void *data = dlg->dlg->udata2;
@@ -1355,40 +1416,44 @@ int input_field_ok(struct dialog_data *dlg, struct dialog_item_data *di)
     return 0;
 }
 
-void input_field_fn(struct dialog_data *dlg)
-{
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
-    max_text_width(term, dlg->dlg->udata, &max);
-    min_text_width(term, dlg->dlg->udata, &min);
-    max_buttons_width(term, dlg->items + 1, 2, &max);
-    min_buttons_width(term, dlg->items + 1, 2, &min);
-    if (max < dlg->dlg->items->dlen) max = dlg->dlg->items->dlen;
-    w = term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    rw = 0; /* !!! FIXME: input field */
-    dlg_format_text(NULL, term, dlg->dlg->udata, 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
-    dlg_format_field(NULL, term, dlg->items, 0, &y, w, &rw, AL_LEFT);
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items + 1, 2, 0, &y, w, &rw, AL_CENTER);
-    w = rw;
-    dlg->xw = rw + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    dlg_format_text(term, term, dlg->dlg->udata, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-    dlg_format_field(term, term, dlg->items, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items + 1, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
+static int input_field_complete(struct dialog_data *dlg, struct dialog_item_data *di){
+    di->item->tabi++;
+    do_tab_compl(&dlg->items[0], 1);
+    if (di->item->tabi == 2) di->item->tabi = 0;
+    redraw_later();
+    return 0;
 }
 
-void input_field(struct memory_list *ml, char *title, char *text, char *okbutton, char *cancelbutton, void *data, struct history *history, int l, char *def, int min, int max, int (*check)(struct dialog_data *, struct dialog_item_data *), void (*fn)(void *, char *), void (*cancelfn)(void *))
+char new_file[MAX_STR_LEN];
+
+void input_field_fn(struct dialog_data *dlg)
+{
+    if (strlen(new_file)){
+        char *c = strrchr(dlg->items[0].cdata, '/');
+        if (c){
+			char *p;
+            *c = '\0';
+            p = g_strconcat(dlg->items[0].cdata, "/", new_file, NULL);
+            safe_strncpy0(dlg->items[0].cdata, p, dlg->items[0].item->dlen);
+            g_free(p);
+        }else{
+            safe_strncpy0(dlg->items[0].cdata, new_file, dlg->items[0].item->dlen);
+        }
+        dlg->items[0].cpos = strlen(dlg->items[0].cdata);
+//        safe_strncpy0(dlg->items[0].cdata, new_file, EQSO_LEN);
+        strcpy(new_file, "");
+    }
+}
+
+
+void input_field(struct memory_list *ml, char *title, char *text, char *okbutton, char *cancelbutton, void *data, struct history *history, int l, char *def, int min, int max, int (*check)(struct dialog_data *, struct dialog_item_data *), void (*fn)(void *, char *), void (*cancelfn)(void *), int tabcompl)
 {                                                                                                                                   
     struct dialog *dlg;
     char *field;
+    int i;
+
+    strcpy(new_file, "");
+
     if (!(dlg = mem_alloc(sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + l)))
         return;
     memset(dlg, 0, sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + l);
@@ -1400,29 +1465,50 @@ void input_field(struct memory_list *ml, char *title, char *text, char *okbutton
             strcpy(field, def);
     }
     dlg->title = title;
-    dlg->fn = input_field_fn;
+    dlg->fn = dlg_pf_fn;
+    if (tabcompl) dlg->fn2 = input_field_fn; 
     dlg->udata = text;
     dlg->udata2 = data;
-    dlg->items[0].type = D_FIELD;
-    dlg->items[0].gid = min;
-    dlg->items[0].gnum = max;
-    dlg->items[0].fn = check;
-    dlg->items[0].history = history;
-    dlg->items[0].dlen = l;
-    dlg->items[0].data = field;
-    dlg->items[1].type = D_BUTTON;
-    dlg->items[1].gid = B_ENTER;
-    dlg->items[1].fn = input_field_ok;
-    dlg->items[1].dlen = 0;
-    dlg->items[1].text = okbutton;
-    dlg->items[1].udata = fn;
-    dlg->items[2].type = D_BUTTON;
-    dlg->items[2].gid = B_ESC;
-    dlg->items[2].fn = input_field_cancel;
-    dlg->items[2].dlen = 0;
-    dlg->items[2].text = cancelbutton;
-    dlg->items[2].udata = cancelfn;
-    dlg->items[3].type = D_END;
+    dlg->y0 = 1;
+
+    dlg->items[i=0].type = D_FIELD;
+    dlg->items[i].gid = min;
+    dlg->items[i].gnum = max;
+    dlg->items[i].fn = check;
+    dlg->items[i].history = history;
+    dlg->items[i].dlen = l;
+    dlg->items[i].maxl = (term->x * 2)/4;
+    dlg->items[i].data = field;
+    dlg->items[i].tabcompl = tabcompl;
+    dlg->items[i].wrap = 0;
+
+    if (tabcompl){
+        dlg->items[++i].type = D_BUTTON;
+        dlg->items[i].gid = 0;
+        dlg->items[i].fn = input_field_complete;
+        dlg->items[i].dlen = 0;
+        dlg->items[i].text = "F2";
+        dlg->items[i].udata = NULL;
+    }
+    dlg->items[i].wrap = 2;
+
+    dlg->items[++i].type = D_BUTTON;
+    dlg->items[i].gid = B_ENTER;
+    dlg->items[i].fn = input_field_ok;
+    dlg->items[i].dlen = 0;
+    dlg->items[i].text = okbutton;
+    dlg->items[i].udata = fn;
+
+    dlg->items[++i].type = D_BUTTON;
+    dlg->items[i].gid = B_ESC;
+    dlg->items[i].fn = input_field_cancel;
+    dlg->items[i].dlen = 0;
+    dlg->items[i].text = cancelbutton;
+    dlg->items[i].udata = cancelfn;
+    dlg->items[i].align = AL_BUTTONS;
+    dlg->items[i].wrap = 1;
+
+    dlg->items[++i].type = D_END;
     add_to_ml(&ml, dlg, NULL);
     do_dialog(dlg, ml);
 }
@@ -1538,4 +1624,240 @@ void errbox(char *text, int errcode, ...){
     
 }
     
+void dlg_pf_init(int phase, struct dialog_data *dlgd){
+    switch(phase){
+        case 0:
+            dlgd->min = 0;
+            dlgd->max = 0;
+            dlgd->yy = -1;
+            dlgd->ofs = 0;
+            break;
+        case 1:
+            dlgd->w = dlgd->win->term->x * 9 / 10 - 2 * DIALOG_LB;
+            if (dlgd->w > dlgd->max) dlgd->w = dlgd->max;
+            if (dlgd->w < dlgd->min) dlgd->w = dlgd->min;
+            if (dlgd->w > dlgd->win->term->x - 2 * DIALOG_LB) dlgd->w = dlgd->win->term->x - 2 * DIALOG_LB;
+            if (dlgd->w < 1) dlgd->w = 1;
+            dlgd->rw = 0;
+            dlgd->yy = -1;
+            dlgd->ofs = 0;
+            break;
+        case 2:
+            dlgd->w = dlgd->rw;
+            dlgd->xw = dlgd->w + 2 * DIALOG_LB;
+            dlgd->yw = dlgd->yy + 2 * DIALOG_TB;
+            center_dlg(dlgd);
+            draw_dlg(dlgd);
+            dlgd->yy = dlgd->y + DIALOG_TB + dlgd->dlg->y0;
+            dlgd->ofs = 0;
+            break;
+    }
     
+}
+    
+void dlg_pf_group(int phase, struct dialog_data *dlgd, int cnt){
+    int i;
+    int nx = 0; /* now X coord */
+    
+    switch(phase){
+        case 0:
+            for (i=0; i<cnt; i++){
+                int wx = (dlgd->items[dlgd->ofs+i].item->type == D_CHECKBOX || dlgd->items[dlgd->ofs+i].item->type == D_CHECKBOX3) ? 4 : 
+                    dlgd->items[dlgd->ofs+i].item->type == D_BUTTON ? strlen(_(dlgd->items[dlgd->ofs+i].item->text)) + 5 : dlgd->items[dlgd->ofs+i].item->dlen + 1;
+                if (dlgd->items[dlgd->ofs+i].item->msg){ 
+                    wx += strlen(_(dlgd->items[dlgd->ofs+i].item->msg));
+                }
+                if (wx > dlgd->max) dlgd->max = wx;
+                if (wx < dlgd->min) dlgd->min = wx;
+            }
+            break;
+        case 1:
+            for (i=0; i<cnt; i++){
+                int sl;
+                struct dialog_item_data *item = &(dlgd->items[dlgd->ofs+i]);
+                int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
+                    item->item->type == D_BUTTON ? 
+                    strlen(_(item->item->text)) + 5 : 
+                      item->item->maxl ? item->item->maxl+1 : item->item->dlen + 1;        /* widget width */
+                if (_(item->item->msg)) sl = strlen(_(item->item->msg));
+                else sl = -1;
+                wx += sl;
+                if (nx && nx + wx > dlgd->w) {
+                    nx = 0;
+                    dlgd->yy += 1;
+                }
+                if (nx + wx > dlgd->rw) if ((dlgd->rw = nx + wx) > dlgd->w) dlgd->rw = dlgd->w;
+                nx += wx + 1;
+            }
+            //dlgd->yy++;
+            break;
+        case 2:
+            for (i=0; i<cnt; i++){
+                int sl;
+                int stringlen;
+                struct dialog_item_data *item = &(dlgd->items[dlgd->ofs+i]);
+                int wx = (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3) ? 4 : 
+                    item->item->type == D_BUTTON ? 
+                    strlen(_(item->item->text)) + 5 : 
+                      item->item->maxl ? item->item->maxl+1 : item->item->dlen + 1;        /* widget width */
+                if (_(item->item->msg)) sl = strlen(_(item->item->msg));
+                else sl = -1;
+                wx += sl;
+                if (nx && nx + wx > dlgd->w) {
+                    nx = 0;
+                    dlgd->yy += 1;
+                }
+                if (item->item->msg){
+                    print_text(dlgd->x + DIALOG_LB + nx + 4 * (item->item->type == D_CHECKBOX || item->item->type == D_CHECKBOX3), dlgd->yy, strlen(_(item->item->msg)), _(item->item->msg), COLOR_DIALOG_TEXT);
+                }
+                item->x = dlgd->x + DIALOG_LB + nx + (sl + 1) * (item->item->type != D_CHECKBOX && item->item->type != D_CHECKBOX3);
+                item->y = dlgd->yy;
+                if (item->item->type == D_FIELD || item->item->type == D_FIELD_PASS) {
+                    item->l = item->item->maxl ? item->item->maxl : item->item->dlen;
+                    stringlen=0;
+                    if (item->item->msg) stringlen = strlen(_(item->item->msg));    
+                    if (item->l > dlgd->w - nx - stringlen) item->l = dlgd->w - nx - stringlen;
+                }
+                nx += wx + 1;
+            }
+            //dlgd->yy++;
+            break;
+    }
+    dlgd->ofs += cnt;
+}
+
+void dlg_pf_buttons(int phase, struct dialog_data *dlgd, int cnt){
+    switch(phase){
+        case 0:
+            max_buttons_width(dlgd->win->term, dlgd->items + dlgd->ofs, cnt, &dlgd->max);
+            min_buttons_width(dlgd->win->term, dlgd->items + dlgd->ofs, cnt, &dlgd->min);
+            break;
+        case 1:
+            dlg_format_buttons(NULL, dlgd->win->term, dlgd->items + dlgd->ofs, cnt, 
+                    0, &dlgd->yy, dlgd->w, &dlgd->rw, 
+                    dlgd->items[dlgd->ofs+cnt-1].item->align);
+            break;
+        case 2:
+            dlg_format_buttons(dlgd->win->term, dlgd->win->term, dlgd->items + dlgd->ofs, cnt, 
+                    dlgd->x + DIALOG_LB, &dlgd->yy, dlgd->w, NULL, 
+                    dlgd->items[dlgd->ofs+cnt-1].item->align);
+            break;
+    }
+    dlgd->ofs += cnt;
+}
+
+void dlg_pf_fn(struct dialog_data *dlgd)
+{
+    int ph;
+    int i,j;
+    
+    if (dlgd->dlg->fn2) dlgd->dlg->fn2(dlgd);
+    
+    j=0;
+    for (ph = 0; ph < 3; ph++){
+        dlg_pf_init(ph, dlgd);
+        for (i=0; i< dlgd->n; i++){
+//            char *names[]={"end", "check", "field", "pass", "butt", "box", "check3", "?", "?"};
+//            dbg("i=%d type=%d %s n=%d\n", i, dlgd->items[i].item->type, names[dlgd->items[i].item->type], dlgd->n);
+            j++;
+            if (!dlgd->items[i].item->wrap) continue;
+            if (dlgd->items[i].item->align & AL_BUTTONS)
+                dlg_pf_buttons(ph, dlgd, j);
+            else
+                dlg_pf_group(ph, dlgd, j);
+            dlgd->yy += dlgd->items[i].item->wrap;
+            j=0;
+        }
+        if (j>0){
+            if (dlgd->items[i].item->align & AL_BUTTONS)
+                dlg_pf_buttons(ph, dlgd, j);
+            else
+                dlg_pf_group(ph, dlgd, j);
+            dlgd->yy += dlgd->items[i].item->wrap;
+        }
+    }
+}
+
+static char *complete_file;
+static int filter_complete(const struct dirent *de){
+    int ret;
+
+    if (strcmp((char *)de->d_name, ".") == 0) return 0;
+    if (strcmp((char *)de->d_name, "..") == 0) return 0;
+    ret = strncmp((char *)de->d_name, complete_file, strlen(complete_file));
+    return !ret;
+}
+
+void complete_choose(cba_t dir, cba_t unused){
+    /*dbg("menu_load_contest('%s')\n", dir);*/
+    safe_strncpy0(new_file, (char *)GETCBA(dir, charp), MAX_STR_LEN);
+  
+    redraw_later();
+} 
+
+
+int do_tab_compl(struct dialog_item_data *di, int tabi){
+    char *path, *file, *slash;
+    int i, n, max = 0;
+    struct dirent **namelist=NULL;
+    struct menu_item *mi = NULL;
+
+	dbg("complete: %d '%s'\n", tabi, di->cdata);
+
+    slash = strrchr(di->cdata, '/');
+    if (slash){
+        if (di->cdata[0] == '~'){
+            char *p = g_strndup(di->cdata + 1 , slash - di->cdata - 1);
+            path = g_strconcat(getenv("HOME"), "/", p, NULL);
+            g_free(p);
+        }else{
+            path = g_strndup(di->cdata, slash - di->cdata);
+        }
+        file = g_strdup(slash + 1);
+    }else{
+        path = g_strdup(".");
+        file = g_strdup(di->cdata);
+    }
+    dbg("path='%s' file='%s'\n", path, file);
+
+    free_namelist();
+
+    complete_file = file;
+    n = tucnak_scandir(path, &namelist, filter_complete, tucnak_alphasort);    
+    complete_file = NULL;
+    dbg("n=%d\n", n);
+    if (n <= 0) goto x;
+
+    for (i=0; i<n; i++){
+        dbg("%s ", namelist[i]->d_name);
+    }
+    dbg("\n");
+
+    if (n == 1){
+        char *p = g_strconcat(path, "/", (char*)namelist[0]->d_name, NULL);
+        safe_strncpy0(di->cdata, p, di->item->dlen);
+        di->cpos = strlen(di->cdata);
+        g_free(p);
+    }else{
+        if (!(mi = new_menu(3))) goto x;
+        for (i=0; i<n; i++){
+            cba_t cba2;
+            char *c = (char*)namelist[i]->d_name;
+            if (strlen(c)>max) max=strlen(c);
+            SETCBA(cba2, charp, namelist[i]->d_name);
+            add_to_menu(&mi, 
+                    stracpy(c), 
+                    "", "", 
+                    complete_choose, cba2, 0);    
+        }
+//        set_window_ptr(gses->win, (term->x-6-max)/2,(term->y-2-i)/2);
+
+        do_menu(mi, CBA0);
+    }
+
+x:;
+    g_free(path);
+    g_free(file);
+    free_namelist();
+    return 0;
+}
