@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -247,10 +247,10 @@ static struct conv_table *get_translation_table_to_utf_8(int from)
 }
 #endif
 
-struct conv_table table[256];
+static struct conv_table table[256];
 static int first = 1;
 
-void free_conv_table()
+void free_conv_table(void)
 {
     if (!utf_table_init) free_utf_table();
     if (first) memset(table, 0, sizeof(struct conv_table) * 256), first = 0;
