@@ -2,18 +2,18 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-char *strstates[]={"RUN","SP","REQR","REQS","RUNING","GIVEN"};
-char *strccmds[]={"REQ","ACC","REJ","BACK"};
+static char *strstates[]={"RUN","SP","REQR","REQS","RUNING","GIVEN"};
+static char *strccmds[]={"REQ","ACC","REJ","BACK"};
 
-void menu_forcerun(union cba_t cba){
+void menu_forcerun(cba_t cba){
     gchar *c;
     struct band *b;
     if (!ctest || !aband) return;
@@ -39,7 +39,7 @@ int can_tx(struct band *b){
 }
 
 
-void menu_grabband(union cba_t cba){
+void menu_grabband(cba_t cba){
     gchar *c;
     struct band *b;
     int ctx1,ctx2;
@@ -187,29 +187,57 @@ void peer_tx(struct band *band, int tx){
     g_free(c);
 }
 
-void menu_spy(union cba_t cba){
+/*void menu_spy(cba_t cba){
     if (!ctest || !aband) return;
 
     aband->spymode++;
     if (aband->spymode>=SM_LAST) aband->spymode=SM_INPUTLINE;
     redraw_later();
-}
+} */
 
-void menu_endspy(union cba_t cba){
+void do_endspy(cba_t cba, cba_t cba1){
+    struct spypeer *sp;
     int i;
+    char *filename;
     
-    if (!aband) return;
-    
-    for (i=aband->spypeers->len-1;i>=0;i--){
-        struct spypeer *sp;
-        sp=(struct spypeer *)g_ptr_array_index(aband->spypeers, i);
-        free_spypeer(sp);
-        g_ptr_array_remove(aband->spypeers, sp);
-    }
-    resize_terminal(CBA0);
+    i=(int)GETCBA(cba, int_);
+    if (i<0 || i>=ctest->spypeers->len) return;
 
+    sp=(struct spypeer *)g_ptr_array_index(ctest->spypeers, i);
+    free_spypeer(sp);
+    g_ptr_array_remove(ctest->spypeers, sp);
+    
+    filename = g_strdup_printf("%s/spy", ctest->directory);
+    save_spypeers_to_file(filename); 
+    g_free(filename);
+    
+    resize_terminal(CBA0);
 }
 
+void menu_endspy(cba_t cba){
+    int i;
+    struct menu_item *mi = NULL;
+    struct spypeer *sp;
+    
+    if (!ctest) return;
+    if (!ctest->spypeers->len) return;
+    
+    if (!(mi = new_menu(3))) return;
+    
+    for (i=0; i<ctest->spypeers->len;i++){
+        cba_t cba2;
+        gchar *c;
+
+        SETCBA(cba2, int_, i);
+        sp=(struct spypeer *)g_ptr_array_index(ctest->spypeers, i);
+        c=g_strdup_printf("%-6s %c   %s", sp->operator_, sp->bandchar, sp->peerid);
+        add_to_menu(&mi,stracpy(c),"", "", do_endspy, cba2, 0);    
+        g_free(c);
+    }
+    do_menu_selected(mi, CBA0, 0);
+}
+
+
 void send_inputline(struct band *band){
     gchar *c;
     
@@ -226,7 +254,7 @@ void send_inputline(struct band *band){
 void free_spypeer(struct spypeer *sp){
     clear_spypeer(sp);
     CONDGFREE(sp->peerid);
-    CONDGFREE(sp->operator);
+    CONDGFREE(sp->operator_);
     CONDGFREE(sp->inputline);
     g_free(sp);
 }
@@ -244,10 +272,11 @@ void free_spypeers(GPtrArray *sps){
     g_ptr_array_free(sps,1);
 }
 
-struct spypeer *init_spypeer(GPtrArray *sps, gchar *peerid){
+ struct spypeer *init_spypeer(GPtrArray *sps, gchar *peerid, char bandchar){
     struct spypeer *sp;
+    char *filename;
 
-    sp=get_spypeer_by_peerid(sps, peerid);
+    sp=get_spypeer_by_peerid(sps, peerid, bandchar);
     if (sp){
         g_ptr_array_remove(sps, sp);
         free_spypeer(sp);
@@ -255,19 +284,28 @@ struct spypeer *init_spypeer(GPtrArray *sps, gchar *peerid){
 
     sp=g_new0(struct spypeer, 1);
     sp->peerid=g_strdup(peerid);
+    sp->bandchar=bandchar;
     sp->peertx=-1;
     g_ptr_array_add(sps, sp);
+    
+    filename = g_strdup_printf("%s/spy", ctest->directory);
+    save_spypeers_to_file(filename); 
+    g_free(filename);
+    
     resize_terminal(CBA0);
     return sp;
 }
 
-struct spypeer *get_spypeer_by_peerid(GPtrArray *sps, gchar *peerid){
+struct spypeer *get_spypeer_by_peerid(GPtrArray *sps, gchar *peerid, char bandchar){
     int i;
 
+    bandchar=toupper(bandchar);
+
     for (i=0;i<sps->len;i++){
         struct spypeer *sp;
         sp=(struct spypeer *)g_ptr_array_index(sps, i);
         if (strcmp(sp->peerid, peerid)) continue;
+        if (sp->bandchar != bandchar) continue;
         return sp;
     }
     return NULL;
@@ -284,27 +322,106 @@ void clear_spypeer(struct spypeer *sp){
     CONDGFREE(sp->remark);
 }
 
-void send_spypeer_request(int no){
-    char *peerid,*c;
+void send_spypeer_request(cba_t cba){
+    char *peerid;
     gchar **items;
-    int i;
+    char bandchar;
+    int i, no;
     
-    log_addf("add_spy '%d'\n", no);
+    
+    
+    no=(int)GETCBA(cba, int_);
+    dbg("send_spypeer_request no='%d'\n", no);
+    dbg("bpeers='%s'\n", net->bpeers);
+    
+    items = g_strsplit(net->bpeers,";",0);
 
-    items = g_strsplit(net->allpeers,";",0);
-
-    for (i=0; items[i]!=NULL;i++){
+    /*for (i=0; items[i]!=NULL;i++){
         if (i==no) goto found;
+    } */
+    for (i=0; 
+         items[i]!=NULL && items[i+1]!=NULL && items[i+2]!=NULL;
+         i+=3){
+        dbg("i=%d\n", i);
+        if (i==no*3) goto found;
     }
+    dbg("not found\n");
     goto x;
 found:    
+    dbg("found i=%d, no=%d\n", i, no);
     peerid=items[i];
-    log_addf("send_spypeer_request: peerid=%s", peerid);
-    init_spypeer(aband->spypeers, peerid);
-
-    c=g_strdup_printf("RT %s;%s\n",aband->pband, peerid);
-    rel_write_all(c);
-    g_free(c);
+    bandchar=items[i+2][0];
+    spypeer_add(peerid, bandchar);
 x:;    
     g_strfreev(items);
 }
+
+
+void spypeer_add(char *peerid, char bandchar){
+    struct band *b;
+    gchar *c;
+    
+    b=find_band_by_bandchar(bandchar);
+    if (!b){
+        dbg("band %c not found\n", bandchar);
+        return;
+    }
+        
+    dbg("spypeer_add: peerid=%s bandchar=%c pband=%s\n", peerid, bandchar, b->pband);
+    init_spypeer(ctest->spypeers, peerid, bandchar);
+
+    c=g_strdup_printf("RT %s;%s\n",b->pband, peerid);
+    rel_write_all(c);
+    g_free(c);
+}
+
+
+int load_spypeers_from_file(char *filename){
+    FILE *f;
+    gchar *c;
+    GString *gs;
+    gchar **items, *peerid;
+    char bandchar;
+    
+    f=fopen(filename,"rt");
+    if (!f) return 1;
+
+    gs = g_string_sized_new(100);
+    while( (c=safe_fgets(gs, f, 0)) != NULL){
+        items = g_strsplit(c, ";", 3);
+        if (!items) continue;
+        if (!items[1] || !items[2]) {
+            g_strfreev(items);
+            continue;
+        }
+        peerid=items[0];
+        bandchar=items[1][0];
+
+        spypeer_add(peerid, bandchar);
+    }
+    g_string_free(gs, 1);
+    fclose(f);
+    return 0;
+}
+
+int save_spypeers_to_file(char *filename){
+    FILE *f;
+    int i;
+
+    f = fopen(filename, "wt");
+    if (!f){
+        log_addf(VTEXT(T_CANT_WRITE_S), filename);
+        return errno;
+    }
+    
+    for (i=0;i<ctest->spypeers->len; i++){
+        struct spypeer *sp;
+
+        sp = (struct spypeer *)g_ptr_array_index(ctest->spypeers, i);
+        fprintf(f, "%s;%c;\n", sp->peerid, sp->bandchar);
+    }
+
+    fclose(f);
+    return 0;
+}
+
