@@ -2,21 +2,20 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
-
 #ifdef HAVE_SDL
 
-#include "cor.inc"
 
 struct cor *cor = NULL;
+static struct cor *cor2 = NULL;
 
-struct kmarray *kmarray_new(){
+struct kmarray *kmarray_new(void){
     struct kmarray *kma;
 
     kma = g_new0(struct kmarray, 1);
@@ -56,28 +55,67 @@ struct cor *init_cor(){
 	cor->km = g_hash_table_new(g_direct_hash, g_direct_equal);
 	cor->wwl4 = g_hash_table_new(g_direct_hash, g_direct_equal);
 	cor->wwl2 = g_hash_table_new(g_direct_hash, g_direct_equal);
+    cor->thread_break = 0;
+	cor->file=cor_tucnakcor;
+	cor->items=COR_ITEMS;
     return cor;
 }
 
+void free_cor(struct cor *cor){
 
-int recalc_cor(struct subwin *sw, gchar *locator){ 
-    int i,j;
+	if (!cor) return;
+
+    if (cor2 && cor2->thread){
+        cor2->thread_break = 1;
+        //dbg("join cor...\n");
+        g_thread_join(cor2->thread);
+        //dbg("done\n");
+        cor2->thread = NULL;
+    }
+	g_hash_table_foreach_remove(cor->km, free_km_item, NULL); 
+	g_hash_table_foreach_remove(cor->wwl4, free_km_item, NULL); 
+	g_hash_table_foreach_remove(cor->wwl2, free_km_item, NULL); 
+	
+    g_free(cor);
+}
+
+int cor_recalc(struct subwin *sw, gchar *locator){ 
+   // dbg("cor_recalc('%s', '%s') %d %d\n", sw->title, locator, sw->myh, sw->myw);
+    if (cor2) {
+        free_cor(cor2);
+    }
+    cor2 = init_cor();
+    cor2->sw_myw = sw->myw;
+    cor2->sw_myh = sw->myh;
+    cor2->thread = g_thread_create(cor_thread_func, (gpointer)cor2, TRUE, NULL);
+    if (!cor2->thread) internal_("Can't run cor thread");
+    return 0;
+}
+
+
+gpointer cor_thread_func(gpointer arg){ 
+    int i,j, ret;
 	const struct cpoint *cp;
     struct kmpoint km, kmfirst, kmlast;
 	double h2, w2;
     gpointer hash, oldhash;
 	int kx, ky, color;
+    struct cor *cor = (struct cor *)arg;
+    char s[16];
 	
    /* dbg("recalc_cor(%s)\n", locator);*/
-    /*return 0;*/ 
-    ST_START;
+   /* return 0; */
+//    ST_START;
 
+    kmfirst.c=0;
+    //dbg("cor_thread_func: items=%d %f %f\n", cor->items, cor->sw_myh, cor->sw_myw);
 	g_hash_table_foreach_remove(cor->km, free_km_item, NULL); 
     oldhash=(gpointer)-1;
-	for (i=0, cp=cor->file1; i<cor->items; i++, cp++){
+	for (i=0, cp=cor->file; i<cor->items; i++, cp++){
+        if (cor->thread_break) return NULL;
         w2=(MY_PI*(double)cp->w)/18000;
 		h2=(MY_PI*(double)cp->h)/18000;
-		hw2km(sw->myh, sw->myw, h2, w2, &kx, &ky);
+		hw2km(cor->sw_myh, cor->sw_myw, h2, w2, &kx, &ky);
 		hash=k2key(kx, ky);
 		km.c=cp->c;
 		km.kx=kx;
@@ -99,6 +137,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
     /* big wwls (JN69) */
 	g_hash_table_foreach_remove(cor->wwl4, free_km_item, NULL); 
     for (i=-89;i<90;i++){
+        if (cor->thread_break) return NULL;
         if (i%10==0) color=-128;
         else color=-127;
         oldhash=(gpointer)-1;
@@ -106,7 +145,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         for (j=0;j<=360;j++){
             w2=(MY_PI*i)/180;
             h2=(MY_PI*j)/180;
-            hw2km(sw->myh, sw->myw, h2, w2, &kx, &ky);
+            hw2km(cor->sw_myh, cor->sw_myw, h2, w2, &kx, &ky);
             hash=k2key(kx, ky);
             km.kx=kx;
             km.ky=ky;
@@ -125,6 +164,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         }
     }
     for (j=0;j<360;j+=2){
+        if (cor->thread_break) return NULL;
         oldhash=(gpointer)-1;
         if (j%20==0) color=-128;
         else color=-127;
@@ -132,7 +172,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         for (i=-89;i<=89;i++){
             w2=(MY_PI*i)/180;
             h2=(MY_PI*j)/180;
-            hw2km(sw->myh, sw->myw, h2, w2, &kx, &ky);
+            hw2km(cor->sw_myh, cor->sw_myw, h2, w2, &kx, &ky);
             hash=k2key(kx, ky);
             km.kx=kx;
             km.ky=ky;
@@ -151,6 +191,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
     /* very big wwls (JN) */
 	g_hash_table_foreach_remove(cor->wwl2, free_km_item, NULL); 
     for (i=-80;i<90;i+=10){
+        if (cor->thread_break) return NULL;
         if (i%10==0) color=-128;
         else color=-127;
         oldhash=(gpointer)-1;
@@ -158,7 +199,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         for (j=0;j<=360;j++){
             w2=(MY_PI*i)/180;
             h2=(MY_PI*j)/180;
-            hw2km(sw->myh, sw->myw, h2, w2, &kx, &ky);
+            hw2km(cor->sw_myh, cor->sw_myw, h2, w2, &kx, &ky);
             hash=k2key(kx, ky);
             km.kx=kx;
             km.ky=ky;
@@ -177,6 +218,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         }
     }
     for (j=0;j<360;j+=20){
+        if (cor->thread_break) return NULL;
         oldhash=(gpointer)-1;
         if (j%10==0) color=-128;
         else color=-127;
@@ -184,7 +226,7 @@ int recalc_cor(struct subwin *sw, gchar *locator){
         for (i=-89;i<=89;i++){
             w2=(MY_PI*i)/180;
             h2=(MY_PI*j)/180;
-            hw2km(sw->myh, sw->myw, h2, w2, &kx, &ky);
+            hw2km(cor->sw_myh, cor->sw_myw, h2, w2, &kx, &ky);
             hash=k2key(kx, ky);
             km.kx=kx;
             km.ky=ky;
@@ -200,18 +242,28 @@ int recalc_cor(struct subwin *sw, gchar *locator){
             km.c=0;
         }
     } 
-    ST_STOP;
-	return 0;
+//    ST_STOP;
+   // sleep(5); 
+
+    sprintf(s, "COR\n");
+    ret = write(tpipe->threadpipe_write, s, strlen(s));
+	return NULL;
 }
 
+void cor_read_handler(char *line){
+//    dbg("cor_read_handler\n");
+    if (!cor2 || !cor2->thread) return;
 
-void read_cor_files(){
-
-	cor->file1=cor_tucnakcor;
-	cor->items=COR_ITEMS;
+    free_cor(cor);
+    cor = cor2;
+    cor2 = NULL;
+    if (gses->ontop->type!=SWT_MAP) return;
+    gses->ontop->gdirty = 1;
+    sw_map_redraw(gses->ontop, 0);
 }
 
 
+
 gboolean free_km_item(gpointer key, gpointer value, gpointer user_data){
 	struct kmarray *kma;
 
@@ -223,16 +275,6 @@ gboolean free_km_item(gpointer key, gpointer value, gpointer user_data){
     return 1;
 }
 
-void free_cor(struct cor *cor){
-
-	if (!cor) return;
-
-	g_hash_table_foreach_remove(cor->km, free_km_item, NULL); 
-	g_hash_table_foreach_remove(cor->wwl4, free_km_item, NULL); 
-	g_hash_table_foreach_remove(cor->wwl2, free_km_item, NULL); 
-	
-    g_free(cor);
-}
 
 gpointer k2key(int kx, int ky){
 	return (gpointer)(vint)( (kx & COR_KM_MASK) | ( (ky & COR_KM_MASK) << 16)) ; 
