@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -15,7 +15,7 @@ struct cw *cw;
 static gchar *cw_next;
 static gchar *wc_next;
 
-struct cw *init_cw(){
+struct cw *init_cw(void){
     struct cw *cw;
     time_t now;
     struct tm tm;
@@ -38,7 +38,7 @@ struct cw *init_cw(){
     return cw;
 }
 
-gboolean free_cw_item(gpointer key, gpointer value, gpointer user_data){
+static gboolean free_cw_item(gpointer key, gpointer value, gpointer user_data){
     struct cw_item *cwi;
     
     g_free(key);
@@ -49,7 +49,7 @@ gboolean free_cw_item(gpointer key, gpointer value, gpointer user_data){
     return 0;    
 }
 
-gboolean free_wc_item(gpointer key, gpointer value, gpointer user_data){
+static gboolean free_wc_item(gpointer key, gpointer value, gpointer user_data){
     struct wc_item *wci;
     
     g_free(key);
@@ -123,13 +123,15 @@ int load_cw_from_file(struct cw *cw, gchar *filename){
 void read_cw_files(struct cw *cw){
     gchar *s;
     
+#ifdef UNIX
     load_cw_from_file(cw, "/etc/tucnakcw");
+#endif
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL); 
     load_cw_from_file(cw, s);
     g_free(s);
 }
 
-void save_one_cw(gpointer key, gpointer value, gpointer user_data){
+static void save_one_cw(gpointer key, gpointer value, gpointer user_data){
     GString *gs;
     gchar *call;
     struct cw_item *cwi;
@@ -300,13 +302,15 @@ gchar *find_call_by_wwl(struct cw *cw, gchar *wwl){
 
 
 gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
-    int i;
+    int i,j;
     gchar *c;
     struct cw_item *cwi;
     gint qrv;
     static gchar qrv_str[33];
     gchar *ret;
-    char s[25];
+    char raw[25], raw2[25];
+    struct qso *q;
+    struct band *band;
     
     if (!aband) return NULL;
 
@@ -315,24 +319,34 @@ gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
     cwi = t_hash_table_lookup(cw->cw, call);
     if (cwi) qrv|=cwi->qrv;
 
-    cwi = t_hash_table_lookup(cw->cw, get_raw_call(s, call));
+    cwi = t_hash_table_lookup(cw->cw, get_raw_call(raw, call));
     if (cwi) qrv|=cwi->qrv;
 
-    c=g_strdup_printf("%s/P", s);
+    c=g_strdup_printf("%s/P", raw);
     cwi = t_hash_table_lookup(cw->cw, c);
     if (cwi) qrv|=cwi->qrv;
     g_free(c);
 
     if (!qrv) return NULL;
     
-    for (i=toupper(aband->bandchar)+1,c=qrv_str;
-         i<='Z';
-         i++){
+    for (i='A',c=qrv_str; i<='Z'; i++){
         
-        if (!(qrv & ctest->qrv & (1<<(i-'A')))) continue;
+        if (i==toupper(aband->bandchar)) continue;          /* same band */
+        if (!(qrv & ctest->qrv & (1<<(i-'A')))) continue;   /* we are not qrv on band */
+
+        band = find_band_by_bandchar(i);
+        if (band){
+            for (j=0; j<band->qsos->len; j++){
+                q = (struct qso *)g_ptr_array_index(band->qsos, j);
+                if (q->error || q->dupe) continue;
+                get_raw_call(raw2, q->callsign);
+                if (strcmp(raw2, raw)==0) goto next;
+            }
+        }
         
         *c=i;
         c++;
+next:;        
     }
     *c='\0';
     ret=strlen(qrv_str)?qrv_str:NULL;
@@ -340,7 +354,7 @@ gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
     return ret;
 }
 
-int is_string_in_array(GPtrArray *arr, gchar *str){
+/*static int is_string_in_array(GPtrArray *arr, gchar *str){
     int i;
     gchar *c;
 
@@ -352,7 +366,7 @@ int is_string_in_array(GPtrArray *arr, gchar *str){
         
     }
     return 0;
-}
+} */
 
 
 int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
@@ -361,6 +375,14 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     int qrb_int, qtf_int;
     char qrv_str[35];
     int qrvfilter;
+    char degree[2];
+
+#ifdef HAVE_SDL    
+    if (sdl) 
+        strcpy(degree, "\xb0");
+    else
+#endif
+        strcpy(degree, "");
 
     qrvfilter = ctest ? ctest->qrv : -1; 
     
@@ -376,11 +398,11 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     if (my_strstr(call, gses->qs_str)){
         if (cwi->wwl0) {
             qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
-            c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl0, qrb_int, qtf_int, qrv_str);
+            c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl0, qrb_int, qtf_int, degree, qrv_str);
             g_ptr_array_add(result, c);
             if (cwi->wwl1){ /* wwl1 is only together with wwl0 */
                 qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
-                c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl1, qrb_int, qtf_int, qrv_str);
+                c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl1, qrb_int, qtf_int, degree, qrv_str);
                 g_ptr_array_add(result, c);
             }
         }else{
@@ -392,7 +414,7 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     
     if (cwi->wwl0 && my_strstr(cwi->wwl0, gses->qs_str)){
         qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
-        c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl0, qrb_int, qtf_int, qrv_str);
+        c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl0, qrb_int, qtf_int, degree, qrv_str);
      /*   dbg(",%s", c);*/
         g_ptr_array_add(result, c);
         goto x;
@@ -400,7 +422,7 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     
     if (cwi->wwl1 && my_strstr(cwi->wwl1, gses->qs_str)){
         qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
-        c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl1, qrb_int, qtf_int, qrv_str);
+        c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl1, qrb_int, qtf_int, degree, qrv_str);
     /*    dbg(",%s", c);  */
         g_ptr_array_add(result, c);
         goto x;
@@ -424,25 +446,34 @@ void get_cw_qs(gchar *str){
     
     
     if (strlen(str)<2) {
+//        CONDGFREE(gses->qs_master);
         g_ptr_array_free_items(gses->qs);
         return;
     }
 
-    CONDGFREE(gses->qs_str);
-    gses->qs_str=g_strdup(str);
-    gses->qs_max_matches = term->y - QSONR_HEIGHT - 4 - cfg->loglines - 1;
-    if (aband) gses->qs_max_matches -= aband->spypeers->len;
-    if (ctest && ctest->bands->len==1) gses->qs_max_matches*=2;
-    if (!ctest) gses->qs_max_matches*=3;
-    qs_thread_create();
+    if (gses && gses->ontop && gses->ontop->type == SWT_HF){
+        CONDGFREE(gses->qs_master);
+       // ST_START;
+        gses->qs_master = masterdb_search(masterdb, str, gses->ontop->w);
+       // ST_STOP;
+        //log_adds(gses->qs_master);
+    }else{
+        CONDGFREE(gses->qs_str);
+        gses->qs_str=g_strdup(str);
+        gses->qs_max_matches = term->y - QSONR_HEIGHT - 4 - cfg->loglines - 1;
+        if (ctest) gses->qs_max_matches -= ctest->spypeers->len;
+        if (ctest && ctest->bands->len==1) gses->qs_max_matches*=2;
+        if (!ctest) gses->qs_max_matches*=3;
+        qs_thread_create();
+    }
 }
 
-gpointer qs_thread_func(gpointer data){
+static gpointer qs_thread_func(gpointer data){
     char s[256];
     GPtrArray *result;
     THashNode *node;
     gint i;
-    int count;   
+    int count, ret;   
 
     result=g_ptr_array_new();
     
@@ -463,35 +494,39 @@ gpointer qs_thread_func(gpointer data){
 x:; 
     g_index_array_qsort((GIndexArray *)result, compare_gstring); 
     
-    LOCK(gses->qs);
+    MUTEX_LOCK(gses->qs);
     g_ptr_array_free_all(gses->qs);
     gses->qs=result;
-    UNLOCK(gses->qs);
+    MUTEX_UNLOCK(gses->qs);
     
     sprintf(s, "CWQS\n");
-    write(tpipe->threadpipe_write, s, strlen(s));
+    ret = write(tpipe->threadpipe_write, s, strlen(s));
 
 	return NULL;
 }
 
-void qs_thread_create(){
-    if (gses->qs_thread) raise(SIGSEGV);
+void qs_thread_create(void){
+    if (gses->qs_thread) internal_("qs_thread_create gses->qs_thread");
     gses->qs_thread_break=0;
     gses->qs_thread=g_thread_create(qs_thread_func, (gpointer)NULL, TRUE, NULL);
-    if (!gses->qs_thread) raise(SIGSEGV);
+    if (!gses->qs_thread) internal_("qs_thread_create !gses->qs_thread");
 }
 
 
-void qs_thread_join(){
+void qs_thread_join(void){
     if (!gses->qs_thread) return;
+    dbg("join qs...\n");
     g_thread_join(gses->qs_thread);
+    dbg("done\n");
     gses->qs_thread=NULL;
 }
 
 void qs_thread_kill(){
     if (!gses->qs_thread) return;
     gses->qs_thread_break=1;
+//    dbg("join qs...\n");
     g_thread_join(gses->qs_thread);
+ //   dbg("done\n");
   /*  dbg("done\n");*/
     gses->qs_thread=NULL;
 }
@@ -538,6 +573,10 @@ void update_cw_from_ctest(struct cw *cw, struct contest *ctest){
 
 
 int similar_calls(const char *call1, const char *call2, int factor, int thr, int p){
+    factor = levenshtein(call1, call2);
+    p=1;
+#if 0    
+    // this code gives bad results
     const char *c1, *c2;
     int f1, f2, f3;
 
@@ -570,19 +609,67 @@ int similar_calls(const char *call1, const char *call2, int factor, int thr, int
 
     }
 
-    if (p) printf("similar %s %s = %d\n", call1, call2, factor);
+#endif    
+    if (p) {
+//        dbg("similar %s %s = %d", call1, call2, factor);
+//        printf("  leven=%d", levenshtein_distance(call1, call2));
+//        dbg("\n");
+    }
     /*else printf("          %s %s = %d\n", call1, call2, factor);*/
     return factor;
 }
+ 
+static int min3(int a, int b, int c)
+{
+    int min = a;
+    if (b < min) min=b;
+    if (c < min) min=c;
+    return min;
+}
+
+int levenshtein(const char *s, const char *t){
+    int i, j, k, n, m, cost, *d, ret;
+    n = strlen(s); 
+    m = strlen(t);
+    if (!n) return -1;
+    if (!m) return -1;
+    
+    // d is a table with n+1 rows and m+1 columns
+    d = mem_alloc(sizeof(int) * (m + 1) * (n + 1));
+    m++;
+    n++;
+    // step 2  
+    for(k = 0; k < n; k++) d[k] = k;    // deletion
+    for(k = 0; k < m; k++) d[k*n] = k;  // insertion
+    // step 3 and 4    
+    for(i = 1; i<n; i++)
+    for(j=1; j<m; j++){
+        // step 5
+        if(s[i - 1] == t[j - 1])
+            cost = 0;
+        else
+            cost = 1;
+        // step 6             
+        d[j * n + i] = min3(
+            d[(j - 1) * n + i] + 1,
+            d[j * n + i - 1] + 1,
+            d[(j - 1) * n + i - 1] + cost);
+    }
+    ret = d[n * m - 1];
+    mem_free(d);
+    return ret;
+}
 
 
-int get_susp_ambiguous_call(struct cw *cw, char *call, char *wwl, GString *gs, int thr){
+
+int get_susp_ambiguous_call(struct cw *cw, struct band *band, char *call, char *wwl, GString *gs, int thr){
     int i;
     THashNode *node;
     struct cw_item *cwi;
 
     for (i = 0; i < cw->cw->size; i++){
         for (node = cw->cw->nodes[i]; node; node = node->next){
+			int factor;
             cwi  = (struct cw_item *) node->value;
             
             if ((cwi->wwl0 && strcmp(cwi->wwl0, wwl)==0 && cwi->stamp0 > cw->minstamp) ||
@@ -590,10 +677,14 @@ int get_susp_ambiguous_call(struct cw *cw, char *call, char *wwl, GString *gs, i
                             
                 /* dbg("hit %s %s\n", node->key, wwl);*/
                 if (strcmp((gchar*)node->key, call)==0) continue;
-                if (similar_calls((gchar*)node->key, call, 0, thr, 0)<=thr){
+                if (band && get_qso_by_callsign(band, call)) continue;  // call is also worked, no warning
+
+                factor = similar_calls((gchar*)node->key, call, 0, thr, 0);
+                if (factor<=thr){
                     if (!gs) return 1;
 
                     g_string_append(gs, node->key);
+//                    g_string_append_printf(gs, "(%d<=%d)", factor, thr);
                     g_string_append_c(gs, ' ');
                 }
             }
@@ -605,6 +696,7 @@ int get_susp_ambiguous_call(struct cw *cw, char *call, char *wwl, GString *gs, i
 int cwdb_call_info(GString *gs, gchar *call, gchar *stroke){
     struct cw_item *cwi;
     char c[40];
+    int i;
 
     strcpy(c, call);
     strcat(c, stroke);
@@ -613,6 +705,13 @@ int cwdb_call_info(GString *gs, gchar *call, gchar *stroke){
     if (!cwi->wwl0) return 0;  /* wwl0 should be always defined */
     g_string_sprintfa(gs, "C_W: %-9s %s %d", c, cwi->wwl0, cwi->stamp0);
     if (cwi->wwl1) g_string_sprintfa(gs, ", %s %d", cwi->wwl1, cwi->stamp1);
-    g_string_append(gs, "\n");
+    g_string_append_c(gs, '\n');
+    if (cwi->qrv>0){
+        g_string_append(gs, "QRV on: ");
+        for (i=0;i<26;i++){
+            if ((1<<i)&cwi->qrv) g_string_append_c(gs, i+'A');
+        }
+        g_string_append_c(gs, '\n');
+    }
     return 1;
 }
