@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -38,13 +38,15 @@ gboolean free_gpointer_item(gpointer key, gpointer value, gpointer user_data){
     return TRUE;
 }
 
-gboolean free_dw_item(gpointer key, gpointer value, gpointer user_data){
+static gboolean free_dw_item(gpointer key, gpointer value, gpointer user_data){
     struct dw_item *dwi;
     
     g_free(key);
     dwi = (struct dw_item *)value;
     if (dwi->wwl0) g_free(dwi->wwl0);
     if (dwi->wwl1) g_free(dwi->wwl1);
+    if (dwi->continent) g_free(dwi->continent);
+    if (dwi->dxcname) g_free(dwi->dxcname);
 
     g_hash_table_foreach_remove(dwi->wwls, free_gpointer_item, NULL);
     g_hash_table_destroy(dwi->wwls);
@@ -53,7 +55,7 @@ gboolean free_dw_item(gpointer key, gpointer value, gpointer user_data){
     return FALSE;
 }
 
-gboolean free_wd_item(gpointer key, gpointer value, gpointer user_data){
+static gboolean free_wd_item(gpointer key, gpointer value, gpointer user_data){
     struct wd_item *wdi;
     
     g_free(key);
@@ -114,7 +116,7 @@ gchar *safe_fgets(GString *gs, FILE *f, int stripcomment){
     if (stripcomment){
         char *c;
 
-        c = index(gs->str, '#');
+        c = strchr(gs->str, '#');
         if (c) g_string_truncate(gs, c - gs->str);
     }
         
@@ -126,8 +128,10 @@ gchar *safe_fgets(GString *gs, FILE *f, int stripcomment){
 gchar *safe_mgets(GString *gs, const char *file, long int *pos, const long int len, int stripcomment){
 	const char *c;
 
-    /*dbg("sage_mgets(%d,%d,%d)\n", *pos, len, stripcomment);*/
+    /*dbg("safe_mgets(%d,%d,%d)\n", *pos, len, stripcomment);*/
 	g_string_truncate(gs, 0);
+    
+    if (*pos>=len) return NULL;
 
 	for (c=file+(*pos); c!=file+len; c++,(*pos)++){
 		switch(*c){
@@ -144,11 +148,11 @@ brk:;
     if (stripcomment){
         char *c;
 
-        c = index(gs->str, '#');
+        c = strchr(gs->str, '#');
         if (c) g_string_truncate(gs, c - gs->str);
     }
-/*    dbg("%3d '%s'\n",gs->len, gs->str);*/
-    if (*pos>=len) return NULL;
+//    dbg("safe_mgets: %3d '%s' *pos=%d len=%d\n",gs->len, gs->str, *pos, len);
+    if (*pos>len) return NULL;
     return gs->str;    
 }
 
@@ -168,7 +172,7 @@ void add_pd(struct dw *dw, gchar *prefix, gchar *dxc){
 }
 
 
-struct dw_item *add_dxc(struct dw *dw, gchar *dxc, gchar *wwl0, gchar *wwl1){
+struct dw_item *add_dxc(struct dw *dw, gchar *dxc, gchar *wwl0, gchar *wwl1, int waz, int itu, gchar *continent, gchar *dxcname, double latitude, double longitude){
     struct dw_item *dxci;
 
     /*dbg("add_dxc(dxc='%s', wwl='%s' '%s')\n",dxc, wwl0, wwl1);*/
@@ -187,6 +191,12 @@ struct dw_item *add_dxc(struct dw *dw, gchar *dxc, gchar *wwl0, gchar *wwl1){
 
     dxci->wwl0 = g_strdup(wwl0);
     dxci->wwl1 = g_strdup(wwl1);
+    if (waz) dxci->waz = waz;
+    if (itu) dxci->itu = itu;
+    if (continent) dxci->continent = g_strdup(continent);
+    if (dxcname) dxci->dxcname = g_strdup(dxcname);
+	if (longitude < 400) dxci->longitude = longitude;
+	if (latitude < 400) dxci->latitude = latitude;
     
     return dxci;
 }
@@ -251,12 +261,13 @@ int load_dw_from_file(struct dw *dw, gchar *filename){
 	int ret;
 	size_t r;
     
+    /*dbg("load_dw_from_file(%s, rt)\n", filename);*/
     f = fopen(filename, "rt");
     if (!f) {
 /*        dbg("Can't open '%s'\n",filename);*/
         return -1;
     }
-	if (fseek(f, SEEK_END, 0L)<0){
+	if (fseek(f, 0L, SEEK_END)<0){
 		fclose(f);
 		return -1;
 	}
@@ -266,7 +277,7 @@ int load_dw_from_file(struct dw *dw, gchar *filename){
 		return -1;
 	}
 	
-	if (fseek(f, SEEK_SET, 0L)<0){
+	if (fseek(f, 0L, SEEK_SET)<0){
 		fclose(f);
 		return -1;
 	}
@@ -317,7 +328,7 @@ int load_dw_from_mem(struct dw *dw, const char *file, const long int len){
         c1 = strtok_r(NULL, DW_DELIM, &token_ptr);
             
 
-        dxci = add_dxc(dw, dxc, c0, c1); 
+        dxci = add_dxc(dw, dxc, c0, c1, 0, 0, NULL, NULL, 404, 404); 
         
         add_dw(dxci, dxc, c0);
         add_dw(dxci, dxc, c1);
@@ -358,7 +369,7 @@ int load_cty_from_file(struct dw *dw, gchar *filename){
 /*        dbg("Can't open '%s'\n",filename);*/
         return -1;
     }
-	if (fseek(f, SEEK_END, 0L)<0){
+	if (fseek(f, 0L, SEEK_END)<0){
 		fclose(f);
 		return -1;
 	}
@@ -368,7 +379,7 @@ int load_cty_from_file(struct dw *dw, gchar *filename){
 		return -1;
 	}
 	
-	if (fseek(f, SEEK_SET, 0L)<0){
+	if (fseek(f, 0L, SEEK_SET)<0){
 		fclose(f);
 		return -1;
 	}
@@ -385,12 +396,13 @@ int load_cty_from_file(struct dw *dw, gchar *filename){
 
 int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
     GString *gs, *gs_next; 
-    gchar **line, **prefs;
-    int i;
-    gchar *dxc;
+    gchar **line, **prefs, *continent;
+    int i, waz, itu;
+    gchar *dxc, *dxcname;
     char wwl4[6];
 	long int pos;
     struct dw_item *dxci;
+	double latitude, longitude;
     
     
     gs      = g_string_sized_new(100);
@@ -419,22 +431,49 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
 
         
         line = g_strsplit(gs->str, CTY_DELIM, 9);
-        for (i=0;i<9;i++) if (line[i] ==NULL) break;
-        if (i!=9) continue;
+        for (i=0;i<9;i++) {
+            if (line[i] ==NULL){
+                break;
+            }
+        }
+        if (i!=9) {
+            g_strfreev(line);
+            continue;
+        }
+        
+        g_strstrip(line[0]);
+        dxcname = line[0];
+        g_strstrip(line[1]);
+        waz = atoi(line[1]);
+        g_strstrip(line[2]);
+        itu = atoi(line[2]);
+        g_strstrip(line[3]);
+        continent = line[3];
         
         g_strstrip(line[4]);
+		latitude = atof(line[4]);
         g_strstrip(line[5]);
+		longitude = - atof(line[5]);
         g_strstrip(line[7]);
         g_strstrip(line[8]);
         
         
         dxc=line[7];
-        if (index(dxc, '/')) {
+        if (dxc[0]=='*') dxc++; // strip leading star, TNX to DL5YBZ
+        /*slash = strchr(dxc, '/');
+        if (slash) {
+            *slash = '\0';
+            slash++;;
+        }         */
+        uc(dxc);
+
+        
+        //dbg("dxc='%s'\n", dxc);
+
+        if (!dxc || !*dxc) {
             g_strfreev(line);
             continue;
         }
-        
-        if (!dxc) continue;
         add_pd(dw, dxc, dxc);
         
 /*        dbg("h=%s, w=%s, d=%s, p=%s\n", line[5], line[6], line[7], line[8]);*/
@@ -444,10 +483,10 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
             char *c;
 
             g_strstrip(prefs[i]);
-            if (index(prefs[i],'/')) continue;
-            if ((c=index(prefs[i],'('))!=NULL) *c='\0';
-            if ((c=index(prefs[i],'['))!=NULL) *c='\0';
-            if ((c=index(prefs[i],';'))!=NULL) *c='\0';
+            if (strchr(prefs[i],'/')) continue;
+            if ((c=strchr(prefs[i],'('))!=NULL) *c='\0';
+            if ((c=strchr(prefs[i],'['))!=NULL) *c='\0';
+            if ((c=strchr(prefs[i],';'))!=NULL) *c='\0';
             
             if (strlen(prefs[i])>2 && !isdigit(prefs[i][ strlen(prefs[i])-1])) continue;
             
@@ -460,7 +499,7 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
         compute_wwl4(wwl4, -atof(line[5]), atof(line[4]));
 
         /*dbg("h=%s, w=%s, d=%s, wwl4=%s \n", line[4], line[5], line[7], wwl4);*/
-        dxci = add_dxc(dw, dxc, wwl4, NULL); 
+        dxci = add_dxc(dw, dxc, wwl4, NULL, waz, itu, continent, dxcname, latitude, longitude); 
         
         g_strfreev(line);
         
@@ -491,19 +530,23 @@ void read_dw_files(struct dw *dw){
 }
 
 
+struct dw_item *get_dw_item_by_call(struct dw *dw, gchar *call){
+    char dxcc[25];
+	
+    get_dxcc(dw, dxcc, call);
+    return g_hash_table_lookup(dw->dw, dxcc);
+}
 
 
 int wwl_is_ok_by_call(struct dw *dw, gchar *wwl, gchar *call){
-    char dxcc[25];
     struct dw_item *dxci;
     char swwl[5];
     
     if (!wwl || !call) return 1;
     
-    get_dxcc(dw, dxcc, call);
     /*dbg("dxcc=%s\n", dxcc);*/
             
-    dxci = g_hash_table_lookup(dw->dw, dxcc);
+    dxci = get_dw_item_by_call(dw, call);
     if (!dxci) {
 /*        dbg("can't find dxcc (%s) for call %s \n", dxcc, call);*/
         return 0;
@@ -548,7 +591,7 @@ int get_susp(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl, int ambig){
     }
 
 skip:;    
-    if (ambig) if (get_susp_ambiguous_call(cw, call, wwl, NULL, 1)) susp|=0x100;
+    if (ambig) if (get_susp_ambiguous_call(cw, NULL, call, wwl, NULL, 1)) susp|=0x100;
     /* dbg("   ret=%d\n", susp);*/
     return susp;
 }
@@ -558,7 +601,7 @@ int get_susp_call(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl){
     int susp;
     char s[40]; 
 
-   /* dbg("get_susp_call('%s', '%s')\n", call, wwl);*/
+    //dbg("get_susp_call('%s', '%s')\n", call, wwl);
     if (!call || !*call){  /* pokud je vsechno NULL nebo "", tak get_susp skonci s nulou a test s /P pokracuje */
         return 0;
     }
@@ -568,21 +611,24 @@ int get_susp_call(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl){
 
     get_raw_call(s, call);
     if (strcmp(s, call)==0){ /* without /p */
-      /*  dbg("without ");*/
         strcat(s, "/P");
+        //dbg("without %s", s);
         if (find_wwl_by_call(cw, s)) {
+            //dbg(" SUSP");
             susp=1;   /* call is without /p and in C_W exists call with /p */
         }
     }else{        /* call is with /p */
-        /*dbg("with ");*/
+        //dbg("with %s", s);
         if (find_wwl_by_call(cw, s)) {
+            //dbg(" SUSP");
             susp=1;   /* call is with /p and in C_W exists call without /p */
         }
     }
 
-    if (wwl && *wwl) susp=get_susp_ambiguous_call(cw, call, wwl, NULL, 1);
     
-   /* dbg("   ret=%d\n", susp);*/
+    if (wwl && *wwl) susp=get_susp_ambiguous_call(cw, NULL, call, wwl, NULL, 1);
+    
+    //dbg("   ret=%d\n", susp);
     return susp;
 }
 
@@ -597,33 +643,56 @@ int get_susp_loc(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl){
         g_string_sprintfa(gs, "%s %s\n", qso->callsign?qso->callsign:"?", qso->locator?qso->locator:"?"); \
     }err=1;}
 
-int qso_info(struct qso *qso, GString *gs, GString *gs2){
+static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
     char s[40],s1[40], raw[40];
-    int err,ret, susp, k;
+    int err, susp, k;
+    enum suspcall ret;
     char *wwl0, *wwl1;
     struct band *b;
     struct qso *q;
     int i,j;
 
 
-    ret=0;
+    ret=SUSP_NONE;
     if (!qso) return -1;
     if (qso->error) return -1;
     if (qso->dupe) return -1;
 
     err=0;
+
+    if (qso->band && qso->band->stats && qso->band->stats->nqsos > 0){
+        if (qso->qsop > qso->band->stats->nqsop/qso->band->stats->nqsos) if (ret < SUSP_QRB) ret = SUSP_QRB;
+    }
+
+    if (qso->remark && *qso->remark) {
+        if (ret < SUSP_REM) ret = SUSP_REM;
+    }
+
     if (!find_name_by_call(namedb, qso->callsign) &&
         !find_wwl_by_call(cw, qso->callsign)){
         ADDQSO;
         g_string_sprintfa(gs, "Callsign unknown in namedb and C_W\n");
-        ret=1;
+        if (ret < SUSP_WARN) ret = SUSP_WARN;
     }
 
+    if (qso->mode == MOD_SSB_CW || qso->mode == MOD_CW_SSB){
+        ADDQSO;
+        g_string_sprintfa(gs, "Crossmode QSO ?\n");
+        if (ret < SUSP_WARN) ret = SUSP_WARN;
+    }
+
+    if (!get_dw_item_by_call(dw, qso->callsign)){
+        ADDQSO;
+        g_string_sprintfa(gs, "Unknown DXCC prefix\n");
+        if (ret < SUSP_ERR) ret = SUSP_ERR;
+    }
+
+
     if (qso->locator && *qso->locator){
         if (!wwl_is_ok_by_call(dw, qso->locator, qso->callsign)){
             ADDQSO;
-            g_string_sprintfa(gs, "Prefix and big wwl dont match\n");
-            ret=2;
+            g_string_sprintfa(gs, "Prefix and big wwl don't match\n");
+            if (ret < SUSP_ERR) ret = SUSP_ERR;
         }
     }
 
@@ -631,7 +700,7 @@ int qso_info(struct qso *qso, GString *gs, GString *gs2){
     wwl1=find_wwl_by_call(cw, NULL); 
 
     susp=0;
-    if (wwl0!=NULL){ /* we don't know this call, cannot be suspicious */
+    if (wwl0 != NULL && qso->locator != NULL){ /* we don't know this call, cannot be suspicious */
         susp = 1;
         if (wwl0 && strcasecmp(wwl0, qso->locator)==0) susp = 0; 
         if (wwl1 && strcasecmp(wwl1, qso->locator)==0) susp = 0; 
@@ -641,7 +710,7 @@ int qso_info(struct qso *qso, GString *gs, GString *gs2){
         if (susp){
             ADDQSO;
             g_string_sprintfa(gs, "Known call from unknown locator\n");
-            if (ret<1) ret=1;
+            if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
     
@@ -650,10 +719,10 @@ int qso_info(struct qso *qso, GString *gs, GString *gs2){
     if (strcmp(s, qso->callsign)==0){ /* without /p */
         strcpy(s1, s);
         strcat(s, "/P");
-        if (find_wwl_by_call_newer(cw, raw, cw->minstamp)) {
+        if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
             ADDQSO;
             g_string_sprintfa(gs, "Call is without /P and in C_W exists call with /P\n");
-            if (ret<1) ret=1;
+            if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
         strcat(s1, "/");
         for (k=0;k<=9;k++){
@@ -663,60 +732,63 @@ int qso_info(struct qso *qso, GString *gs, GString *gs2){
             if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
                 ADDQSO;
                 g_string_sprintfa(gs, "Call is without /%d and in C_W exists call with /%d\n", k, k);
-                if (ret<1) ret=1;
+                if (ret < SUSP_WARN) ret = SUSP_WARN;
             }
         }
 
     }else{        /* call is with /p */
         if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
             ADDQSO;
-            g_string_sprintfa(gs, "Call is with /P and in C_W exists call without /P\n");
-            if (ret<1) ret=1;
+            g_string_sprintfa(gs, "Call is with /P and in C_W exists call without /P or CEPT prefix\n");
+            if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
 
-    g_string_truncate(gs2, 0);
-    get_susp_ambiguous_call(cw, qso->callsign, qso->locator, gs2, 1);
-    if (gs2->len>0){
-        ADDQSO;
-        g_string_sprintfa(gs, "Other similar call known from this locator. Maybe %s?\n", gs2->str);
-        if (ret<1) ret=1;
-    }
-
-    for (i=0; i<ctest->bands->len; i++){
-        b = (struct band*)g_ptr_array_index(ctest->bands, i);
-        if (b == qso->band) continue;
-        for (j=0; j<b->qsos->len; j++){
-            q = (struct qso *)g_ptr_array_index(b->qsos, j);
-            if (q->error || q->dupe) continue;
-
-            get_raw_call(s, q->callsign);
-
-            /*if (q->callsign[0]=='D' && q->callsign[1]=='M' && q->callsign[2]=='7'){
-                 dbg("call %s %s %s %d\n", q->callsign, s, raw, strcmp(s, raw));
-            } */
-            if (strcmp(s, raw) != 0) continue; /* not same station */
-             /*dbg("RAW match %s %s\n", s, raw);*/
-            if (strcmp(q->callsign, qso->callsign) != 0){ 
-                ADDQSO;
-                g_string_sprintfa(gs, "Callsign on band %c is %s\n", toupper(b->bandchar), q->callsign);
-                if (ret<1) ret=1;
-            }
-            
-            if (strcmp(q->locator, qso->locator) != 0){
-                ADDQSO;
-                g_string_sprintfa(gs, "Locator on band %c is %s\n", toupper(b->bandchar), q->locator);
-                if (ret<1) ret=1;
-            }
-
+    if (qso->locator != NULL){
+        g_string_truncate(gs2, 0);
+        get_susp_ambiguous_call(cw, qso->band, qso->callsign, qso->locator, gs2, 1);
+        if (gs2->len>0){
+            ADDQSO;
+            g_string_sprintfa(gs, "Other similar call known from this locator. Maybe %s?\n", gs2->str);
+            if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
+
+    if (ctest){
+        for (i=0; i<ctest->bands->len; i++){
+            b = (struct band*)g_ptr_array_index(ctest->bands, i);
+            if (b == qso->band) continue;
+            for (j=0; j<b->qsos->len; j++){
+                q = (struct qso *)g_ptr_array_index(b->qsos, j);
+                if (q->error || q->dupe) continue;
+
+                get_raw_call(s, q->callsign);
+
+                /*if (q->callsign[0]=='D' && q->callsign[1]=='M' && q->callsign[2]=='7'){
+                     dbg("call %s %s %s %d\n", q->callsign, s, raw, strcmp(s, raw));
+                } */
+                if (strcmp(s, raw) != 0) continue; /* not same station */
+                 /*dbg("RAW match %s %s\n", s, raw);*/
+                if (strcmp(q->callsign, qso->callsign) != 0){ 
+                    ADDQSO;
+                    g_string_sprintfa(gs, "Callsign on band %c is %s\n", toupper(b->bandchar), q->callsign);
+                    if (ret < SUSP_WARN) ret = SUSP_WARN;
+                }
+                
+                if (qso->locator != NULL && strcmp(q->locator, qso->locator) != 0){
+                    ADDQSO;
+                    g_string_sprintfa(gs, "Locator on band %c is %s\n", toupper(b->bandchar), q->locator);
+                    if (ret < SUSP_WARN) ret = SUSP_WARN;
+                }
+
+            }
+        }
     
-
+    }
     return ret;
 }
 
-void menu_qso_check(union cba_t cba){
+void menu_qso_check(cba_t cba){
     GString *gs, *gs2;
     struct band *band;
     struct qso *qso;
@@ -744,7 +816,7 @@ void menu_qso_check(union cba_t cba){
             qso = (struct qso *)g_ptr_array_index(band->qsos, j);
             ret=qso_info(qso, gs, gs2);
             if (ret) g_string_sprintfa(gs, "\n");
-            if (ret>=0) qso->suspcall=ret;
+            if (ret>=0) qso->suspcall1=ret;
         }
     }
 
@@ -777,13 +849,14 @@ void menu_qso_check(union cba_t cba){
     return;
 }
 
-void call_info(union cba_t cba){
+void call_info(cba_t cba){
     struct qso *qso=NULL;
     GString *gs, *gs2;
     static char ss[1024];
     int err,freeqso,i;
     char *title, *c;
     char raw[30];
+    struct qrv_item *qi;
 
     gs=g_string_sized_new(500);
     gs2=g_string_sized_new(500);
@@ -792,9 +865,19 @@ void call_info(union cba_t cba){
 	freeqso=0;
 /*    dbg("TMPQ.callsign=%s %p\n", TMPQ.callsign, aband);*/
 
-    if (cba.qso){
-		qso=cba.qso;
+    if (GETCBA(cba,qso) != NULL){
+		qso = (struct qso *)GETCBA(cba, qso);
         qso_info(qso, gs, gs2);
+    }else if (strlen(INPUTLN(aband)->cdata) > 0){
+        qso=g_new0(struct qso, 1);
+        qso->callsign = INPUTLN(aband)->cdata;    /* foreign key */
+        qso->locator = NULL;
+        qso->rsts = "";
+        qso->qsonrs = "";
+        qso->rstr = "";
+        qso->qsonrr = "";
+        qso_info(qso, gs, gs2);
+		freeqso=1;
     }else if (aband && (!TMPQ.callsign || !strlen(TMPQ.callsign))){
         if (aband->qsos->len<=0) goto x;
         qso=(struct qso *)g_ptr_array_index(aband->qsos, aband->qsos->len-1);
@@ -803,23 +886,15 @@ void call_info(union cba_t cba){
         /* tmpqso is valid */
         qso=g_new0(struct qso, 1);
         qso->callsign = TMPQ.callsign;    /* foreign keys */
-        qso->locator = TMPQ.locator;
-        qso->rsts = TMPQ.rsts;
-        qso->qsonrs = TMPQ.qsonrs;
-        qso->rstr = TMPQ.rstr;
-        qso->qsonrr = TMPQ.qsonrr;
+        qso->locator = TMPQ.locator?TMPQ.locator:"";
+        qso->rsts = TMPQ.rsts?TMPQ.rsts:"";
+        qso->qsonrs = TMPQ.qsonrs?TMPQ.qsonrs:"";
+        qso->rstr = TMPQ.rstr?TMPQ.rstr:"";
+        qso->qsonrr = TMPQ.qsonrr?TMPQ.qsonrr:"";
         qso_info(qso, gs, gs2);
 		freeqso=1;
     }else{
-        qso=g_new0(struct qso, 1);
-        qso->callsign = gses->il->cdata;    /* foreign key */
-        qso->locator = "";
-        qso->rsts = "";
-        qso->qsonrs = "";
-        qso->rstr = "";
-        qso->qsonrr = "";
-        qso_info(qso, gs, gs2);
-		freeqso=1;
+        goto x;
     }
 
     if (gs->len>0){
@@ -850,6 +925,34 @@ void call_info(union cba_t cba){
         g_string_sprintfa(gs, "C_W: %s not found\n", qso->callsign);
     }
 
+    /* wkd */
+    if (ctest){
+        g_string_append(gs, "WKD on: ");
+        for (i = 0; i < ctest->bands->len; i++){
+            struct band *b = (struct band*) g_ptr_array_index(ctest->bands, i);
+            if (get_qso_by_callsign(b, qso->callsign)) g_string_append_c(gs, toupper(b->bandchar));
+        }
+        g_string_append_c(gs, '\n');
+    }
+
+    /* qrv */
+    qi = qrv_get(qrvdb, qso->callsign);
+    g_string_sprintfa(gs, "QRV: ");
+    if (qi){
+//        g_string_sprintfa(gs, " %s", qi->wwl);
+        if (aband){
+            g_string_sprintfa(gs, " %dx", qi->wkd[aband->bandchar - 'a']);
+        }
+        if (qi->kst_time != 0){
+            struct tm utc;
+            gmtime_r(&qi->kst_time, &utc);
+            g_string_sprintfa(gs, " %2d:%02d", utc.tm_hour, utc.tm_min);
+        }
+    }else{
+        g_string_append(gs, " 0x (not in database)");
+    }
+    g_string_append_c(gs, '\n');
+
     /* namedb */
     c=find_name_by_call(namedb, qso->callsign);
     if (c){
@@ -902,26 +1005,64 @@ gchar *find_dxc_by_wwl(struct dw *dw, gchar *wwl){
 
 
 char *get_dxcc(struct dw *dw, char *buf, char *call){ 
-    char *c0, *c1, *dxc;
+    char *c0, *c1, *c2, *dxc;
+    char s[100], *stroke = NULL;
+    
+    c0=c1=c2=NULL;
+    safe_strncpy0(buf, call, 20);
+
+    if (regmatch(call, "^(.*[0-9])[A-Z]*(/[A-Z])?", &c0, &c1, &c2, NULL)==0){
+        
+      //  dbg("\n%s '%s' '%s' '%s'\n", call, c0, c1, c2);
+        safe_strncpy0(buf, c1, 20);
+        if (c2 != NULL){
+            stroke = c2;
+        }
+        while (*buf!='\0'){
+            safe_strncpy0(s, buf, 20);
+            if (stroke) strcat(s, stroke);
+        //    dbg("trying1 '%s'\n", s);
+            dxc = g_hash_table_lookup(dw->pd, s);
+            if (dxc) {
+          //      dbg("get_dxcc(%s)='%s'\n", call, s); 
+                safe_strncpy0(buf, dxc, 20);
+                goto found;
+            }
+            buf[strlen(buf)-1]='\0'; 
+        }
+        
+        safe_strncpy0(buf, c1, 20);
+        while (*buf!='\0'){
+            safe_strncpy0(s, buf, 20);
+           // dbg("trying2 '%s'\n", s);
+            dxc = g_hash_table_lookup(dw->pd, s);
+            if (dxc) {
+             //   dbg("get_dxcc(%s)='%s'\n", call, s); 
+                safe_strncpy0(buf, dxc, 20);
+                goto found;
+            }
+            buf[strlen(buf)-1]='\0'; 
+        }
+        safe_strncpy0(buf, call, 20);
+    }
+found:;    
+    if (c0) mem_free(c0);
+    if (c1) mem_free(c1);
+    if (c2) mem_free(c2);
+//    dbg("get_dxcc(%s)='%s'\n", call, buf);
+    return buf;  
+}
+
+char *get_pref(char *buf, char *call){ 
+    char *c0, *c1;
     
     c0=c1=NULL;
     safe_strncpy0(buf, call, 20);
     if (regmatch(call, "^(.*[0-9])[A-Z]*", &c0, &c1, NULL)==0){
-        
         safe_strncpy0(buf, c1, 20);
-        while (*c1!='\0'){
-           dxc = g_hash_table_lookup(dw->pd, c1);
-           if (dxc) {
-      /*        dbg("get_dxcc(%s)='%s'\n", call, c1); */
-              safe_strncpy0(buf, dxc, 20);
-              break;
-           }
-           c1[strlen(c1)-1]='\0'; 
-        }
     }
     if (c0) mem_free(c0);
     if (c1) mem_free(c1);
-/*    dbg("get_dxcc='%s'\n", dxc);*/
-    return buf;  
+    dbg("get_pref(%s)='%s'\n", call, buf);
+    return buf;
 }
-
