@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -11,13 +11,16 @@
 #include "header.h"
 
 
+static int nlocs = 0;
+static int nnames = 0;
+
 int load_ebw_from_file(struct cw *cw, struct namedb *namedb, char *filename){
     FILE *f;
     struct dbfhdr hdr;
     struct dbffield *fields,*field;
-    int i,j;
+    int i, j;
     unsigned char *record;
-    int ret=0,ofs;
+    int ret=0, ret2, ofs;
 	int callofs,calllen,wwlofs,wwllen,nameofs,namelen;
     gchar *call,*wwl,*name;
 	
@@ -77,7 +80,7 @@ int load_ebw_from_file(struct cw *cw, struct namedb *namedb, char *filename){
     record=g_new0(unsigned char, hdr.recsize);
     
     for (j=0; j<hdr.records; j++){
-        fread(record, hdr.recsize, 1, f);
+        ret2 = fread(record, hdr.recsize, 1, f);
         if (record[0]=='*') continue; /* deleted record */
       
 		call=wwl=name=NULL;
@@ -102,15 +105,15 @@ int load_ebw_from_file(struct cw *cw, struct namedb *namedb, char *filename){
 
 		if (wwl && strlen(wwl)==6){
 			if (!find_wwl_by_call(cw, call)){  /*only unknown calls, we don't know which is newer*/
-				add_cw(cw, call, wwl, 19700101, 0);
+				add_cw(cw, call, wwl, 19700101, NULL);
 				add_wc(cw, wwl, call, 19700101);
+				nlocs++;
 			}
 		}
 		
-		if (name && strlen(name)>=2){
-			if (namedb && g_hash_table_size(namedb->names)){
-			 	add_namedb(namedb, call, name);   
-			}
+		if (namedb && name && strlen(name)>=2){
+		  	add_namedb(namedb, call, name);   
+			nnames++;
 		}
 
 		g_free(call);
@@ -118,7 +121,7 @@ int load_ebw_from_file(struct cw *cw, struct namedb *namedb, char *filename){
 		CONDGFREE(name);
     }
     g_free(record);
-    ret=1;
+    ret=0;
 x:;
     fclose(f);
     
@@ -129,5 +132,27 @@ x:;
 }
 
 int read_ebw_files(struct cw *cw, struct namedb *namedb){
-	return load_ebw_from_file(cw, namedb, SHAREDIR"/vhf.dbf");
+    gchar *s, *s1;
+	int ret;
+    
+	nlocs = 0;
+	nnames = 0;
+    s = g_strconcat(getenv("HOME"), "/tucnak/vhf.dbf", NULL); 
+    ret=load_ebw_from_file(cw, namedb, s);
+	if (ret==0){
+		log_addf("Loaded %d locators and %d names from %s", nlocs, nnames, s);
+        g_free(s);
+		return 0;		
+	}
+	
+	s1 = SHAREDIR"/vhf.dbf";
+	ret=load_ebw_from_file(cw, namedb, s1);
+	if (ret==0){
+		log_addf("Loaded %d locators and %d names from %s", nlocs, nnames, s1);
+		return 0;		
+	}else{
+		log_addf("Can't read %s or %s", s, s1);
+	}
+    g_free(s);
+	return ret;
 }
