@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -12,6 +12,8 @@
 
 static int edi_serid_used;
 
+#define dumpvar(var) { int dvi; dbg("%s=",#var); for (dvi=0; dvi<sizeof(var); dvi++) dbg("%02x ", ((char *)&var)[dvi]); dbg("\n");}
+
 void add_qso_str1(GString *gs, struct qso *q, struct band *b){
     char s[1024];
 
@@ -55,19 +57,19 @@ void add_qso_str1(GString *gs, struct qso *q, struct band *b){
 void add_qso_str2(GString *gs, struct qso *q, struct band *b){
     
     if (!q->dupe && !q->error && strcasecmp(q->callsign, "ERROR")!=0)
-        g_string_sprintfa(gs, "%d;", q->qsop * b->bandmulti);
+        g_string_sprintfa(gs, "%d;", q->qsop/* * b->bandmulti*/);
     else{
         g_string_append(gs, "0;");
         dump_qso(q, "write_edi_qsos");
     }
     
-    if (q->new & NEW_EXC) g_string_append_c(gs, 'N');
+    if (q->new_ & NEW_EXC) g_string_append_c(gs, 'N');
     g_string_append_c(gs, ';');
     
-    if (q->new & NEW_WWL) g_string_append_c(gs, 'N');
+    if (q->new_ & NEW_WWL) g_string_append_c(gs, 'N');
     g_string_append_c(gs, ';');
     
-    if (q->new & NEW_DXC) g_string_append_c(gs, 'N');
+    if (q->new_ & NEW_DXC) g_string_append_c(gs, 'N');
     g_string_append_c(gs, ';');
     
     if (q->dupe) g_string_append_c(gs, 'D');
@@ -79,7 +81,7 @@ void add_qso_str3(GString *gs, struct qso *q, struct band *b){
     g_string_append(gs, q->source);
     g_string_append_c(gs, ';');
     
-    g_string_append(gs, fixsemi(q->operator));
+    g_string_append(gs, fixsemi(q->operator_));
     g_string_append_c(gs, ';');
     
     g_string_sprintfa(gs, "%d;", (int)q->stamp);
@@ -95,13 +97,16 @@ void add_qso_str3(GString *gs, struct qso *q, struct band *b){
     g_string_append(gs, fixsemi(s));
     
     g_string_sprintfa(gs, ";%d", q->ser_id);
+    
+    g_string_sprintfa(gs, ";%1.0f", q->qrg);
 }
 
 
-int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
+static int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     char s[1024];
     GString *gs;
     int ret;
+	int qsomult;
     
     gs = g_string_sized_new(1024);
     g_string_sprintfa(gs, "[REG1TEST;1]\r\n");
@@ -134,11 +139,13 @@ int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "SRXEq=%s\r\n",    safe_strncpy0(s, b->srxeq, 69));
     g_string_sprintfa(gs, "SAnte=%s\r\n",    safe_strncpy0(s, b->sante, 69));
     g_string_sprintfa(gs, "SAntH=%s\r\n",    safe_strncpy0(s, b->santh, 69));
-    g_string_sprintfa(gs, "CQSOs=%d;%d\r\n",    b->stats->nqsos, b->bandmulti);         
+    qsomult = ctest->qsomult;
+    if (ctest->qsop_method==16) qsomult = qsomult16(b);
+    g_string_sprintfa(gs, "CQSOs=%d;%d\r\n",    b->stats->nqsos, qsomult);         
     g_string_sprintfa(gs, "CQSOP=%d\r\n",       b->stats->nqsop);            
     g_string_sprintfa(gs, "CWWLs=%d;%d;%d\r\n", g_hash_table_size(b->stats->wwls),  ctest->wwlbonu, ctest->wwlmult==0 ? 1 : ctest->wwlmult);  
     g_string_sprintfa(gs, "CWWLB=%d\r\n",       g_hash_table_size(b->stats->wwls) * ctest->wwlbonu);
-    g_string_sprintfa(gs, "CExcs=%d;%d;%d\r\n", g_hash_table_size(b->stats->excs),  ctest->excbonu, ctest->excmult==0 ? 1 : ctest->excmult); 
+    g_string_sprintfa(gs, "CExcs=%d;%d;%d\r\n", b->stats->nexcp,  ctest->excbonu, ctest->excmult==0 ? 1 : ctest->excmult); 
     g_string_sprintfa(gs, "CExcB=%d\r\n",       g_hash_table_size(b->stats->excs) * ctest->excbonu);             
     g_string_sprintfa(gs, "CDXCs=%d;%d;%d\r\n", g_hash_table_size(b->stats->dxcs),  ctest->dxcbonu, ctest->dxcmult==0 ? 1 : ctest->dxcmult); 
     g_string_sprintfa(gs, "CDXCB=%d\r\n",       g_hash_table_size(b->stats->dxcs) * ctest->dxcbonu);
@@ -152,7 +159,7 @@ int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     return ret;
 }
 
-int write_edi_empty_remarks(struct band *b, struct config_band *confb, FILE *f){
+static int write_edi_empty_remarks(struct band *b, struct config_band *confb, FILE *f){
     char s[1024]; 
     int ret;
     GString *gs;
@@ -170,7 +177,7 @@ int write_edi_empty_remarks(struct band *b, struct config_band *confb, FILE *f){
     return ret;
 }   
 
-int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f){
+static int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f){
     char s[1024];
     struct qso *q;
     GString *gs;
@@ -187,25 +194,35 @@ int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "[TUCNAK extension]\r\n");
     g_string_sprintfa(gs, "default_rs=%s\r\n",  ctest->default_rs);
     g_string_sprintfa(gs, "default_rst=%s\r\n", ctest->default_rst);
+    g_string_sprintfa(gs, "prefmult=%d\r\n", ctest->prefmult);
+    g_string_sprintfa(gs, "prefglob=%d\r\n", ctest->prefglob);
     g_string_sprintfa(gs, "dxcbonu=%d\r\n", ctest->dxcbonu);
     g_string_sprintfa(gs, "dxcmult=%d\r\n", ctest->dxcmult);
     g_string_sprintfa(gs, "excused=%d\r\n", ctest->excused);
+    g_string_sprintfa(gs, "exctype=%d\r\n", ctest->exctype);
+    g_string_sprintfa(gs, "excname=%s\r\n", ctest->excname);
     g_string_sprintfa(gs, "excbonu=%d\r\n", ctest->excbonu);
     g_string_sprintfa(gs, "excmult=%d\r\n", ctest->excmult);
+    g_string_sprintfa(gs, "exccfm=%d\r\n",  ctest->exccfm);
     g_string_sprintfa(gs, "qsoused=%d\r\n", ctest->qsoused);
     g_string_sprintfa(gs, "qsomult=%d\r\n", ctest->qsomult);
+    g_string_sprintfa(gs, "qsoglob=%d\r\n", ctest->qsoglob);
     g_string_sprintfa(gs, "qsop_method=%d\r\n",  ctest->qsop_method);
     g_string_sprintfa(gs, "rstused=%d\r\n", ctest->rstused);
+    g_string_sprintfa(gs, "defrstr=%d\r\n", ctest->defrstr);
     g_string_sprintfa(gs, "total_method=%d\r\n", ctest->total_method);
     g_string_sprintfa(gs, "wwltype=%d\r\n", ctest->wwltype);
     g_string_sprintfa(gs, "wwlused=%d\r\n", ctest->wwlused);
     g_string_sprintfa(gs, "wwlbonu=%d\r\n", ctest->wwlbonu);
     g_string_sprintfa(gs, "wwlmult=%d\r\n", ctest->wwlmult);
+    g_string_sprintfa(gs, "wwlcfm=%d\r\n", ctest->wwlcfm);
+    g_string_sprintfa(gs, "tttype=%d\r\n", ctest->tttype);
+    g_string_sprintfa(gs, "expmode=%d\r\n", ctest->expmode);
     
     g_string_sprintfa(gs, "bandchar=%c\r\n", b->bandchar);
-    g_string_sprintfa(gs, "bandmulti=%d\r\n", b->bandmulti);
-    g_string_sprintfa(gs, "mode=%d\r\n", b->mode);
-    g_string_sprintfa(gs, "operator=%s\r\n", b->operator);
+//    g_string_sprintfa(gs, "bandmulti=%d\r\n", b->bandmulti);
+    g_string_sprintfa(gs, "mode=%d\r\n", gses->mode);
+    g_string_sprintfa(gs, "operator=%s\r\n", b->operator_);
     g_string_sprintfa(gs, "psect=%d\r\n", b->psect);
     g_string_sprintfa(gs, "readonly=%d\r\n", b->readonly);
     g_string_sprintfa(gs, "saveid=%d\r\n", b->saveid);
@@ -228,7 +245,7 @@ int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f){
 }   
 
 
-int write_edi_qsos(struct band *b, struct config_band *confb, FILE *f){
+static int write_edi_qsos(struct band *b, struct config_band *confb, FILE *f){
     struct qso *q;
     GString *gs;
     int i;
@@ -256,7 +273,7 @@ int write_edi_qsos(struct band *b, struct config_band *confb, FILE *f){
 }
 
 
-int write_edi_unfi(struct band *b, struct config_band *confb, FILE *f){
+static int write_edi_unfi(struct band *b, struct config_band *confb, FILE *f){
     int i;
     gchar *c;
     GString *gs;
@@ -274,7 +291,29 @@ int write_edi_unfi(struct band *b, struct config_band *confb, FILE *f){
     return ret;
 }
 
-int export_all_bands_edi(){
+static int write_edi_sked(struct band *b, struct config_band *confb, FILE *f){
+    int i;
+    GString *gs;
+    int ret;
+    struct sked *sked;
+    struct zstring *zs;
+
+    gs = g_string_new("");
+    
+    for (i=0; i<b->skeds->len; i++){
+        sked = (struct sked *) g_ptr_array_index(b->skeds, i);
+        zs = sked_format(sked);
+        g_string_sprintfa(gs, "sked%d=%s\r\n", i, zs->str);
+        zfree(zs);
+    }
+    ret=fprintf(f, "%s", gs->str) != gs->len;
+    g_string_free(gs, TRUE);
+    
+    return ret;
+}
+
+
+int export_all_bands_edi(void){
     struct band *band;
     struct config_band *confb;
     int i, err;
@@ -287,16 +326,25 @@ int export_all_bands_edi(){
     
     /*dbg("export_all_bands_edi()\n");*/
     for (i=0; i<ctest->bands->len; i++){
+        char *section;
+        
         band = g_ptr_array_index(ctest->bands, i);
         confb = get_config_band_by_bandchar(band->bandchar);
 
         stats_thread_join(band);
         if (band->stats->nqsos <=0) continue;
 
-        filename = g_strdup_printf("%s/%s%s.edi",
-                        ctest->directory,
-                        band->psect ? confb->ok_section_single : confb->ok_section_multi,
-                        uc(get_raw_call(callbuf,ctest->pcall)));
+        section = band->psect ? confb->ok_section_single : confb->ok_section_multi;
+        if (section && *section)
+            filename = g_strdup_printf("%s/%s%s.edi",
+                            ctest->directory,
+                            section,
+                            uc(get_raw_call(callbuf,ctest->pcall)));
+        else
+            filename = g_strdup_printf("%s/%s_%c.edi",
+                            ctest->directory,
+                            uc(get_raw_call(callbuf,ctest->pcall)),
+                            toupper(band->bandchar));
         
         f=fopen(filename,"wb"); /* must be b for windoze */
         if (!f) {
@@ -324,7 +372,7 @@ int export_all_bands_edi(){
     return 0;
 }
 
-gchar *get_ctest_directory(int floppy){
+static gchar *get_ctest_directory(int floppy){
     gchar *dir;
     
     if (floppy==0){
@@ -416,6 +464,7 @@ int save_all_bands_txt(int autosave){
             err|=write_edi_header  (band,confb,f);
             err|=write_edi_remarks (band,confb,f);
             err|=write_edi_unfi    (band,confb,f);
+            err|=write_edi_sked    (band,confb,f);
             err|=write_edi_qsos    (band,confb,f);
             err|=fclose(f);
             if (err) {    
@@ -471,9 +520,27 @@ int save_all_bands_txt(int autosave){
         save_fifo_to_file(gtalk, filename);
         g_free(filename);
     
-        filename = g_strdup_printf("%s/sked",dir);
+/*        filename = g_strdup_printf("%s/sked",dir);
         save_fifo_to_file(gsked, filename);
+        g_free(filename);*/
+
+        filename = g_strdup_printf("%s/spy", dir);  
+        save_spypeers_to_file(filename); /* called also in control.c */
         g_free(filename);
+        
+        filename = g_strdup_printf("%s/hicalls", dir);  
+        save_hicalls_to_file(filename);
+        g_free(filename);
+
+        filename = g_strdup_printf("%s/qrv", dir);  
+        save_qrv_to_file(filename);
+        g_free(filename);
+
+        filename = g_strdup_printf("%s/chart", dir);  
+        save_chart_to_file(filename);
+        g_free(filename);
+
+        
 
         g_free(dir);
     }
@@ -482,7 +549,7 @@ int save_all_bands_txt(int autosave){
 }
 
 
-GHashTable *load_edi_until(FILE *f, GHashTable *hash, gchar *until){
+static GHashTable *load_edi_until(FILE *f, GHashTable *hash, gchar *until){
     GString *gs;
     gchar *c;
     gchar **a;
@@ -526,7 +593,7 @@ GHashTable *load_edi_until(FILE *f, GHashTable *hash, gchar *until){
     return hash;
 }
 
-gchar *load_edi_remarks_until(FILE *f, gchar *until){
+static gchar *load_edi_remarks_until(FILE *f, gchar *until){
     GString *gs;
     gchar *c;
     gchar *ret,*tmp;
@@ -550,7 +617,7 @@ gchar *load_edi_remarks_until(FILE *f, gchar *until){
     return ret;
 }
 
-GPtrArray *load_edi_qsos_until(FILE *f, gchar *until){
+static GPtrArray *load_edi_qsos_until(FILE *f, gchar *until){
     GString *gs;
     GPtrArray *array;
     gchar *c;
@@ -583,7 +650,8 @@ x:;
     return array;
 }
 
-void skip_edi_until(FILE *f, gchar *until){
+#if 0
+static void skip_edi_until(FILE *f, gchar *until){
     GString *gs;
     gchar *c;
     
@@ -596,20 +664,21 @@ void skip_edi_until(FILE *f, gchar *until){
 /*    dbg("skip_edi_until: END '%s' \n", c);*/
     g_string_free(gs, 1);
 }
+#endif
 
-gboolean free_item(gpointer key, gpointer value, gpointer x){
+static gboolean free_item(gpointer key, gpointer value, gpointer x){
     g_free(key);
     g_free(value);
     return TRUE;
 }
 
-void destroy_string_hash(GHashTable *hash){
+static void destroy_string_hash(GHashTable *hash){
     g_hash_table_foreach_remove(hash, free_item, NULL);
     g_hash_table_destroy(hash);
 }
 
 
-void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar *date){
+static void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar *date){
     gchar **items;
     int i, j;
     gchar *c, *add;
@@ -650,7 +719,7 @@ void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar
             safe_strncpy0(century, q->date_str, 3);
             add=g_strdup_printf("%s;%s;%d;%s;"
                                 "%s;%s;%d;",    
-                    net->myid, fixsemi(b->operator), (int)now, "",
+                    net->myid, fixsemi(b->operator_), (int)now, "",
                     century, "", -1);
             items = g_strsplit(add, ";", 0);
             g_free(add);
@@ -660,14 +729,20 @@ void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar
        
         for (j=0;j<=4;j++) if (!items[j]) goto x;
         
-        q->source   = g_strdup(items[0]);
-        q->operator = g_strdup(items[1]);
-        fixsemi(q->operator);
+        q->source    = g_strdup(items[0]);
+        q->operator_ = g_strdup(items[1]);
+        fixsemi(q->operator_);
         q->stamp    = atoi(items[2]);
         if (!items[5]){
             q->remark   = g_strdup("");
         }else{
             q->remark   = g_strdup(items[5]);
+
+            if (!items[6]){
+                q->qrg = 0.0;
+            }else{
+                q->qrg = atof(items[6]);
+            }
         }
 
         if (q->error && *items[3]){
@@ -689,6 +764,12 @@ void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar
         }else{
             q->ser_id=-1; /* filled in add_qso_to_index and rewritten after loop. only for compatibility */
         }
+
+        if (items[6] && items[7]){
+            q->qrg = atof(items[7]);
+        }else{
+            q->qrg = 0.0; 
+        }
         
         g_strfreev(items);
         compute_qrbqtf(q);
@@ -698,16 +779,15 @@ void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar
         
 x:;
     }
+    recalc_allb_stats();
     minute_stats(b);
 /*    g_hash_table_foreach(ctest->bystamp, foreach_source_print, NULL);*/
 
-/*   don't needed   DIRTY_BAND(); */
-    recalc_statsfifo(b);
     
 }
 
 
-void add_unfi_from_mem(struct band *b, GHashTable *hash){
+static void add_unfi_from_mem(struct band *b, GHashTable *hash){
     int i;
     gchar *add, *c;
     
@@ -724,7 +804,32 @@ void add_unfi_from_mem(struct band *b, GHashTable *hash){
     
 }
 
-int select_enable_all(const struct dirent *de){
+static void add_sked_from_mem(struct band *b, GHashTable *hash){
+    int i;
+    gchar *add, *c;
+    struct zstring *zs;
+    struct sked *sked;
+    
+    for (i=0;;i++){
+        c = g_strdup_printf("sked%d", i);
+        add = g_hash_table_lookup(hash, c);
+        g_free(c);
+        if (!add) break;
+//        dbg("sked=''\n", add);
+        zs = zstrdup(add);
+        sked = sked_parse(zs);
+        zfree(zs);
+        if (!sked) {
+            dbg("bad format of sked '%s'\n", add);
+            continue;
+        }
+        g_ptr_array_add(b->skeds, sked);
+        sked->worked = get_qso_by_callsign(b, sked->callsign) != NULL;
+    }
+}
+
+
+static int select_enable_all(const struct dirent *de){
     return 1;
 }
 
@@ -737,7 +842,7 @@ int select_enable_all(const struct dirent *de){
     }
 
 
-int convert_bo_s_hash(GHashTable *hash, char *dir){
+static int convert_bo_s_hash(GHashTable *hash, char *dir){
     struct config_band *tmpcb;
     gchar *c, *date;
 
@@ -751,7 +856,7 @@ int convert_bo_s_hash(GHashTable *hash, char *dir){
     }
 
     date = g_strdup(dir);
-    c = index(date,'.');
+    c = strchr(date,'.');
     if (c) *c='\0';
     if (!g_hash_table_lookup(hash, "tdate"))
         g_hash_table_insert(hash, g_strdup("tdate"), g_strdup(date));
@@ -775,10 +880,12 @@ int convert_bo_s_hash(GHashTable *hash, char *dir){
     return 0;
 }
 
+//int *pnn = NULL;
 /* loads edi and txt */
 void load_contest_edi(gchar *date, int edi){
     struct dirent **namelist=NULL;
-    int n=0, i;
+    int nn=0; // probably buggy register optimisation in gcc 4.3.2 amd64
+    int i;
     gchar *datedir, *file, *pband;
     struct stat st;
     char *ext, *c;
@@ -796,7 +903,6 @@ void load_contest_edi(gchar *date, int edi){
     file = NULL;
     datedir = NULL;
     
-	ST_START;
     c=g_strdup_printf("%s/tucnak/%s/desc", getenv("HOME"), date);
     descfile = fopen(c, "rt+");
     if (!descfile){
@@ -842,24 +948,19 @@ void load_contest_edi(gchar *date, int edi){
     load_fifo_from_file(gtalk, file,1);
     fifo_addfq(gtalk, VTEXT(T_LOADING_CTEST_S), date);
     CONDGFREE(file);
-    
-    file = g_strdup_printf("%s/sked", datedir);
-    load_fifo_from_file(gsked, file,1);
-    fifo_addfq(gsked, VTEXT(T_LOADING_CTEST_S), date);
-    CONDGFREE(file);
-    
-    n = tucnak_scandir(datedir, &namelist, select_enable_all, tucnak_alphasort);
-    if (n<=0) {
+
+
+    nn = tucnak_scandir(datedir, &namelist, select_enable_all, tucnak_alphasort);
+    //printf("%lx\n", &nn);
+    if (nn<=0) {
         errbox(VTEXT(T_NO_FILES), 0);
         fclose(descfile); 
         fclose(logfile); 
         goto x;
     }
     
- 
-
     edi_serid_used=0;
-    for (i=0; i<n; i++){
+    for (i=0; i<nn; i++){
         GString *gs;
 
         ext = edi ? "[a-z]\\.edi$" : "[a-z]\\.txt$";
@@ -917,9 +1018,10 @@ destroy:;
             init_ctest();
             load_ctest_from_mem(ctest, datedir, hash);
             maps_reload();
+            chart_reload();
         }
         
-        b = init_band(NULL, hash);
+        b = init_band(NULL, hash, NULL);
         if (!b) {
           /*  dbg("load_contest_edi: b=NULL!\n");*/
             goto destroy;
@@ -928,7 +1030,8 @@ destroy:;
         /* edi_serid_used=0; must be assigned but before band loop */
         add_qsos_from_mem(b, hash, qsos, date);
         add_unfi_from_mem(b, hash);
-        clear_tmpqsos(b);
+        add_sked_from_mem(b, hash);
+        clear_tmpqsos(b, 1);
         destroy_string_hash(hash);  
         if (remarks){
             CONDGFREE(b->remarks);
@@ -961,20 +1064,20 @@ destroy:;
 /*                dbg("read '%s'\n", gs->str);*/
                         
                 if (strncmp(gs->str,"# ",2)==0){
-                    process_input(b, gs->str+2);
+                    process_input(b, gs->str+2, 0);
                     continue;
                 }
                 if (strncmp(gs->str,"CLR",3)==0){
-                    clear_tmpqsos(b);
+                    clear_tmpqsos(b, 1);
                     continue;
                 }
                 if (strncmp(gs->str,"Q ",2)==0){
-                    process_input(b, "");
+                    process_input(b, "", 0);
                     continue;
                 }
                 if (strncmp(gs->str,"O ",2)==0){
-                    if (b->operator) g_free(b->operator);
-                    b->operator = fixsemi(g_strdup(gs->str+2));
+                    if (b->operator_) g_free(b->operator_);
+                    b->operator_ = fixsemi(g_strdup(gs->str+2));
                     continue;
                 }
                     
@@ -997,12 +1100,32 @@ destroy:;
         free_ctest();
         goto x;
     }
+    
+    
+    CONDGFREE(file);
+    
+    file = g_strdup_printf("%s/spy", datedir);
+    load_spypeers_from_file(file);
+    CONDGFREE(file);
+
+    file = g_strdup_printf("%s/hicalls", datedir);
+    load_hicalls_from_file(file);
+    CONDGFREE(file);
+
+    file = g_strdup_printf("%s/qrv", datedir);
+    load_qrv_from_file(qrvdb, file);
+    CONDGFREE(file);
+    
+    file = g_strdup_printf("%s/chart", datedir);
+    load_chart_from_file(file);
+    CONDGFREE(file);
+    
     check_autosave();
 
     ctest->descfile = descfile;
     ctest->logfile = logfile;
     ctest->cdate = g_strdup(date);
-    c = index(ctest->cdate, '.');
+    c = strchr(ctest->cdate, '.');
     if (c) *c='\0';
 
     now = time(NULL);
@@ -1010,10 +1133,10 @@ destroy:;
     nowdate_int = (tm.tm_year+1900)*10000 + (tm.tm_mon+1)*100 + tm.tm_mday;
     ctestdate_int = atoi(ctest->cdate);
     if (/* nowdate_int >= ctestdate_int &&*/ nowdate_int <= ctestdate_int + 2) 
-        ctest->recording = 1;
+        ctest->oldcontest = 0;
     else
-        ctest->recording = 0;
-    dbg("ctestdate=%d nowdate=%d recording=%d\n", ctestdate_int, nowdate_int, ctest->recording);
+        ctest->oldcontest = 1;
+//    dbg("ctestdate=%d nowdate=%d oldcontest=%d\n", ctestdate_int, nowdate_int, ctest->oldcontest);
 
     
     if (!edi_serid_used){
@@ -1022,6 +1145,19 @@ destroy:;
     }
 
     g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
+    g_index_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
+
+    
+    set_ctest_title();
+    clear_exc(excdb);
+    read_exc_files(excdb, ctest->exctype, ctest->excname);
+    
+    recalc_allb_stats();
+    for (i=0; i<ctest->bands->len; i++){
+        struct band *b = (struct band *)g_ptr_array_index(ctest->bands, i);
+        b->dirty_statsf = 1;
+        recalc_statsfifo(b);
+    }
     
     dump_all_sources(ctest);
     net_send_ac();
@@ -1029,21 +1165,28 @@ destroy:;
     if (cfg->startband){
         struct band *b;
         b=find_band_by_pband(cfg->startband);
+    //  pnn = &nn;
         activate_band(b); /* b==NULL is handled by function */
     }
 /*#ifdef HAVE_SDL		
     maps_reload(); dela se pri activate_band 
 #endif*/
+    qrv_recalc_wkd(qrvdb);
+    qrv_recalc_qrbqtf(qrvdb);
+    qrv_recalc_gst(qrvdb);
     net_send_operator();
+    net_send_read_write_bands();
 
 x: 
+    dbg("nnx=%d\n", nn);
+//     dumpvar(nn);
+//     dumpvar(i);
     if (namelist){
-        for (i=0; i<n; i++) free(namelist[i]);
+        for (i=0; i<nn; i++) free(namelist[i]);
         free(namelist);
     }
     CONDGFREE(datedir);   
     CONDGFREE(file);   
-	ST_STOP;
 }
 
 void import_edi(void *xxx, char *filename){
