@@ -3,16 +3,20 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-GHashTable *mbs;
-GMutex *mbs_mutex;
+static GHashTable *mbs;
+static GMutex *mbs_mutex;
+#ifdef LEAK_DEBUG_LIST
+char *mbs_file;
+int mbs_line;
+#endif
 
 void do_not_optimize_here(void *p)
 {
@@ -23,8 +27,11 @@ void do_not_optimize_here(void *p)
 long mem_amount = 0;
 long last_mem_amount = -1;
 #ifdef LEAK_DEBUG_LIST
-struct list_head memory_list = { &memory_list, &memory_list };
+/*struct list_head memory_list = { &memory_list, &memory_list };*/
+struct alloc_header memory_list = { &memory_list, &memory_list };
 GMutex *memory_list_mutex;
+char *memory_list_file;
+int memory_list_line;
 #endif
 #endif
 
@@ -36,7 +43,7 @@ static inline void force_dump(void)
     exit(1);
 }
 
-void check_memory_leaks()
+void check_memory_leaks(void)
 {
 #ifdef LEAK_DEBUG
     if (mem_amount) {
@@ -47,13 +54,13 @@ void check_memory_leaks()
         {
             int r = 0;
             struct alloc_header *ah;
-            LOCK(memory_list);
+            MUTEX_LOCK(memory_list);
             foreach (ah, memory_list) {
                 fprintf(stderr, "%s%p:%d @ %s:%d", r ? ", ": "", (char *)ah + L_D_S, ah->size, ah->file, ah->line), r = 1;
                 if (ah->comment) fprintf(stderr, ":\"%s\"", ah->comment);
             }
             fprintf(stderr, "\n");
-            UNLOCK(memory_list);
+            MUTEX_UNLOCK(memory_list);
         }
 #endif
         force_dump();
@@ -61,7 +68,7 @@ void check_memory_leaks()
 #endif
 }
 
-void er(int b, char *m, va_list l)
+static void er(int b, char *m, va_list l)
 {
     if (b) fprintf(stderr, "%c", (char)7);
     vfprintf(stderr, m, l);
@@ -74,23 +81,41 @@ void error(char *m, ...)
     va_list l;
     va_start(l, m);
     er(1, m, l);
-}
+} 
 
 int errline;
 char *errfile;
 
-char errbuf[4096];
+static char errbuf[4096];
 
-void int_error(char *m, ...)
+#ifdef UNIX
+void internal_error(char *m, ...)
 {
     va_list l;
     free_all_itrms();
     va_start(l, m);
-    sprintf(errbuf, "\033[1mINTERNAL ERROR\033[0m at %s:%d: ", errfile, errline);
+    sprintf(errbuf, "\033[1mINTERNAL ERROR\033[0m in pid=%d at %s:%d: ", getpid(), errfile, errline);
     strcat(errbuf, m);
     er(1, errbuf, l);
     force_dump();
 }
+#endif
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	       
+void internal_error(char *m, ...)
+{
+	char buf[4096];
+    va_list l;
+    va_start(l, m);
+    sprintf(errbuf, "At %s:%d\r\n", errfile, errline);
+    strcat(errbuf, m);
+    vsnprintf(buf, sizeof(buf)-1, errbuf, l);
+	MessageBox(NULL, buf, "Tucnak internal error", MB_OK | MB_ICONERROR);
+    exit(1);
+}
+
+#endif
 
 void debug_msg(char *m, ...)
 {
@@ -104,9 +129,9 @@ void debug_msg(char *m, ...)
 
 int debug_type=0;
 char *debug_filename=NULL;
-FILE *debug_file=NULL;
+static FILE *debug_file=NULL;
 
-void init_debug(){
+void init_debug(void){
     mbs=g_hash_table_new(g_direct_hash, g_direct_equal);
     mbs_mutex=g_mutex_new();
 #ifdef LEAK_DEBUG_LIST
@@ -115,15 +140,16 @@ void init_debug(){
     
     switch (debug_type){
         case 1:
-            debug_file=fopen(debug_filename, "at");
+            debug_file=fopen(debug_filename, "wt"); // fixme was at
             break;
         case 2:
             debug_file=stderr;
             break;    
     }
+    //dbg("init_debug: sizeof(alloc_header)=%d, L_D_S=%d\n", sizeof(struct alloc_header), L_D_S);
 }
 
-void free_debug(){
+void free_debug(void){
     if (!debug_file) return;
     if (debug_file!=stderr) fclose(debug_file);
 
@@ -141,100 +167,31 @@ void dbg(char *m, ...)
     if (!debug_file) return;
     va_start(l, m);
     vfprintf(debug_file, m, l);
+#ifdef _MSC_VER
+	{	
+		gchar *c = g_strdup_vprintf(m, l);
+		OutputDebugString(c);
+		g_free(c);
+	}
+#endif
     va_end(l);
     fsync(fileno(debug_file));
 }
 
-void dbg_bcast(char *m, ...)
+void trace(int enable, char *m, ...)
 {
     va_list l;
-    gchar *c;
 
-    if (!cfg || !cfg->trace_bcast) return;
+    if (!debug_file) return;
+    if (!enable) return;
     va_start(l, m);
-    c=g_strdup_vprintf(m, l);
+    vfprintf(debug_file, m, l);
     va_end(l);
-    
-    dbg("%s",c);
-    log_adds(c);
-
-    g_free(c);
-}
-
-void dbg_sock(char *m, ...)
-{
-    va_list l;
-    gchar *c;
-
-    if (!cfg || !cfg->trace_sock) return;
-    va_start(l, m);
-    c=g_strdup_vprintf(m, l);
-    va_end(l);
-    
-    dbg("%s",c);
-    log_adds(c);
-
-    g_free(c);
-}
-
-void dbg_recv(char *m, ...)
-{
-    va_list l;
-    gchar *c;
-
-    if (!cfg || !cfg->trace_recv) return;
-    va_start(l, m);
-    c=g_strdup_vprintf(m, l);
-    va_end(l);
-    
-    dbg("%s",c);
-    log_adds(c);
-
-    g_free(c);
-}
-
-void dbg_send(char *m, ...)
-{
-    va_list l;
-    gchar *c;
-
-    if (!cfg || !cfg->trace_send) return;
-    va_start(l, m);
-    c=g_strdup_vprintf(m, l);
-    va_end(l);
-    
-    dbg("%s",c);
-    log_adds(c);
-
-    g_free(c);
-}
-
-void dbg_qsos(char *m, ...)
-{
-    va_list l;
-    gchar *c;
-
-    if (!cfg || !cfg->trace_qsos) return;
-    va_start(l, m);
-    c=g_strdup_vprintf(m, l);
-    va_end(l);
-    
-    dbg("%s\n",c);
-#if 0    
- raise(SIGSEGV); log_adds(c);
-#else
-    if (ctest && ctest->logfile){
-        fprintf(ctest->logfile, "%s\n", c);
-    }
-#endif
-
-    g_free(c);
+    fsync(fileno(debug_file));
 }
 
 
-
-
-void print_str_hash(gpointer key, gpointer value, gpointer data){
+static void print_str_hash(gpointer key, gpointer value, gpointer data){
     dbg("\tkey='%s' \tvalue='%s'\n", key, value);
 }
 
@@ -252,6 +209,7 @@ void *debug_mem_alloc(char *file, int line, size_t size)
 #endif
 #ifdef LEAK_DEBUG_LIST
     gpointer orig_key, value;
+    char *c;
 #endif    
     
     if (!size) return DUMMY;
@@ -259,7 +217,7 @@ void *debug_mem_alloc(char *file, int line, size_t size)
     mem_amount += size;
     size += L_D_S;
 #endif
-    if (!(p = xmalloc(size))) {
+    if (!(p = xmalloc(size + MCHK_LEAD))) {
         error("ERROR: out of memory (malloc returned NULL)\n");
         return NULL;
     }
@@ -268,14 +226,24 @@ void *debug_mem_alloc(char *file, int line, size_t size)
     p = (char *)p + L_D_S;
     ah->size = size - L_D_S;
 #ifdef LEAK_DEBUG_LIST
+//    fprintf(stderr, "malloc(%d)=%p p=%p L_D_S=%d\n", size + MCHK_LEAD, ah, p, L_D_S);
+    for (c = (char *)p - MCHK_LEAD; c != p; c++) {
+        *c = MCHK_PATTERN;
+//        fprintf(stderr, "%p=%02x\n", c, (unsigned char)*c);
+
+    }
+    for (c = (char *)p + ah->size; c != (char *)p + ah->size + MCHK_TRAIL; c++){
+        *c = MCHK_PATTERN;
+//        fprintf(stderr, "%p=%02x\n", c, (unsigned char)*c);
+    }
     ah->file = file;
     ah->line = line;
     ah->comment = NULL;
-    LOCK(memory_list);
+    MUTEX_LOCK(memory_list);
     add_to_list(memory_list, ah);
-    UNLOCK(memory_list);
+    MUTEX_UNLOCK(memory_list);
 
-    LOCK(mbs);
+    MUTEX_LOCK(mbs);
     if (g_hash_table_lookup_extended(mbs, (char*)p, &orig_key, &value)){
         int cnt=(vint)value;
         if (value!=0){
@@ -285,7 +253,7 @@ void *debug_mem_alloc(char *file, int line, size_t size)
         }
     }
     g_hash_table_insert(mbs, p, (gpointer)1);
-    UNLOCK(mbs);
+    MUTEX_UNLOCK(mbs);
 #endif
 #endif
 /*    fprintf(stderr, "debuf_mem_alloc(%s:%d, %d)=%p\n", file, line, size, p);*/
@@ -300,12 +268,13 @@ void debug_mem_free(char *file, int line, void *p)
 #endif
 #ifdef LEAK_DEBUG_LIST
     gpointer orig_key, value;
+    char *c;
 #endif    
 
     
     if (p == DUMMY) return;
     if (!p) {
-        errfile = file, errline = line, int_error("mem_free(NULL)");
+        errfile = file, errline = line, internal_error("mem_free(NULL)");
         return;
     }
 #ifdef LEAK_DEBUG
@@ -313,10 +282,25 @@ void debug_mem_free(char *file, int line, void *p)
     p = (char *)p - L_D_S;
     ah = p;
 #ifdef LEAK_DEBUG_LIST
-    LOCK(mbs);
+    for (c = (char *)xp - MCHK_LEAD; c != xp; c++) {
+        if (*c == MCHK_PATTERN) continue;
+        fprintf(stderr, " free: corrupted leading block %p:%d @ %s:%d  ", xp, ah->size, ah->file, ah->line);
+        for (c = (char *)xp - MCHK_LEAD; c != xp; c++) fprintf(stderr, " %02x", (unsigned char)*c);
+        fprintf(stderr, "\n");
+        break;
+    }
+    for (c = (char *)xp + ah->size; c != (char *)xp + ah->size + MCHK_TRAIL; c++) {
+        if (*c == MCHK_PATTERN) continue;
+        fprintf(stderr, " free: corrupted trailing block %p:%d @ %s:%d  ", xp, ah->size, ah->file, ah->line);
+        for (c = (char *)xp - MCHK_LEAD; c != xp; c++) fprintf(stderr, " %02x", (unsigned char)*c);
+        fprintf(stderr, "\n");
+        break;
+    }
+    MUTEX_LOCK(mbs);
     if (g_hash_table_lookup_extended(mbs, xp, &orig_key, &value)){
         int cnt=(vint)value;
         if (cnt!=1){
+            MUTEX_UNLOCK(mbs);
             free_all_itrms();
             fprintf(stderr, " free: usage count is %d: %p:%d @ %s:%d\n", cnt, xp, ah->size, ah->file, ah->line);
             force_dump();
@@ -324,10 +308,10 @@ void debug_mem_free(char *file, int line, void *p)
         g_hash_table_remove(mbs, xp);
         g_hash_table_insert(mbs, xp, (gpointer)0);
     }
-    UNLOCK(mbs);
-    LOCK(memory_list);
+    MUTEX_UNLOCK(mbs);
+    MUTEX_LOCK(memory_list);
     del_from_list(ah);
-    UNLOCK(memory_list);
+    MUTEX_UNLOCK(memory_list);
     if (ah->comment) free(ah->comment);
 #endif
     mem_amount -= ah->size;
@@ -343,12 +327,13 @@ void *debug_mem_realloc(char *file, int line, void *p, size_t size)
 #endif
 #ifdef LEAK_DEBUG_LIST
     gpointer orig_key, value;
+    char *c;
 #endif    
 
 
     if (p == DUMMY) return debug_mem_alloc(file, line, size);
     if (!p) {
-        errfile = file, errline = line, int_error("mem_realloc(NULL, %d)", size);
+        errfile = file, errline = line, internal_error("mem_realloc(NULL, %d)", size);
         return NULL;
     }
     if (!size) {
@@ -356,22 +341,37 @@ void *debug_mem_realloc(char *file, int line, void *p, size_t size)
         return DUMMY;
     }
 #ifdef LEAK_DEBUG_LIST    
-    LOCK(mbs);
+    ah = (struct alloc_header*)((char*)p - L_D_S);
+    for (c = (char *)p - MCHK_LEAD; c != p; c++) {
+        if (*c == MCHK_PATTERN) continue;
+        fprintf(stderr, " free: corrupted leading block %p:%d @ %s:%d  ", p, ah->size, ah->file, ah->line);
+        for (c = (char *)p - MCHK_LEAD; c != p; c++) fprintf(stderr, " %02x", (unsigned char)*c);
+        fprintf(stderr, "\n");
+        break;
+    }
+    for (c = (char *)p + ah->size; c != (char *)p + ah->size + MCHK_TRAIL; c++) {
+        if (*c == MCHK_PATTERN) continue;
+        fprintf(stderr, " free: corrupted trailing block %p:%d @ %s:%d  ", p, ah->size, ah->file, ah->line);
+        for (c = (char *)p - MCHK_LEAD; c != p; c++) fprintf(stderr, " %02x", (unsigned char)*c);
+        fprintf(stderr, "\n");
+        break;
+    }
+    MUTEX_LOCK(mbs);
     if (g_hash_table_lookup_extended(mbs, p, &orig_key, &value)){
         int cnt=(vint)value;
         if (cnt!=1){
 			dbg("debug_mem_realloc: cnt=%d\n", cnt);
-			UNLOCK(mbs);
+			MUTEX_UNLOCK(mbs);
             free_all_itrms();
-            fprintf(stderr, " realloc1: usage count is %d: %p @ %s:%d (allocated %s:%d)\n", cnt, p, file, line, ah->file, ah->line);
+            fprintf(stderr, " realloc1: usage count is %d: %p @ %s:%d\n", cnt, p, file, line);
             force_dump();
         }
         g_hash_table_remove(mbs, p);
         g_hash_table_insert(mbs, p, (gpointer)0);
     }
-    UNLOCK(mbs);
+    MUTEX_UNLOCK(mbs);
 #endif    
-    if (!(p = xrealloc((char *)p - L_D_S, size + L_D_S))) {
+    if (!(p = xrealloc((char *)p - L_D_S, size + L_D_S + MCHK_TRAIL))) {
         error("ERROR: out of memory (realloc returned NULL)\n");
         return NULL;
     }
@@ -379,12 +379,23 @@ void *debug_mem_realloc(char *file, int line, void *p, size_t size)
     ah = p;
     mem_amount += size - ah->size;
     ah->size = size;
+    p = (char *)p + L_D_S;
 #ifdef LEAK_DEBUG_LIST
+//    fprintf(stderr, "realloc(%d)=%p p=%p L_D_S=%d\n", size + MCHK_LEAD, ah, p, L_D_S);
+    for (c = (char *)p - MCHK_LEAD; c != p; c++) {
+        *c = MCHK_PATTERN;
+//        fprintf(stderr, "%p=%02x\n", c, (unsigned char)*c);
+
+    }
+    for (c = (char *)p + ah->size; c != (char *)p + ah->size + MCHK_TRAIL; c++){
+        *c = MCHK_PATTERN;
+//        fprintf(stderr, "%p=%02x\n", c, (unsigned char)*c);
+    }
     ah->prev->next = ah;
     ah->next->prev = ah;
     
-    LOCK(mbs);
-    if (g_hash_table_lookup_extended(mbs, (char*)p+L_D_S, &orig_key, &value)){
+    MUTEX_LOCK(mbs);
+    if (g_hash_table_lookup_extended(mbs, p, &orig_key, &value)){
         int cnt=(vint)value;
         if (value!=0){
             free_all_itrms();
@@ -392,11 +403,11 @@ void *debug_mem_realloc(char *file, int line, void *p, size_t size)
             force_dump();
         }
     }
-    g_hash_table_insert(mbs, (char*)p+L_D_S, (gpointer)1);
-    UNLOCK(mbs);
+    g_hash_table_insert(mbs, p, (gpointer)1);
+    MUTEX_UNLOCK(mbs);
 #endif
 #endif
-    return (char *)p + L_D_S;
+    return p;
 }
 
 void set_mem_comment(void *p, char *c, int l)
@@ -451,28 +462,40 @@ gchar *debug_g_strndup(char *file, int line, gchar *str, int len){
     return c;
 }
 
+#undef g_strdup_vprintf
+#undef g_free
+
 gchar *debug_g_strdup_printf(char *file, int line, gchar *fmt, ...){
     va_list l;
-    int len;
-    char *c;
+    //int len;
+    char *c,*d;
 
     va_start(l, fmt);
-    len=g_printf_string_upper_bound(fmt, l);
+  /*  len=g_printf_string_upper_bound(fmt, l);
     c=debug_mem_alloc(file, line, len);
     if (c==DUMMY) return NULL;
-    g_vsprintf(c, fmt, l);
+    g_vsprintf(c, fmt, l);*/
+
+    d = g_strdup_vprintf(fmt, l);
+    c = debug_g_strdup(file, line, d);
+    g_free(d);
+
     va_end(l);
     return c;
 }
 
 gchar *debug_g_strdup_vprintf(char *file, int line, gchar *fmt, va_list args){
-    int len;
-    char *c;
+//  int len;
+    char *c,*d;
 
-    len=g_printf_string_upper_bound(fmt, args);
+/*    len=g_printf_string_upper_bound(fmt, args);
     c=debug_mem_alloc(file, line, len);
     if (c==DUMMY) return NULL;
-    g_vsprintf(c, fmt, args);
+    g_vsprintf(c, fmt, args);*/
+    
+    d = g_strdup_vprintf(fmt, args);
+    c = debug_g_strdup(file, line, d);
+    g_free(d);
     return c;
 }
 
