@@ -3,17 +3,17 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of tuxpaint http://www.newbreedsoftware.com/tuxpaint/
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-#if defined(HAVE_SDL) && defined(HAVE_LIBPNG)
 
 /* getpixel reads data from screen not surf! */
+#if defined HAVE_SDL && defined(HAVE_LIBPNG)
 int do_png_save(SDL_Surface * surf, char * fname)
 {
     png_structp png_ptr;
@@ -24,6 +24,7 @@ int do_png_save(SDL_Surface * surf, char * fname)
     int x, y, count;
     FILE *fi;
 
+	wokna(fname);
     fi=fopen(fname,"wb");
     if (!fi){
         return -1;
@@ -55,12 +56,9 @@ int do_png_save(SDL_Surface * surf, char * fname)
 	    
     png_init_io(png_ptr, fi);
 
-    info_ptr->width = surf->w;
-    info_ptr->height = surf->h;
-    info_ptr->bit_depth = 8;
-    info_ptr->color_type = PNG_COLOR_TYPE_RGB;
-    info_ptr->interlace_type = 1;
-    info_ptr->valid = 0;
+    png_set_IHDR(png_ptr, info_ptr, surf->w, surf->h, 8,
+    	PNG_COLOR_TYPE_RGB, 1, PNG_COMPRESSION_TYPE_DEFAULT, 
+	PNG_FILTER_TYPE_DEFAULT);
 
     /* Set headers */
 
@@ -76,11 +74,11 @@ int do_png_save(SDL_Surface * surf, char * fname)
 
     /* Save the picture: */
 
-    png_rows = malloc(sizeof(char *) * surf->h);
+    png_rows = (unsigned char **)malloc(sizeof(char *) * surf->h);
 
     for (y = 0; y < surf->h; y++)
     {
-        png_rows[y] = malloc(sizeof(char) * 3 * surf->w);
+        png_rows[y] = (unsigned char *)malloc(sizeof(char) * 3 * surf->w);
 
         for (x = 0; x < surf->w; x++)
         {
