@@ -74,21 +74,35 @@ static void g_index_array_maybe_expand (GRealIndexArray * array, gint len);
 
 static GMemChunk *index_array_mem_chunk = NULL;
 
-G_LOCK_DEFINE_STATIC (index_array_mem_chunk);
+//G_LOCK_DEFINE_STATIC (index_array_mem_chunk);
+static GMutex *index_array_mem_chunk_mutex;
+#ifdef LEAK_DEBUG_LIST
+char *index_array_mem_chunk_file;
+int index_array_mem_chunk_line;
+#endif
 
+void init_iarray(void){
+    index_array_mem_chunk_mutex = g_mutex_new();
+}
+
+void free_iarray(void){
+    g_mutex_free(index_array_mem_chunk_mutex);
+}
 
 GIndexArray *g_index_array_new (void)
 {
     GRealIndexArray *array;
 
-    G_LOCK (index_array_mem_chunk);
+    //G_LOCK (index_array_mem_chunk);
+    MUTEX_LOCK (index_array_mem_chunk);
     if (!index_array_mem_chunk)
         index_array_mem_chunk = g_mem_chunk_new ("array mem chunk",
                                                  sizeof (GRealIndexArray),
                                                  1024, G_ALLOC_AND_FREE);
 
     array = g_chunk_new (GRealIndexArray, index_array_mem_chunk);
-    G_UNLOCK (index_array_mem_chunk);
+    //G_UNLOCK (index_array_mem_chunk);
+    MUTEX_UNLOCK (index_array_mem_chunk);
 
     array->pdata = NULL;
     array->len = 0;
@@ -101,12 +115,14 @@ void g_index_array_free (GIndexArray * array, gboolean free_segment)
 {
     g_return_if_fail (array);
 
-    if (free_segment)
+    if (free_segment && array->pdata != NULL)
         g_free (array->pdata);
 
-    G_LOCK (index_array_mem_chunk);
+    //G_LOCK (index_array_mem_chunk);
+    MUTEX_LOCK (index_array_mem_chunk);
     g_mem_chunk_free (index_array_mem_chunk, array);
-    G_UNLOCK (index_array_mem_chunk);
+    //G_UNLOCK (index_array_mem_chunk);
+    MUTEX_UNLOCK (index_array_mem_chunk);
     
 }
 
@@ -242,6 +258,22 @@ void g_index_array_qsort (GIndexArray *farray,
     qsort( (void *)array->pdata, array->len, sizeof(gpointer), compar);
 }
 
+void g_index_array_uniq (GIndexArray *farray, 
+        int (*compar)(const void *, const void *), int free_dups){
+    
+    int i=0;
+
+    while (i + 1 < farray->len){
+        if (compar(farray->pdata+i, farray->pdata+i+1) == 0){
+            if (free_dups) g_free(g_index_array_index(farray, i + 1));
+            g_index_array_remove_index(farray, i + 1);
+        }else{
+            i++;
+        }
+
+    }
+}
+
 gpointer *g_index_array_bsearch(GIndexArray *farray, const void *key,
         int (*compar)(const void *, const void *)){
     
