@@ -45,6 +45,8 @@ struct _GIndexArray
 };
 
 
+void init_iarray(void);
+void free_iarray(void);
 #define     g_index_array_index(array,index) (array->pdata)[index]
 GIndexArray* g_index_array_new (void);
 void      g_index_array_free (GIndexArray *array, gboolean free_seg);
@@ -58,6 +60,7 @@ void      g_index_array_add (GIndexArray *array, gpointer data);
 
 
 void g_index_array_qsort (GIndexArray *farray, int (*compar)(const void *, const void *));
+void g_index_array_uniq (GIndexArray *farray, int (*compar)(const void *, const void *), int free_dups);
 gpointer *g_index_array_bsearch(GIndexArray *farray, const void *key, int (*compar)(const void *, const void *));
 gint g_index_array_bsearch_index(GIndexArray *farray, const void *key, int (*compar)(const void *, const void *));
 
