@@ -3,14 +3,17 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
+#define FREE_Cx if (c1) {mem_free(c1); c1=NULL;}\
+                if (c2) {mem_free(c2); c2=NULL;}\
+                if (c3) {mem_free(c3); c3=NULL;}
 
 void draw_inputln(struct inputln *il, int sel){
 
@@ -29,16 +32,17 @@ void draw_inputln(struct inputln *il, int sel){
 }
 
 void clear_inputline(struct inputln *il){
+    //if (il->band && il->band->readonly) return;
     il->cpos = 0;
     memset(il->cdata,0,il->dlen);
 }
     
-void il_hist_check_bounds(struct inputln *il){
+static void il_hist_check_bounds(struct inputln *il){
     if (il->hist_i < -1 ) il->hist_i = -1;
     if ((signed)il->hist_i >= (signed)il->history->len) il->hist_i = il->history->len - 1;
 }
 
-void il_add_to_history(struct inputln *il){
+static void il_add_to_history(struct inputln *il){
     if (strlen(il->cdata)==0) return;
     g_ptr_array_add(il->history, g_strdup(il->cdata));
     if (il->history->len == IL_HIST_LEN){
@@ -49,38 +53,43 @@ void il_add_to_history(struct inputln *il){
     }
 }
 
-struct inputln *gil;
+static struct inputln *gil;
 
-void il_history_func (union cba_t cba, union cba_t unused){
+void il_history_func (cba_t cba, cba_t unused){
     int active;
     struct inputln *il;
 
-    active=cba.int_;
+    active=(int)GETCBA(cba, int_);
     il=gil;
     
     if (active<0 || active>il->history->len) return;
     strncpy(il->cdata,
-            g_ptr_array_index(il->history, active),
+            (char *)g_ptr_array_index(il->history, active),
             il->dlen);
+    il->cpos = strlen(il->cdata);
+
     redraw_later();
 }
 
 /* vraci 0 v pripade neobslouzeno */
 int inputln_func(struct inputln *il, struct event *ev){
+    static char callkst[EQSO_LEN];
+    int i;
+    struct menu_item *mi;
 
-/*    dbg("inputln_func(%p,[%d,%d,%d,%d])\n",il,ev->ev,ev->x,ev->y,ev->b);*/
+    //dbg("inputln_func(%p,[%d,%d,%d,%d])\n",il,ev->ev,ev->x,ev->y,ev->b);
     
     switch (ev->ev) {
         case EV_INIT:
 /*            dbg("inputln_func EV_INIT\n");*/
             il->x = 20;
             il->y = term->y - cfg->loglines - 1;
-            if (il->band) il->y -= il->band->spypeers->len;
+            if (il->band) il->y -= ctest->spypeers->len;
             
             il->l = 40;
             il->dlen = 256;
             if (!il->cdata)
-               il->cdata=mem_alloc(il->dlen);
+               il->cdata=(char*)mem_alloc(il->dlen);
 
             memset(il->cdata,0,il->dlen);
             il->cpos = strlen(il->cdata);
@@ -99,7 +108,7 @@ int inputln_func(struct inputln *il, struct event *ev){
             }else{
                 il->x = 20;
                 il->y = term->y - cfg->loglines - 1;
-                if (il->band) il->y -= il->band->spypeers->len;
+                if (il->band) il->y -= ctest->spypeers->len;
             }
             /*dbg("inputln_func RESIZE %p term->y=%d il->x=%d il->y=%d il->band=%p\n", il, term->y, il->x, il->y, il->band);*/
         
@@ -111,10 +120,13 @@ int inputln_func(struct inputln *il, struct event *ev){
         case EV_KBD:
             if ((ev->y & KBD_CTRL) &&
                 upcase(ev->x) == 'V' && il->allow_ctrlv){
+                if (il->readonly) break;
                 il->wasctrlv=1;
+                goto dsp_f;
             }else{   
                 if (il->wasctrlv){
                     il->wasctrlv=0;
+                    if (il->readonly) break;
 /*                    dbg("ctrl+%c\n", ev->x); */
                     if (strlen(il->cdata) < il->dlen - 1) {
                         memmove(il->cdata + il->cpos + 1, il->cdata + il->cpos, strlen(il->cdata) - il->cpos + 1);
@@ -129,15 +141,19 @@ int inputln_func(struct inputln *il, struct event *ev){
             }
             switch (kbd_action(KM_EDIT, ev)) {
                 case ACT_RIGHT:
+                    if (il->readonly) break;
                     if (il->cpos < strlen(il->cdata)) il->cpos++;
                     goto dsp_f;
                 case ACT_LEFT:
+                    if (il->readonly) break;
                     if (il->cpos > 0) il->cpos--;
                     goto dsp_f;
                 case ACT_HOME:
+                    if (il->readonly) break;
                     il->cpos = 0;
                     goto dsp_f;
                 case ACT_END:
+                    if (il->readonly) break;
                     il->cpos = strlen(il->cdata);
                     goto dsp_f;
                 case ACT_BACKSPACE:
@@ -145,6 +161,17 @@ int inputln_func(struct inputln *il, struct event *ev){
                     if (il->cpos) {
                         memmove(il->cdata + il->cpos - 1, il->cdata + il->cpos, strlen(il->cdata) - il->cpos + 1);
                         il->cpos--;
+//                        if (aband) dbg("inputln_func: call_played='%s'\n", aband->call_played);
+                        if (il->band && 
+                            il->band->tmpqsos[0].callsign == NULL &&
+                            il->band->call_played == NULL && 
+                            cwda && 
+                            cwda->back && 
+                            cfg->cwda_autgive > 0 &&
+                            get_mode() == MOD_CW_CW){
+
+                            cwda->back(cwda); 
+                        }
                     }
                     goto dsp_f;
                 case ACT_DELETE:
@@ -161,10 +188,53 @@ int inputln_func(struct inputln *il, struct event *ev){
                     memmove(il->cdata, il->cdata + il->cpos, strlen(il->cdata + il->cpos) + 1);
                     il->cpos = 0;
                     goto dsp_f;
-                case ACT_KILL_TO_EOL:
+/*                case ACT_KILL_TO_EOL:
                     if (il->readonly) break;
                     il->cdata[il->cpos] = 0;
                     goto dsp_f;
+                    break;*/
+                case ACT_NEWCALL_KST:
+                    if (il->band) break; // only for subwin inputline
+                    if (il->readonly) break;
+
+                    strcpy(callkst, "");
+                    input_field(NULL, CTEXT(T_CALLSIGN3), CTEXT(T_CALLSIGN_KST) ,
+                               CTEXT(T_OK), CTEXT(T_CANCEL), il, 
+                               NULL, 20, callkst, 0, 0, NULL,
+                               (void (*)(void *, char *)) newkst, NULL, 0);
+                    goto dsp_f;
+                    break;
+                case ACT_LASTCALL_KST:
+                case ACT_KILL_TO_EOL:
+                    if (il->readonly) break;
+                    il->cdata[il->cpos] = 0;
+                    dbg("\ni=%d len=%d\n", il->hist_i, il->history->len);
+                    
+                    if (il->band) break;
+                    if (il->hist_i==-1 && strlen(il->cdata)>0){
+                        il_add_to_history(il);
+                        il->hist_i++;
+                    }
+                    for (il->hist_i++; (signed)il->hist_i < (signed)il->history->len; il->hist_i++){
+                        char *c, *c1=NULL, *c2=NULL, *c3=NULL;
+                        c = (char *)g_ptr_array_index(il->history, il->history->len - il->hist_i - 1);
+                        if (!c) continue; /* probably unreached */
+                        dbg("regmatch('%s')\n", c);
+                        if (regmatch(c, "^/[cC][qQ] ([a-zA-Z0-9/-]{3,14})", &c1, &c2, &c3, NULL)==0){
+                            char s[20];
+                            dbg("c1=%s c2=%s c3=%s\n", c1, c2, c3);
+                            
+                            safe_strncpy(s, c2, sizeof(s));
+                            uc(s);
+                            g_snprintf(il->cdata, il->dlen, "/cq %s ", s);
+                            FREE_Cx;
+                            break;
+                        }
+                        FREE_Cx;
+                    }
+                    il->cpos = strlen(il->cdata);
+                    goto dsp_f;
+                    break;
                 case ACT_PREV_HISTORY:    
                     if (il->readonly) break;
                     if (il->hist_i==-1 && strlen(il->cdata)>0){
@@ -175,7 +245,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                     il_hist_check_bounds(il);
                     if (il->hist_i>=0)
                         strncpy(il->cdata,
-                                g_ptr_array_index(il->history, il->history->len - il->hist_i - 1),
+                                (char *)g_ptr_array_index(il->history, il->history->len - il->hist_i - 1),
                                 il->dlen);
                     else
                         strcpy(il->cdata, ""); /* FIXME */        
@@ -188,7 +258,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                     il_hist_check_bounds(il);
                     if (il->hist_i>=0)
                         strncpy(il->cdata,
-                                g_ptr_array_index(il->history, il->history->len - il->hist_i - 1),
+                                (char *)g_ptr_array_index(il->history, il->history->len - il->hist_i - 1),
                                 il->dlen);
                     else
                         strcpy(il->cdata, ""); /* FIXME */        
@@ -196,24 +266,24 @@ int inputln_func(struct inputln *il, struct event *ev){
                     goto dsp_f;
                     break;
                 case ACT_SHOW_HISTORY:{
+                    if (il->readonly) break;
 
-                    int i;
-                    struct menu_item *mi;
                     
                     gil=il;
                     if (il->history->len==0) break;
                     if (!(mi = new_menu(1))) break;
                     for (i = 0; i < il->history->len; i++) {
-                        union cba_t cba2;
-                        cba2.int_=i;
-                        add_to_menu(&mi, g_ptr_array_index(il->history, i),
+                        cba_t cba2;
+                        SETCBA(cba2, int_, i);
+                        add_to_menu(&mi, (char *)g_ptr_array_index(il->history, i),
                                 "", "", il_history_func, cba2, 0);
                     }
                     set_window_ptr(gses->win,il->x,il->y-2-i);
-                    do_menu_selected(mi, (union cba_t)0, il->history->len-1);
+                    do_menu_selected(mi, (cba_t)0, il->history->len-1);
                     
                     }goto dsp_f;
                 case ACT_COPY_CLIPBOARD:
+                    if (il->readonly) break;
                     /* Copy to clipboard */
                     set_clipboard_text(il->cdata);
                     break;  /* We don't need to redraw */
@@ -234,18 +304,46 @@ int inputln_func(struct inputln *il, struct event *ev){
                     mem_free(clipboard);*/
                     goto dsp_f;
                 }
-/*                case ACT_AUTO_COMPLETE: 
-                      do_tab_compl(term, &il->history, win);
-                    goto dsp_f;*/
                 default:
+                    if (cfg->slashkey && *cfg->slashkey){
+                        if (ev->x == '/') ev->x = *cfg->slashkey;
+                        else if (ev->x == *cfg->slashkey) ev->x = '/';
+                    }
                     if (ev->x >= ' ' && ev->x < 0x100 && !ev->y) {
+                        char c;
                         if (il->readonly) break;
                         if (strlen(il->cdata) < il->dlen - 1) {
                             memmove(il->cdata + il->cpos + 1, il->cdata + il->cpos, strlen(il->cdata) - il->cpos + 1);
-                            if (il->upconvert)
-                                il->cdata[il->cpos++] = upcase(ev->x);
-                            else
-                                il->cdata[il->cpos++] = ev->x;
+                            c = ev->x;
+                            if (il->upconvert) c = upcase(c);
+                            if (!il->valid_chars || strchr(il->valid_chars, c)!=NULL){
+                                il->cdata[il->cpos++] = c;
+                                il->cdata[il->dlen-1] = '\0';
+
+//                                if (il->band) dbg("inputln_func2: call_played='%s' agcall='%d'\n", il->band->call_played, il->band->agcall);
+                                if (il->band && 
+                                    il->band->tmpqsos[0].callsign == NULL && 
+                                    il->band->call_played == NULL && 
+                                    cwda && 
+                                    cwda->text &&
+                                    cfg->cwda_autgive > 0 &&
+                                    get_mode() == MOD_CW_CW){
+
+                                    if (il->cpos == cfg->cwda_autgive){
+                                        cwda->text(cwda, il->cdata);
+                                        il->band->agcall = 1;
+                                        il->band->iscall = 1;
+//                                        dbg("inputln_func2:  agcall='%d' iscall='%d'\n", il->band->agcall, il->band->iscall);
+                                    }
+                                    if (il->cpos > cfg->cwda_autgive){
+                                        char sss[2];
+                                        sss[0]=c;
+                                        sss[1]='\0';
+                                        cwda->text(cwda, sss); 
+                                    }
+                                }
+                                    
+                            }
                         }
                         goto dsp_f;
                     }
@@ -263,7 +361,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                 il_add_to_history(il);
                 il->hist_i=-1;
                 
-                il->enter(il->enterdata, il->cdata);
+                il->enter(il->enterdata, il->cdata, 1);
                 clear_inputline(il);
                 draw_inputln(il,1);
                 return 1;
@@ -303,3 +401,14 @@ void il_readonly(struct inputln *il, int ro){
     else
         strncpy(il->cdata , "", il->dlen);
 }
+
+void newkst(struct inputln *il, char *callkst){
+
+    if (!il || !callkst) return;
+    uc(callkst);
+    //g_snprintf(il->cdata, il->dlen, "/cq %s (%s) ", callkst, callkst);
+    g_snprintf(il->cdata, il->dlen, "/cq %s ", callkst);  // new format since abt. jun 2011
+    il->cpos = strlen(il->cdata);
+
+}
+
