@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -37,80 +37,87 @@ struct itrm {
     char *orig_title;
 };
 
-void free_trm(union cba_t);
-void in_kbd(union cba_t);
-void in_sock(union cba_t);
+void free_trm(cba_t);
+void in_kbd(cba_t);
+void in_sock(cba_t);
 
-struct itrm *ditrm = NULL;
+static struct itrm *ditrm = NULL;
 
-int is_blocked()
+int is_blocked(void)
 {
     return ditrm && ditrm->blocked;
 }
 
-void free_all_itrms()
+void free_all_itrms(void)
 {
-    if (ditrm) free_trm((union cba_t)ditrm);
+    if (ditrm) free_trm((cba_t)ditrm);
 }
 
-void write_ev_queue(struct itrm *itrm)
+static void write_ev_queue(struct itrm *itrm)
 {
     int l;
-    if (!itrm->eqlen) internal("event queue empty");
+    if (!itrm->eqlen) internal_("event queue empty");
     if ((l = write(itrm->sock_out, itrm->ev_queue, itrm->eqlen > 128 ? 128 : itrm->eqlen)) == -1) {
-        free_trm((union cba_t)itrm);
+        free_trm((cba_t)itrm);
         return;
     }
 #ifdef HAVE_SDL    
     if (sdl){
-        LOCK(sdl->eventpipestate);
+        MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate+=l;
 /*        dbg("u %d\n", sdl->eventpipestate);*/
-        UNLOCK(sdl->eventpipestate);
+        MUTEX_UNLOCK(sdl->eventpipestate);
     }
 #endif
     
     memmove(itrm->ev_queue, itrm->ev_queue + l, itrm->eqlen -= l);
-    if (!itrm->eqlen) set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), NULL, get_handler(itrm->sock_out, H_ERROR), (union cba_t)(void*)get_handler(itrm->sock_out, H_DATA));
+    if (!itrm->eqlen) set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), NULL, get_handler(itrm->sock_out, H_ERROR), (cba_t)(void*)get_handler(itrm->sock_out, H_DATA));
 }
 
-void queue_event(struct itrm *itrm, char *data, int len)
+static void queue_event(struct itrm *itrm, char *data, int len)
 {
     int w = 0;
     if (!len) return;
-    if (!itrm->eqlen && can_write(itrm->sock_out) && (w = write(itrm->sock_out, data, len)) <= 0) {
-        /*free_trm(itrm);*/
-        register_bottom_half((void (*)(union cba_t))free_trm, (union cba_t)itrm);
-        return;
+    if (!itrm->eqlen && can_write(itrm->sock_out)){
+#if defined(_MSC_VER) || defined(__MINGW32__)
+		w = send(itrm->sock_out, data, len, 0);
+#else
+		w = write(itrm->sock_out, data, len);
+#endif
+		if (w <= 0) {
+			/*free_trm(itrm);*/
+			register_bottom_half((void (*)(cba_t))free_trm, (cba_t)itrm);
+			return;
+		}
     }
     
 #ifdef HAVE_SDL    
     if (sdl){
-        LOCK(sdl->eventpipestate);
+        MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate+=w;
 /*        dbg("q %d\n", sdl->eventpipestate);    */
-        UNLOCK(sdl->eventpipestate);
+        MUTEX_UNLOCK(sdl->eventpipestate);
     }
 #endif
         
     if (w < len) {
         char *c;
         if (!(c = mem_realloc(itrm->ev_queue, itrm->eqlen + len - w))) {
-            free_trm((union cba_t)itrm);
+            free_trm((cba_t)itrm);
             return;
         }
         itrm->ev_queue = c;
         memcpy(itrm->ev_queue + itrm->eqlen, data + w, len - w);
         itrm->eqlen += len - w;
         set_handlers(itrm->sock_out, 
-                (void (*)(union cba_t))get_handler(itrm->sock_out, H_READ), 
-                (void (*)(union cba_t))write_ev_queue, 
-                (void (*)(union cba_t))free_trm, 
-                (union cba_t)itrm);
+                (void (*)(cba_t))get_handler(itrm->sock_out, H_READ), 
+                (void (*)(cba_t))write_ev_queue, 
+                (void (*)(cba_t))free_trm, 
+                (cba_t)itrm);
     }
 }
 
-void kbd_ctrl_c()
+void kbd_ctrl_c(void)
 {
     /*struct event ev = { EV_KBD, KBD_CTRL_C, 0, 0 };*/
     struct event ev = { EV_KBD, 'C', KBD_CTRL, 0 };
@@ -122,16 +129,16 @@ char *init_seq = "\033[?1000h\033[?47h\0337";
 char *term_seq = "\033[2J\033[?1000l\033[?47l\0338\b \b";
 */
 
-char *init_seq = "\033)0\0337";
-char *init_seq_x_mouse = "\033[?1000h";
-char *init_seq_tw_mouse = "\033[?9h";
-char *term_seq = "\033[2J\0338\r \b";
-char *term_seq_x_mouse = "\033[?1000l";
-char *term_seq_tw_mouse = "\033[?9l";
+static char *init_seq = "\033)0\0337";
+static char *init_seq_x_mouse = "\033[?1000h";
+static char *init_seq_tw_mouse = "\033[?9h";
+static char *term_seq = "\033[2J\0338\r \b";
+static char *term_seq_x_mouse = "\033[?1000l";
+static char *term_seq_tw_mouse = "\033[?9l";
 
 /*char *term_seq = "\033[2J\033[?1000l\0338\b \b";*/
 
-void send_init_sequence(int h,int flags)
+static void send_init_sequence(int h,int flags)
 {
     hard_write(h, init_seq, strlen(init_seq));
     if (flags & USE_TWIN_MOUSE) {
@@ -141,7 +148,7 @@ void send_init_sequence(int h,int flags)
     }
 }
 
-void send_term_sequence(int h,int flags)
+static void send_term_sequence(int h,int flags)
 {
     hard_write(h, term_seq, strlen(term_seq));
     if (flags & USE_TWIN_MOUSE) {
@@ -151,7 +158,7 @@ void send_term_sequence(int h,int flags)
     }
 }
 
-void resize_terminal(union cba_t cba)
+void resize_terminal(cba_t cba)
 {
     struct event ev = { EV_RESIZE, 0, 0, 0 };
     int x, y;
@@ -169,7 +176,7 @@ void resize_terminal(union cba_t cba)
     queue_event(ditrm, (char *)&ev, sizeof(struct event));
 }
 
-int setraw(int fd, struct termios *p)
+static int setraw(int fd, struct termios *p)
 {
     struct termios t;
     memset(&t, 0, sizeof(struct termios));
@@ -220,9 +227,9 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
     itrm->eqlen = 0;
     if (ctl_in >= 0) {
         setraw(ctl_in, &itrm->t);
-        set_handlers(std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
+        set_handlers(std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
     }
-    if (sock_in != std_out) set_handlers(sock_in, in_sock, NULL, free_trm, (union cba_t)itrm);
+    if (sock_in != std_out) set_handlers(sock_in, in_sock, NULL, free_trm, (cba_t)itrm);
     ev.x = x;
     ev.y = y;
     handle_terminal_resize(ctl_in, resize_terminal);
@@ -249,7 +256,7 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
     set_window_title(PACKAGE_NAME " " PACKAGE_VERSION);
 }
 
-void unblock_itrm_x(union cba_t cba)
+void unblock_itrm_x(cba_t cba)
 {
     close_handle(cba);
     if (!ditrm) return;
@@ -262,13 +269,13 @@ int unblock_itrm(int fd)
     struct itrm *itrm = ditrm;
     if (!itrm) return -1;
     /*if (ditrm->sock_out != fd) {
-        internal("unblock_itrm: bad fd: %d", fd);
+        internal_("unblock_itrm: bad fd: %d", fd);
         return -1;
     }*/
     if (itrm->ctl_in >= 0 && setraw(itrm->ctl_in, NULL)) return -1;
     itrm->blocked = 0;
     send_init_sequence(itrm->std_out,itrm->flags);
-    set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
+    set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
     handle_terminal_resize(itrm->ctl_in, resize_terminal);
     unblock_stdin();
     return 0;
@@ -279,7 +286,7 @@ void block_itrm(int fd)
     struct itrm *itrm = ditrm;
     if (!itrm) return;
     /*if (ditrm->sock_out != fd) {
-        internal("block_itrm: bad fd: %d", fd);
+        internal_("block_itrm: bad fd: %d", fd);
         return;
     }*/
     itrm->blocked = 1;
@@ -287,14 +294,14 @@ void block_itrm(int fd)
     unhandle_terminal_resize(itrm->ctl_in);
     send_term_sequence(itrm->std_out,itrm->flags);
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
-    set_handlers(itrm->std_in, NULL, NULL, free_trm, (union cba_t)itrm);
+    set_handlers(itrm->std_in, NULL, NULL, free_trm, (cba_t)itrm);
 }
 
-void free_trm(union cba_t cba)
+void free_trm(cba_t cba)
 {
     struct itrm *itrm;
 
-    itrm=cba.itrm;
+    itrm=(struct itrm*)GETCBA(cba, itrm);
     if (!itrm) return;
     set_window_title(itrm->orig_title);
     if (itrm->orig_title) mem_free(itrm->orig_title), itrm->orig_title = NULL;
@@ -314,13 +321,13 @@ void free_trm(union cba_t cba)
      itrm=NULL;
 }
 
-void itrm_safe_abort(){
+void itrm_safe_abort(void){
     struct itrm *itrm = ditrm;
     if (!itrm) return;
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
 }
 
-void resize_terminal_x(char *text)
+static void resize_terminal_x(char *text)
 {
     int x, y;
     char *p;
@@ -344,13 +351,13 @@ void dispatch_special(char *text)
     }
 }
 
-char buf[OUT_BUF_SIZE];
+static char buf[OUT_BUF_SIZE];
 
 /*#define RD ({ char cc; if (p < c) cc = buf[p++]; else if ((dl = hard_read(itrm->sock_in, &cc, 1)) <= 0) {debug("%d %d", dl, errno);goto fr;} cc; })*/
 /*udefine RD ({ char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; cc; })*/
 #define RD(xx) { char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; xx = cc; }
 
-void in_sock(union cba_t cba)
+void in_sock(cba_t cba)
 {
     char *path, *delete;
     int pl, dl;
@@ -359,10 +366,10 @@ void in_sock(union cba_t cba)
     int c, i, p;
     struct itrm *itrm;
 
-    itrm=cba.itrm;
+    itrm=(struct itrm *)GETCBA(cba, itrm);
     if ((c = read(itrm->sock_in, buf, OUT_BUF_SIZE)) <= 0) {
         fr:
-        free_trm((union cba_t)itrm);
+        free_trm((cba_t)itrm);
         return;
     }
     qwerty:
@@ -419,10 +426,10 @@ void in_sock(union cba_t cba)
         }
         mem_free(param);
         if (fg == 1) {
-            set_handlers(blockh, unblock_itrm_x, NULL, unblock_itrm_x, (union cba_t)blockh);
+            set_handlers(blockh, unblock_itrm_x, NULL, unblock_itrm_x, (cba_t)blockh);
             /*block_itrm(itrm->ctl_in);*/
         } else {
-            set_handlers(blockh, close_handle, NULL, close_handle, (union cba_t)blockh);
+            set_handlers(blockh, close_handle, NULL, close_handle, (cba_t)blockh);
         }
     }
     to_je_ale_hnus:
@@ -435,7 +442,7 @@ void in_sock(union cba_t cba)
 
 int process_queue(struct itrm *);
 
-void kbd_timeout(struct itrm *itrm)
+static void kbd_timeout(struct itrm *itrm)
 {
     struct event ev = {EV_KBD, KBD_ESC, 0, 0};
     itrm->tm = -1;
@@ -444,7 +451,7 @@ void kbd_timeout(struct itrm *itrm)
         return;
     }*/
     if (!itrm->qlen) {
-        internal("timeout on empty queue");
+        internal_("timeout on empty queue");
         return;
     }
     queue_event(itrm, (char *)&ev, sizeof(struct event));
@@ -452,7 +459,7 @@ void kbd_timeout(struct itrm *itrm)
     while (process_queue(itrm)) ;
 }
 
-int get_esc_code(char *str, int len, char *code, int *num, int *el)
+static int get_esc_code(char *str, int len, char *code, int *num, int *el)
 {
     int pos;
     *num = 0;
@@ -471,7 +478,7 @@ struct key {
     int x, y;
 };
 
-struct key os2xtd[256] = {
+static struct key os2xtd[256] = {
 /* 0 */
 {0,0},{0,0},{' ',KBD_CTRL},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{KBD_BS,KBD_ALT},{0,0},
 /* 16 */
@@ -527,7 +534,7 @@ struct key os2xtd[256] = {
 /* 256 */
 };
 
-int xterm_button = -1;
+static int xterm_button = -1;
 
 int process_queue(struct itrm *itrm)
 {
@@ -613,7 +620,7 @@ int process_queue(struct itrm *itrm)
                         if ( ev.x & (1 << 13)) ev.x = 0; /* ev.x |= ~0 << 14; */
                         ev.y = (char)(itrm->kqueue[el+3]) - ' ' - 1 + ((int)((char)(itrm->kqueue[el+4]) - ' ' - 1) << 7);
                         if ( ev.y & (1 << 13)) ev.y = 0; /* ev.y |= ~0 << 14; */
-                        /*dbg("process_queue: %d\n", itrm->kqueue[el]);*/
+                        //dbg("process_queue: %d\n", itrm->kqueue[el]);
                         switch ((itrm->kqueue[el] - ' ') ^ xterm_button) { /* Every event changhes only one bit */
                             case TW_BUTT_LEFT:   ev.b = B_LEFT | ( (xterm_button & TW_BUTT_LEFT) ? B_UP : B_DOWN ); break; 
                             case TW_BUTT_MIDDLE: ev.b = B_MIDDLE | ( (xterm_button & TW_BUTT_MIDDLE) ? B_UP :  B_DOWN ); break;
@@ -624,7 +631,7 @@ int process_queue(struct itrm *itrm)
                         xterm_button = itrm->kqueue[el] - ' ';
                         el += 5;
                     } else {
-                        /*dbg("process_queue: %d\n", itrm->kqueue[el]-' ');*/
+//                        dbg("process_queue: %d\n", itrm->kqueue[el]-' ');
                         ev.x = itrm->kqueue[el+1] - ' ' - 1;
                         ev.y = itrm->kqueue[el+2] - ' ' - 1;
                         ev.b = B_DOWN;
@@ -702,7 +709,7 @@ int process_queue(struct itrm *itrm)
     }
     l1:
     if (itrm->qlen < el) {
-        internal("event queue underflow");
+        internal_("event queue underflow");
         itrm->qlen = el;
     }
     if (ev.x != -1) {
@@ -713,27 +720,27 @@ int process_queue(struct itrm *itrm)
         memmove(itrm->kqueue, itrm->kqueue + el, itrm->qlen -= el);
     }
     end:
-    if (itrm->qlen < IN_BUF_SIZE) set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
+    if (itrm->qlen < IN_BUF_SIZE) set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
     return el;
     ret:
-    itrm->tm = install_timer(ESC_TIMEOUT, (void (*)(union cba_t))kbd_timeout, (union cba_t)itrm);
+    itrm->tm = install_timer(ESC_TIMEOUT, (void (*)(cba_t))kbd_timeout, (cba_t)itrm);
     return 0;
 }
 
-void in_kbd(union cba_t cba)
+void in_kbd(cba_t cba)
 {
     int r;
     struct itrm *itrm;
 
-    itrm=cba.itrm;
+    itrm=(struct itrm *)GETCBA(cba, itrm);
     if (itrm->tm != -1) kill_timer(itrm->tm), itrm->tm = -1;
     if (itrm->qlen >= IN_BUF_SIZE) {
-        set_handlers(itrm->std_in, NULL, NULL, free_trm, (union cba_t)itrm);
+        set_handlers(itrm->std_in, NULL, NULL, free_trm, (cba_t)itrm);
         while (process_queue(itrm));
         return;
     }
     if ((r = read(itrm->std_in, itrm->kqueue + itrm->qlen, IN_BUF_SIZE - itrm->qlen)) <= 0) {
-        free_trm((union cba_t)itrm);
+        free_trm((cba_t)itrm);
         return;
     }
     if ((itrm->qlen += r) > IN_BUF_SIZE) {
