@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -22,7 +22,7 @@ struct keybinding {
     long y;
 };
 
-static struct list_head keymaps[KM_MAX];
+static struct keybinding keymaps[KM_MAX];
 
 static void add_keybinding(int km, int act, long x, long y)
 {
@@ -45,17 +45,23 @@ static void add_keybinding(int km, int act, long x, long y)
 }
 
 
-void init_keymaps()
+void init_keymaps(void)
 {
         int i;  
     for (i = 0; i < KM_MAX; i++) init_list(keymaps[i]);
     add_default_keybindings();
 }
 
-void free_keymaps() 
+void free_keymaps(void) 
 {
     int i;
-    for (i = 0; i < KM_MAX; i++) free_list(keymaps[i]);
+    for (i = 0; i < KM_MAX; i++) {
+        while (keymaps[i].next != &(keymaps[i])) {
+            struct keybinding *a=(keymaps[i]).next; 
+            del_from_list(a); 
+            mem_free(a); 
+        }
+    }
 }
 
 int kbd_action(int kmap, struct event *ev)
@@ -64,7 +70,7 @@ int kbd_action(int kmap, struct event *ev)
     if (ev->ev == EV_KBD)
         foreach(kb, keymaps[kmap]){
             if (ev->x == kb->x && ev->y == kb->y) {
-        /*        dbg("kbd_action=%d\n",kb->act);*/
+            //    dbg("kbd_action=%d\n",kb->act);
                 return kb->act;
             }
             if (kb->y & (KBD_CTRL|KBD_ALT)){
@@ -89,11 +95,14 @@ struct default_kb {
 /*  !!!! CTRL needs uppercase, ALT lowercase */
 
 static struct default_kb default_main_keymap[] = {
+    { ACT_BACKSPACE, KBD_BS, 0 },
+    { ACT_BACKSPACE, KBD_BS, KBD_SHIFT },
     { ACT_CALLINFO, 'i', KBD_ALT },
     { ACT_CHOP, 'o', KBD_ALT },
     { ACT_CLEAR_TMPQSOS, 'y', KBD_ALT },
     { ACT_CLEAR_TMPQSOS_INPUTLINE, KBD_F3, 0 },
     { ACT_CONFIRM_CALL, 'd', KBD_ALT },
+    { ACT_CONFIRM_EXC, 'e', KBD_ALT },
     { ACT_CONFIRM_WWL, 'x', KBD_ALT },
     { ACT_COPY_CLIPBOARD, 'C', KBD_CTRL },
     { ACT_COPY_CLIPBOARD, KBD_INS, KBD_CTRL },
@@ -103,6 +112,7 @@ static struct default_kb default_main_keymap[] = {
     { ACT_CQ_3, KBD_F8, 0 },
     { ACT_CQ_4, KBD_F11, 0 },
     { ACT_CQ_5, KBD_F12, 0 },
+    { ACT_DELETE, KBD_DEL, 0 },
     { ACT_DOWN, KBD_DOWN, 0 },
     { ACT_END, KBD_END, 0 },
     { ACT_ENTER, KBD_ENTER, 0 },
@@ -112,6 +122,7 @@ static struct default_kb default_main_keymap[] = {
     { ACT_FIND_NEXT, 'n', 0 },
     { ACT_GRAB_BAND, 'g', KBD_ALT },
     { ACT_HOME, KBD_HOME, 0 },
+    { ACT_INSERT, KBD_INS, 0 },
     { ACT_LEFT, KBD_LEFT, 0 },
     { ACT_MENU, KBD_F9, 0 },
     { ACT_MODE, 'm', KBD_ALT },
@@ -142,6 +153,7 @@ static struct default_kb default_main_keymap[] = {
     { ACT_UNFINISHED, 'u', KBD_ALT },
     { ACT_UP, KBD_UP, 0 },
     { ACT_WINDOWSHOT, KBD_F1, KBD_ALT },
+    { ACT_AC_CQ, 'j', KBD_ALT },
     { 0, 0, 0 }
 };
 
@@ -159,6 +171,8 @@ static struct default_kb default_edit_keymap[] = {
     { ACT_PREV_HISTORY, 'p', KBD_ALT },
     { ACT_NEXT_HISTORY, 'n', KBD_ALT },
     { ACT_SHOW_HISTORY, 'h', KBD_ALT },
+    { ACT_LASTCALL_KST, 'k', KBD_ALT },
+    { ACT_NEWCALL_KST, 'K', KBD_CTRL },
 /* ACT_KILL_TO_BOL, 'U', KBD_CTRL },
     { ACT_KILL_TO_EOL, 'K', KBD_CTRL },*/
     { 0, 0, 0 }
