@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -24,12 +24,12 @@ char dummyarray[T__N_TEXTS];
 
 #include "language.inc"
 
-char **translation_array[N_LANGUAGES][N_CODEPAGES];
+static char **translation_array[N_LANGUAGES][N_CODEPAGES];
 
 int current_language;
-int current_lang_charset;
+static int current_lang_charset;
 
-void init_trans()
+void init_trans(void)
 {
     int i, j;
     for (i = 0; i < N_LANGUAGES; i++)
@@ -40,7 +40,7 @@ void init_trans()
     translations[0].t=translation_english;
 }
 
-void shutdown_trans()
+void shutdown_trans(void)
 {
     int i, j, k;
     for (i = 0; i < N_LANGUAGES; i++)
@@ -81,7 +81,7 @@ char *get_english_translation(char *text)
     return translation_english[text - dummyarray].name;
 }
 
-int n_languages()
+int n_languages(void)
 {
     return N_LANGUAGES;
 }
@@ -95,17 +95,29 @@ void set_language(int l)
 {
     int i;
     char *cp;
-    for (i = 0; i < T__N_TEXTS; i++) if (translations[l].t[i].code != i) {
-        printf("  i=%d ;l=%d; tr[..]=%d \n", i,l,translations[l].t[i].code);
-        internal("Bad table for language %s. Run script synclang.", translations[l].t[T__LANGUAGE].name);
-        return;
+    dbg("set_language charset=%d language=%d\n", current_lang_charset, current_language);
+    for (i = 0; i < T__N_TEXTS; i++){
+    //    dbg("  i=%d ;l=%d; tr[..]=%d \n", i,l,translations[l].t[i].code);
+        if (translations[l].t[i].code != i) {
+            internal_("Bad table for language %s (%d!=%d). Run script fixlang.pl or maybe add this language into this script.", 
+                    translations[l].t[T__LANGUAGE].name,
+                    translations[l].t[i].code, i);
+            return;
+        }
     }
     current_language = l;
     cp = translations[l].t[T__CHAR_SET].name;
+    dbg("codepage=%s\n", cp);
     i = get_cp_index(cp);
     if (i == -1) {
-        internal("Unknown charset for language %s.", translations[l].t[T__LANGUAGE].name);
+        internal_("Unknown charset for language %s.", translations[l].t[T__LANGUAGE].name);
         i = 0;
     }
     current_lang_charset = i;
+    dbg("return charset=%d language=%d\n", current_lang_charset, current_language);
+#ifdef HAVE_SDL
+    if (sdl && term && term->spec){
+        term->spec->charset = i;
+    }
+#endif
 }
