@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -22,18 +22,18 @@
 
 #define LOCDEBUGx
 
-char *tostr(char *s,double x){
+static char *tostr(char *s,double x){
     int a,b;
 
     x=x*180/MY_PI;
     /*fprintf(stderr, "%6.2f\n",x);*/
-    a=x;
+    a=(int)x;
     if (x>0)
         x-=a;
     else
         x=a-x;
     x*=60;
-    b=x;
+    b=(int)x;
     if (x>0)
         x-=b;
     else
@@ -52,7 +52,7 @@ double qth(char *qth,int width) {
     char *c,d;
     double res;
     
-    /*dbg("qth('%s',%d)\n", qth, width);*/
+   /* dbg("qth('%s',%d)\n", qth, width);*/
     c=qth;
     while (1){
         d=lowcase(*c);
@@ -72,7 +72,7 @@ double qth(char *qth,int width) {
     res+=(*c-'0')*MY_PI/90.0;
     c+=2;
     
-    /*dbg("strlen(c)=%d, strlen(qth)=%d\n", strlen(c), strlen(qth));  */
+/*   dbg("strlen(c)=%d, strlen(qth)=%d\n", strlen(c), strlen(qth));  */
     
     if (strlen(qth)>=6){
         d=lowcase(*c);
@@ -116,6 +116,7 @@ int qrbqtf(char *myqth,char *recqth,double *qrb,double *qtf,char *str, int flags
     /*dbg("h1=%7.4f w1=%7.4f h2=%7.4f  w2=%7.4f \n",h1,w1,h2,w2);*/
 #ifdef LOCDEBUG    
     fprintf(stderr, "%s->%s, h1=%7.9f w1=%7.9f h2=%7.9f  w2=%7.9f \n",myqth,recqth,h1,w1,h2,w2);
+    fprintf(stderr, "h1=%7.9fdeg w1=%7.9fdeg h2=%7.9fdeg  w2=%7.9fdeg \n",h1*180.0/MY_PI,w1*180.0/MY_PI,h2*180.0/MY_PI,w2*180.0/MY_PI);
 #endif    
 
     if (str!=NULL){
@@ -243,18 +244,18 @@ char *hw2loc(char *buf, double h, double w){
     h=fmod(h,360.0);
     w=fmod(w,180.0);
 
-    buf[0]='A'+ h/20.0;
-    buf[1]='A'+ w/10.0;
+    buf[0]='A'+ (int)(h/20.0);
+    buf[1]='A'+ (int)(w/10.0);
     h=fmod(h,20);
     w=fmod(w,10);
-    buf[2]='0'+ h/2;
-    buf[3]='0'+ w;
+    buf[2]='0'+ (int)(h/2);
+    buf[3]='0'+ (int)(w);
     h=fmod(h,2);
     w=fmod(w,1);
     h*=12;
     w*=24;
-    buf[4]='A'+ h;
-    buf[5]='A'+ w;
+    buf[4]='A'+ (int)(h);
+    buf[5]='A'+ (int)(w);
     buf[6]='\0';
    
     return buf;
@@ -314,14 +315,14 @@ char *compute_wwl4(char *s, double h, double w){
     h += 180.0;
     w +=  90.0;
     
-    hi = h/20;
+    hi = (int)(h/20);
     s[0] = 'A' + hi;
 
-    wi = w/10;
+    wi = (int)(w/10);
     s[1] = 'A' + wi;
         
-    hi = h/2; hi %= 10;
-    wi = w;   wi %= 10;
+    hi = (int)(h/2); hi %= 10;
+    wi = (int)w;     wi %= 10;
     
     s[2] = '0' + hi;
     s[3] = '0' + wi;
@@ -334,8 +335,8 @@ void qrb_qtf_int(gchar *wwl, int *qrb_int, int *qtf_int){
     double qrb, qtf;
 
     qrbqtf(ctest ? ctest->pwwlo : cfg->pwwlo, wwl, &qrb, &qtf, NULL, 0); 
-    *qrb_int=qrb;
-    *qtf_int=qtf;
+    *qrb_int=(int)qrb;
+    *qtf_int=(int)qtf;
 	/*dbg("qrb_qtf_int(%s->%s %5dkm %3ddeg\n", ctest->pwwlo, wwl, *qrb_int, *qtf_int);*/
 }
 #endif
