@@ -3,31 +3,28 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-#include <sched.h>
-
-
 
 int retval = RET_OK;
-long int starting_sbrk;
+char * starting_sbrk;
 
 
     
-void sig_terminate(union cba_t cba)
+void sig_terminate(cba_t cba)
 {
     unhandle_basic_signals();
     terminate = 1;
     retval = RET_SIGNAL;
 }
 
-void sig_intr(union cba_t cba)
+void sig_intr(cba_t cba)
 {              
     if (!term) {
         unhandle_basic_signals();
@@ -38,17 +35,17 @@ void sig_intr(union cba_t cba)
     }
 }
 
-void sig_ctrl_c(union cba_t cba)
+void sig_ctrl_c(cba_t cba)
 {
     dbg("sig_ctrl_c\n");
     if (!is_blocked()) kbd_ctrl_c();
 }
 
-void sig_ign(union cba_t cba)
+void sig_ign(cba_t cba)
 {
 }
 
-void sig_tstp(union cba_t cba)
+void sig_tstp(cba_t cba)
 {              
 #ifdef SIGSTOP
     int pid = getpid();
@@ -64,13 +61,14 @@ void sig_tstp(union cba_t cba)
 #endif
 }
 
-void sig_cont(union cba_t cba)
+void sig_cont(cba_t cba)
 {
     if (!unblock_itrm(0)) resize_terminal(CBA0);
 }
 
-void sig_segv(union cba_t cba){
+void sig_segv(cba_t cba){
     signal(SIGSEGV, SIG_DFL);
+    fprintf(stderr, "sig_segv(%d)", (int)GETCBA(cba, int_));
 /*    printf("\n\n\n  sig_segv() sdl=%p \n\n\n",sdl);*/
 /*  install_signal_handler(SIGSEGV, NULL, CBA0, 0);*/
     cwdaemon_safe_abort(cwda);
@@ -86,8 +84,10 @@ void sig_segv(union cba_t cba){
     raise(SIGSEGV);
 }   
 
-void sig_pipe(int a){
-    dbg("sig_pipe(%d)\n", a);
+static void sig_pipe(int a){
+   /* dbg("sig_pipe(%d)\n", a);
+    dbg("pid=%d\n", getpid());
+    sleep(10);*/
 }
     
 void handle_basic_signals(struct terminal *term)
@@ -116,7 +116,7 @@ void handle_basic_signals(struct terminal *term)
 	signal(SIGPIPE, sig_pipe);
 }
 
-void handle_sdl_signals(void){
+static void handle_sdl_signals(void){
     install_signal_handler(SIGSEGV, sig_segv, CBA0, 1);
 }
 
@@ -168,9 +168,10 @@ void unhandle_basic_signals()
 
 int term_attach_terminal(int in, int out, int ctl)
 {
+    int fd[2];
+
     /*struct terminal *term;*/
     dbg("attach_terminal\n");
-    int fd[2];
     if (c_pipe(fd)) {
         error("ERROR: can't create pipe for internal communication");
         return -1;
@@ -183,46 +184,27 @@ int term_attach_terminal(int in, int out, int ctl)
         return fd[1];
     }
 
-    close(fd[0]);
-    close(fd[1]);
+    closesocket(fd[0]);
+    closesocket(fd[1]);
     return -1;
 }
 
 /*struct status dump_stat;*/
-int dump_pos;
+//static int dump_pos;
 
-int ac;
-char **av;
+static int ac;
+static char **av;
 
 char *path_to_exe;
 
 /*int init_b = 0;*/
 
-char *tucnak_home = NULL;
+static char *tucnak_home = NULL;
 int first_use = 0;
 int first_contest_def = 0;
 
-/*void get_system_name(void)
-{
-    FILE *f;
-    char *p;
-    memset(system_name, 0, MAX_STR_LEN);
-    if (!(f = popen("uname -srm", "r"))) goto fail;
-    if (fread(system_name, 1, MAX_STR_LEN - 1, f) <= 0) {
-        pclose(f);
-        goto fail;
-    }
-    pclose(f);
-    for (p = system_name; *p; p++) if (*p < ' ') {
-        *p = 0;
-        break;
-    }
-    if (system_name[0]) return;
-    fail:
-    strcpy(system_name, SYSTEM_NAME);
-} */
 
-char *get_home(int *n)
+static char *get_home(int *n)
 {
     struct stat st;
     char *home = stracpy(getenv("HOME"));
@@ -263,7 +245,7 @@ char *get_home(int *n)
         mem_free(config_dir);
     } else add_to_strn(&home_tucnak, "tucnak");
     if (stat(home_tucnak, &st)) {
-        if (!mkdir(home_tucnak, 0777)) goto home_creat;
+        if (!tmkdir(home_tucnak, 0777)) goto home_creat;
         if (config_dir) goto failed;
         goto first_failed;
     }
@@ -273,7 +255,7 @@ char *get_home(int *n)
     home_tucnak = stracpy(home);
     add_to_strn(&home_tucnak, "tucnak");
     if (stat(home_tucnak, &st)) {
-        if (!mkdir(home_tucnak, 0777)) goto home_creat;
+        if (!tmkdir(home_tucnak, 0777)) goto home_creat;
         goto failed;
     }
     if (S_ISDIR(st.st_mode)) goto home_ok;
@@ -293,7 +275,7 @@ char *get_home(int *n)
     return home_tucnak;
 }
 
-void init_home(void)
+static void init_home(void)
 {
     tucnak_home = get_home(&first_use);
     if (!tucnak_home) {
@@ -303,10 +285,21 @@ void init_home(void)
     }
 }
 
-void init(void)
+//#define INIT(cmd) dbg("%s...\n", #cmd); cmd;
+#define INIT(cmd) cmd
+
+static void init(void)
 {
+
     parse_options(ac, av);
     if (opt_i){
+        printf("\n  %s %s\n\n", PACKAGE, VERSION);
+#ifdef __GLIBC__
+        printf("\n  glibc info\n");
+        printf("compiled version: %d.%d\n", __GLIBC__, __GLIBC_MINOR__);
+        printf("runtime version: %s\n", gnu_get_libc_version());
+        printf("runtime release: %s\n\n", gnu_get_libc_release());
+#endif
 #ifdef HAVE_SDL        
         sdl_info();
 #endif
@@ -314,33 +307,94 @@ void init(void)
         parport_info();
 #endif        
 #ifdef HAVE_ALSA        
+#ifdef HAVE_SNDFILE
         alsa_info();
+#endif
 #endif        
 #ifdef HAVE_LIBFTDI
         usb_info();
 #endif
+        iface_info();
+        exit(0);
+    }
+    if (opt_s){
+        printf("%s", txt_settings);
+        printf("    macros: ");
+#ifdef UNIX
+        printf("UNIX ");  
+#endif
+#ifdef _UNIX
+        printf("_UNIX ");  
+#endif
+#ifdef __UNIX
+        printf("__UNIX ");  
+#endif
+#ifdef LINUX
+        printf("LINUX ");
+#endif
+#ifdef _LINUX
+        printf("_LINUX ");
+#endif
+#ifdef __LINUX
+        printf("__LINUX ");
+#endif
+#ifdef WIN32
+        printf("WIN32 ");
+#endif
+#ifdef _WIN32
+        printf("_WIN32 ");
+#endif
+#ifdef _WIN64
+        printf("_WIN64 ");
+#endif
+#ifdef __CYGWIN__
+        printf("__CYGWIN__ ");
+#endif
+#ifdef __GNUC__
+        printf("__GNUC__=%d.%d.%d ", __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__);
+#endif
+#ifdef _MSC_VER
+        printf("_MSC_VER=%d.%02d ", _MSC_VER / 100, _MSC_VER % 100);
+#endif
+#ifdef __MINGW32__
+        printf("__MINGW32__ ");
+#endif
+        printf("\n");
         exit(0);
     }
         
     g_thread_init(NULL);
-    init_debug();
+#ifdef __MINGW32__
+    init_mingw();
+#endif
+    INIT(init_debug());
 
-    init_trans();
-    init_rc();
-    term_spec_init();
+    /*char *ccc = mem_alloc(8);
+    strcpy(ccc, "0123456");
+    ccc = mem_realloc(ccc, 16);
+    mem_free(ccc);
+    exit(0);*/
+
+    INIT(init_iarray());
+    INIT(init_ghash());
+
+    INIT(init_trans());
+    INIT(init_rc());
+    INIT(term_spec_init());
 	
-    set_sigcld();
-    init_home();
-    init_keymaps();
-    tpipe=init_threadpipe();
+    INIT(set_sigcld());
+    INIT(init_home());
+    INIT(init_keymaps());
+    INIT(tpipe=init_threadpipe());
 	
-	read_rc_files();
+	INIT(read_rc_files());
     	
 #ifdef HAVE_SDL    
-    sdl=init_sdl();
+    INIT(sdl=init_sdl());
     if (sdl) handle_sdl_signals(); 
 
     if (sdl){
+        setenv("TERM", "sdl", 1);
         if (!sdl_attach_terminal(get_input_handle(), get_output_handle(), get_ctl_handle())==-1){
             retval = RET_FATAL;
             terminate = 1;
@@ -354,167 +408,236 @@ void init(void)
             terminate = 1;
         }
     }
-    if (!term) raise(SIGSEGV);
+    if (!term) internal_("init() !term");
         
     
-	glog = init_fifo(1000);        
-    gtalk = init_fifo(1000);        
-    gsked = init_fifo(1000);        
+	INIT(glog = init_fifo(1000));        
+    INIT(gtalk = init_fifo(1000));        
 
     
 
     
-    cw = init_cw();
-    dw = init_dw();
-    wizz = init_wizz();
-	namedb = init_namedb();
-    sdevlist = init_sdevlist();
-    init_rotars();
+    INIT(cw = init_cw());
+    INIT(dw = init_dw());
+    INIT(excdb = init_exc());
+    INIT(qrvdb = init_qrv());
+    INIT(wizz = init_wizz());
+	INIT(namedb = init_namedb());
+	INIT(masterdb = init_masterdb());
+    INIT(init_sconns());
+    INIT(init_rotars());
+#ifdef HAVE_LIBFTDI
+	hdkeyb = init_hdkeyb(); // after init_rotars
+#endif
 #ifdef HAVE_SDL    
-    cor = init_cor();
+    INIT(cor = init_cor());
 #endif
 
-    disable_screensaver();
+    INIT(disable_screensaver());
     
-    read_cw_files(cw);
-    read_dw_files(dw);
-    read_wizz_files(wizz);
-	read_namedb_files(namedb);
+    INIT(read_cw_files(cw));
+    INIT(read_dw_files(dw));
+    INIT(read_wizz_files(wizz));
+	INIT(read_namedb_files(namedb));
 	/*read_ebw_files(cw,namedb);*/
-#ifdef HAVE_SDL    
-    read_cor_files();
+	INIT(read_masterdb_files(masterdb));
+    
+    INIT(init_sound());
+    INIT(net = init_net());
+	INIT(httpd = init_httpd());
+    INIT(cwda = init_cwdaemon());
+#ifdef HAVE_SNDFILE
+    INIT(dsp = init_dsp());
+    INIT(ssbd = init_ssbd());
 #endif
-    
-    init_sound();
-    net = init_net();
-    cwda = init_cwdaemon();
-    dsp = init_dsp();
-    ssbd = init_ssbd();
 
-    spotdb = init_spotdb();
-    
-    
+    INIT(spotdb = init_spotdb());
+#ifndef __CYGWIN__
+    INIT(ntpq = init_ntpq());        
+#endif
+#ifdef HAVE_HAMLIB
+    INIT(trig = init_trig());        
+#endif
+	INIT(fft = init_fft());
 }
 
-void terminate_all_subsystems(void)
+#define FREE(cmd) dbg("%s...\n", #cmd); cmd;
+//#define FREE(cmd) cmd
+
+static void terminate_all_subsystems(void)
 {
 #ifdef HAVE_SDL
     /*free_gfx();*/
 #endif
     dbg("terminate_all_subsystems\n");
-    check_bottom_halves();
-    check_bottom_halves();
-	check_bottom_halves();
+    FREE(check_bottom_halves());
+    FREE(check_bottom_halves());
+	FREE(check_bottom_halves());
 
 
 #ifdef HAVE_SDL    
-    sdl_stop_event_thread();
-    free_cor(cor);
+    FREE(sdl_stop_event_thread());
+    FREE(free_cor(cor));
 #endif
-    free_dw(dw);
-    free_cw(cw);
-    free_wizz(wizz);
-	free_namedb(namedb);
-    free_rotars();
-	free_sdevlist(sdevlist);
-    free_spotdb(spotdb);
-    free_net(net);
-    free_cwdaemon(cwda);
-    free_ssbd(ssbd);
-    free_dsp(dsp);
-    free_sound();
-	free_threadpipe(tpipe);
+#ifdef HAVE_HAMLIB
+    FREE(free_trig(trig)); 
+#endif
+#ifndef __CYGWIN__
+    FREE(free_ntpq(ntpq)); 
+#endif
+    FREE(free_qrv(qrvdb));
+    FREE(free_exc(excdb));
+    FREE(free_dw(dw));
+    FREE(free_cw(cw));
+    FREE(free_wizz(wizz));
+	FREE(free_namedb(namedb));
+	FREE(free_masterdb(masterdb));
+#ifdef HAVE_LIBFTDI
+    free_hdkeyb(hdkeyb);
+#endif
+    FREE(free_rotars());
+    FREE(free_spotdb(spotdb));
+	FREE(free_httpd(httpd));
+    FREE(free_net(net));
+    FREE(free_cwdaemon(cwda); cwda = NULL);
+#ifdef HAVE_SNDFILE
+    FREE(free_ssbd(ssbd); ssbd = NULL);
+    FREE(free_dsp(dsp));
+#endif
+    FREE(free_sound());
+	FREE(free_fft(fft));  // after ssb
+    FREE(free_threadpipe(tpipe));
     
-    free_fifo(glog);
-    free_fifo(gtalk);
-    free_fifo(gsked);
+    FREE(free_fifo(glog));
+    FREE(free_fifo(gtalk));
 	
 
-    free_namelist();
-    free_all_itrms();
-    shrink_memory(1);
+    FREE(free_namelist());
+    FREE(free_all_itrms());
+    FREE(shrink_memory(1));
 
-    free_history_lists();
-    free_term_specs();
-    free_keymaps();
-    free_conv_table();
-    check_bottom_halves();
-    check_bottom_halves();
-    destroy_terminal(CBA0);
+    FREE(free_history_lists());
+    FREE(free_term_specs());
+    FREE(free_keymaps());
+    FREE(free_conv_table());
+    FREE(check_bottom_halves());
+    FREE(check_bottom_halves());
+    FREE(destroy_terminal(CBA0));
 #ifdef HAVE_SDL    
-    free_sdl();
+    FREE(free_sdl());
 #endif
 
     if (tucnak_home) mem_free(tucnak_home);
 
-    shutdown_trans();
-    terminate_osdep();
-    free_rc();
-    check_memory_leaks();
-    free_debug();
+    FREE(shutdown_trans());
+    FREE(terminate_osdep());
+    FREE(free_rc());
+    FREE(check_memory_leaks());
+    FREE(free_ghash());
+    FREE(free_iarray());
+    FREE(free_debug());
 }
 
-#if 0
-gpointer func(gpointer data){
-   raise(SIGSEGV); 
-}
-#endif
 
 int main(int argc, char *argv[])
 {
+#if 0
+	GString *gs = g_string_new("");
+	zg_string_eprintfa("", gs, "%b", "A", 1);
+	g_string_assign(gs, "");
+	zg_string_eprintfa("", gs, "%b", "AA", 2);
+	g_string_assign(gs, "");
+	zg_string_eprintfa("", gs, "%b", "AAA", 3);
+	g_string_assign(gs, "");
+	zg_string_eprintfa("", gs, "%b", "AAAA", 4);
+
+#endif
+#if 0
+    char s[20], t[20];
+    strcpy(s, "  abc  "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, "  abc"); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, "abc  "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, " abc "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, " abc"); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, "abc "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, "abc"); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, "  "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, " "); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    strcpy(s, ""); strcpy(t, s); printf("'%s'   '%s'\n", s, trim(t)); 
+    printf("'%s'   '%s'\n", NULL, trim(NULL)); 
+    exit(0);
+#endif
+
+#if 0
+	char str[256];
+	printf("%s\n", sunrisesetstr(str, 50.1, -100));
+	printf("%s\n", sunrisesetstr(str, 50.1, 0));
+	printf("%s\n", sunrisesetstr(str, 50.1, 100));
+	return 0;
+#endif
 #if 0
     g_thread_init(NULL);
     debug_type=2;
     init_debug();
+    int thr=2;
     ST_START;
-    similar_calls("OK2M", "OK1ZIA",0,1);
-    similar_calls("OK2M", "OK1KRQ",0,1);
+    similar_calls("OK2M", "OK1ZIA",0,thr,1);
+    similar_calls("OK2M", "OK1KRQ",0,thr,1);
 
-    similar_calls("OK2M", "OK2",0,1);
-    similar_calls("OK2M", "OKM",0,1);
-    similar_calls("OK2M", "O2M",0,1);
-    similar_calls("OK2M", "K2M",0,1);
+    similar_calls("OK2M", "OK2",0,thr,1);
+    similar_calls("OK2M", "OKM",0,thr,1);
+    similar_calls("OK2M", "O2M",0,thr,1);
+    similar_calls("OK2M", "K2M",0,thr,1);
     
-    similar_calls("OK2M", "OK2X",0,1);
-    similar_calls("OK2M", "OKXM",0,1);
-    similar_calls("OK2M", "OX2M",0,1);
-    similar_calls("OK2M", "XK2M",0,1);
+    similar_calls("OK2M", "KO2M",0,thr,1);
     
-    similar_calls("OK2M", "OK2MX",0,1);
-    similar_calls("OK2M", "OK2XM",0,1);
-    similar_calls("OK2M", "OKX2M",0,1);
-    similar_calls("OK2M", "OXK2M",0,1);
-    similar_calls("OK2M", "XOK2M",0,1);
+    similar_calls("OK2M", "OK2X",0,thr,1);
+    similar_calls("OK2M", "OKXM",0,thr,1);
+    similar_calls("OK2M", "OX2M",0,thr,1);
+    similar_calls("OK2M", "XK2M",0,thr,1);
     
-    similar_calls("OK2M", "DK2M",0,1);
-    similar_calls("OK2M", "DK3M",0,1);
-    similar_calls("OK2M", "DK3O",0,1);
-    similar_calls("OK2M", "DL2M",0,1);
-    similar_calls("OK2M", "DL3M",0,1);
-    similar_calls("OK2M", "DL3O",0,1);
+    similar_calls("OK2M", "OK2MX",0,thr,1);
+    similar_calls("OK2M", "OK2XM",0,thr,1);
+    similar_calls("OK2M", "OKX2M",0,thr,1);
+    similar_calls("OK2M", "OXK2M",0,thr,1);
+    similar_calls("OK2M", "XOK2M",0,thr,1);
     
-    similar_calls("OK1KRQ", "OK1ZIA",0,1);
+    similar_calls("OK2M", "DK2M",0,thr,1);
+    similar_calls("OK2M", "DK3M",0,thr,1);
+    similar_calls("OK2M", "DK3O",0,thr,1);
+    similar_calls("OK2M", "DL2M",0,thr,1);
+    similar_calls("OK2M", "DL3M",0,thr,1);
+    similar_calls("OK2M", "DL3O",0,thr,1);
+    
+    similar_calls("OK1KRQ", "OK1ZIA",0,thr,1);
 
-    similar_calls("OK1KRQ", "OK1KRQ",0,1);
-    similar_calls("OK1KRQ", "OK1KR",0,1);
-    similar_calls("OK1KRQ", "OK1KQ",0,1);
-    similar_calls("OK1KRQ", "OK1RQ",0,1);
-    similar_calls("OK1KRQ", "OKKRQ",0,1);
-    similar_calls("OK1KRQ", "K1KRQ",0,1);
+    similar_calls("OK1KRQ", "OK1KRQ",0,thr,1);
+    similar_calls("OK1KRQ", "OK1KR",0,thr,1);
+    similar_calls("OK1KRQ", "OK1KQ",0,thr,1);
+    similar_calls("OK1KRQ", "OK1RQ",0,thr,1);
+    similar_calls("OK1KRQ", "OKKRQ",0,thr,1);
+    similar_calls("OK1KRQ", "K1KRQ",0,thr,1);
     
-    similar_calls("OK1KRQ", "OK1KQR",0,1);
-    similar_calls("OK1KRQ", "OK1RKQ",0,1);
-    similar_calls("OK1KRQ", "OKK1RQ",0,1);
-    similar_calls("OK1KRQ", "O1KKRQ",0,1);
-    similar_calls("OK1KRQ", "KO1KRQ",0,1);
+    similar_calls("OK1KRQ", "OK1KQR",0,thr,1);
+    similar_calls("OK1KRQ", "OK1RKQ",0,thr,1);
+    similar_calls("OK1KRQ", "OKK1RQ",0,thr,1);
+    similar_calls("OK1KRQ", "O1KKRQ",0,thr,1);
+    similar_calls("OK1KRQ", "KO1KRQ",0,thr,1);
 
-    similar_calls("OK1KRQ", "OK1KRX",0,1);
-    similar_calls("OK1KRQ", "OK1KXQ",0,1);
-    similar_calls("OK1KRQ", "OK1XRQ",0,1);
-    similar_calls("OK1KRQ", "OKXKRQ",0,1);
-    similar_calls("OK1KRQ", "OX1KRQ",0,1);
-    similar_calls("OK1KRQ", "XK1KRQ",0,1);
-    similar_calls("OK1KRQ", "OK1Q",0,1);
+    similar_calls("OK1KRQ", "OK1KRX",0,thr,1);
+    similar_calls("OK1KRQ", "OK1KXQ",0,thr,1);
+    similar_calls("OK1KRQ", "OK1XRQ",0,thr,1);
+    similar_calls("OK1KRQ", "OKXKRQ",0,thr,1);
+    similar_calls("OK1KRQ", "OX1KRQ",0,thr,1);
+    similar_calls("OK1KRQ", "XK1KRQ",0,thr,1);
+    similar_calls("OK1KRQ", "OK1Q",0,thr,1);
+    
+    similar_calls("OM3I", "OM3RLA",0,thr,1);
+    similar_calls("OM3I", "OM3RL",0,thr,1);
+    similar_calls("OM3I", "OM3R",0,thr,1);
+    similar_calls("OM3RKA", "OM5MZ",0,thr,1);
+    similar_calls("HG10P", "HG3A",0,thr,1);
+    similar_calls("HG3A", "HG10P",0,thr,1);
     ST_STOP;
     return;
 #endif    
@@ -634,13 +757,13 @@ int main(int argc, char *argv[])
     g_free(s);
     
     exit(1);
-#endif    
+#endif  
 
     path_to_exe = argv[0];
     ac = argc;
     av = (char **)argv;
 
-    starting_sbrk = (long int)sbrk;
+    starting_sbrk = (char *)sbrk(0);
 /*    dbg("\n\n");*/
     select_loop(init);
     terminate_all_subsystems();
