@@ -2,13 +2,13 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
-#include <header.h>
+#include "header.h"
 
 
 /*********************** DUPLICATE CALLSIGN *********************************/
@@ -71,7 +71,7 @@ void duplicate_callsign_fn(struct dialog_data *dlg)
 
 int dlg_clear_tmpqsos(struct dialog_data *dlg, struct dialog_item_data *di){
 
-    clear_tmpqsos(aband);
+    clear_tmpqsos(aband,1);
     return cancel_dialog(dlg,di);
 }
 
@@ -79,7 +79,7 @@ int dlg_clear_tmpqsos(struct dialog_data *dlg, struct dialog_item_data *di){
 int dlg_edit_qso(struct dialog_data *dlg, struct dialog_item_data *di){
     edit_qso((struct qso *)dlg->dlg->refresh_data);
 
-    clear_tmpqsos(aband);
+    clear_tmpqsos(aband, 1);
     return cancel_dialog(dlg,di);
 }
 
@@ -90,7 +90,7 @@ int dlg_mark_as_error(struct dialog_data *dlg, struct dialog_item_data *di){
     qso_mark_as_error(aband, atoi(qso->qsonrs)-1);
     recalc_stats(aband);
     
-    aband->dupe_in_tmpqso = 0;
+    aband->dupe_in_tmpqso_index = -1;
     ADD_TMPQSO_STRING(aband, callsign, qso->callsign, 1, ucallsign);
     wkd_tmpqso(aband, WT_CALLSIGN, qso->callsign);
 
@@ -100,12 +100,12 @@ int dlg_mark_as_error(struct dialog_data *dlg, struct dialog_item_data *di){
 int dlg_store_as_dupe(struct dialog_data *dlg, struct dialog_item_data *di){
     struct qso *qso;             
    
-    aband->dupe_in_tmpqso = 0;
+    aband->dupe_in_tmpqso_index = -1;
     
     qso = (struct qso*) dlg->dlg->refresh_data;
 
     aband->tmpqsos[0].dupe = 1; 
-    /*ADD_TMPQSO_STRING(aband, callsign, qso->callsign, 1, ucallsign);*/
+    ADD_TMPQSO_STRING(aband, callsign, qso->callsign, 1, ucallsign);
     TMPQ.ucallsign=1;
     if (!TMPQ.ulocator) add_tmpqso_locator(aband, qso->locator, 0, 0); 
      
@@ -157,7 +157,7 @@ void duplicate_callsign(struct qso *qso)
 
 /**************************** EDIT ***********************************/
 
-void menu_add_error(union cba_t cba){
+void menu_add_error(cba_t cba){
     add_error(aband, "");
 } 
 
@@ -176,8 +176,8 @@ char *chop_msg[] = {
 void chop(struct band *band, char *op){
     gchar *c;
     
-    g_free(band->operator);
-    band->operator = fixsemi(g_strdup(op));
+    g_free(band->operator_);
+    band->operator_ = fixsemi(g_strdup(op));
     c = g_strconcat("O ", op, NULL);
     add_swap(band, c);
     g_free(c);  
@@ -256,14 +256,14 @@ void chop_fn(struct dialog_data *dlg)
 }
 
 
-void menu_chop(union cba_t cba){
+void menu_chop(cba_t cba){
     struct dialog *d;
     int i;
     
     if (!ctest || !aband) return;
     
-    fixsemi(aband->operator);
-    safe_strncpy0(op, aband->operator, EQSO_LEN);
+    fixsemi(aband->operator_);
+    safe_strncpy0(op, aband->operator_, EQSO_LEN);
     global_operator=cfg->global_operator;
 
 /*    input_field(NULL, VTEXT(T_CHOP), VTEXT(T_N_CALLSIGN), 
@@ -305,7 +305,7 @@ void menu_chop(union cba_t cba){
 }
 /*********************** SKEDQRG - Change QRG for sked *************************/
 
-void skedqrg(struct session *ses, char *qrg){
+void do_skedqrg(struct session *ses, char *qrg){
     gchar *c;
 
     if (!qrg) return;
@@ -321,7 +321,7 @@ void skedqrg(struct session *ses, char *qrg){
     redraw_later();
 }
 
-void menu_skedqrg(union cba_t cba){
+void menu_skedqrg(cba_t cba){
     static char qrg[20];
     
     if (!ctest || !aband) return;
@@ -332,13 +332,13 @@ void menu_skedqrg(union cba_t cba){
     input_field(NULL, VTEXT(T_SKED_QRG), VTEXT(T_N_SKED_QRG), 
                VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
                NULL, 20, qrg, 0, 0, NULL,
-               (void (*)(void *, char *)) skedqrg, NULL);
+               (void (*)(void *, char *)) do_skedqrg, NULL, 0);
 
 }
 
 /******************** FILL OPERATORS **************************/
 
-void menu_fillop(union cba_t cba){
+void menu_fillop(cba_t cba){
     struct band *b;
     struct qso *q;
     int i,j,k;
@@ -355,10 +355,10 @@ void menu_fillop(union cba_t cba){
             q=(struct qso *)g_ptr_array_index(b->qsos,j);
 
             for (k=0;k<calls->len;k++){
-                if (strcmp(q->operator,g_ptr_array_index(calls,k))==0) break;
+                if (strcmp(q->operator_,g_ptr_array_index(calls,k))==0) break;
             }
             if (k==calls->len)
-                g_ptr_array_add(calls, q->operator);
+                g_ptr_array_add(calls, q->operator_);
             
         }
 
@@ -390,13 +390,16 @@ void menu_fillop(union cba_t cba){
 
 /******************** RECALC ALL QRB and QTF  **************************/
 
-void menu_recalc_qrb(union cba_t cba){
+void menu_recalc_qrb(cba_t cba){
 	if (ctest) recalc_all_qrbqtf(ctest);
+    qrv_recalc_wkd(qrvdb);
+    qrv_recalc_qrbqtf(qrvdb);
+    qrv_recalc_gst(qrvdb);
 }
 
 /******************** UPDATE C_W ******************************/
 
-void menu_cw_update_contest(union cba_t cba){
+void menu_cw_update_contest(cba_t cba){
     gchar *s;
     int ret;
     
@@ -411,7 +414,7 @@ void menu_cw_update_contest(union cba_t cba){
     g_free(s);
 }
 
-void menu_cw_update_band(union cba_t cba){
+void menu_cw_update_band(cba_t cba){
     gchar *s;
     int ret;
     
@@ -426,11 +429,58 @@ void menu_cw_update_band(union cba_t cba){
     g_free(s);
 }
 
-void menu_import_ebw(union cba_t cba){
+/******************** UPDATE EXC ******************************/
+
+void menu_exc_update_contest(cba_t cba){
+    gchar *s, *ename;
+    int ret;
+    
+	
+	if (!ctest) return;
+    dbg("menu_exc_update_contest(%s)\n", ctest->excname);
+    if (!ctest->excname || !*ctest->excname) return;
+
+    ename = g_strdup(ctest->excname);
+    lc(ename);
+    
+    update_exc_from_ctest(excdb, ctest);
+    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakexc", ename, NULL);
+    ret=save_exc_into_file(excdb, s);        
+    if (ret){
+        errbox(VTEXT(T_CANT_WRITE), ret);
+    }else{
+        log_addf(VTEXT(T_SAVED_S), s);
+    }
+    g_free(s);
+}
+
+void menu_exc_update_band(cba_t cba){
+    gchar *s, *ename;
+    int ret;
+    
+	if (!ctest) return;
+    dbg("menu_exc_update_band(%s)\n", ctest->excname);
+    if (!ctest->excname || !*ctest->excname) return;
+
+    ename = g_strdup(ctest->excname);
+    lc(ename);
+    update_exc_from_band(excdb, aband);
+    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakexc", ename, NULL);
+    ret=save_exc_into_file(excdb, s);        
+    if (ret){
+        errbox(VTEXT(T_CANT_WRITE), ret);
+    }else{
+        log_addf(VTEXT(T_SAVED_S), s);
+    }
+    g_free(s);
+}
+
+void menu_import_ebw(cba_t cba){
     gchar *s;
     int ret;
     
     ret=read_ebw_files(cw, namedb);
+	if (ret!=0) return;
     
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
     ret=save_cw_into_file(cw, s);        
@@ -439,6 +489,7 @@ void menu_import_ebw(union cba_t cba){
     }else{
         log_addf(VTEXT(T_SAVED_S), s);
     }
+    g_free(s);
 
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnaknames", NULL);
     ret=save_namedb_into_file(namedb, s);        
@@ -447,9 +498,6 @@ void menu_import_ebw(union cba_t cba){
     }else{
         log_addf(VTEXT(T_SAVED_S), s);
     }
-
-
-    
     g_free(s);
 }
 
@@ -577,7 +625,7 @@ void autosave_fn(struct dialog_data *dlg)
     
 }
 
-void menu_autosave(union cba_t cba)
+void menu_autosave(cba_t cba)
 {
     struct dialog *d;
     int i;
@@ -619,10 +667,16 @@ void menu_autosave(union cba_t cba)
     d->items[i=0].type = D_FIELD;
     d->items[i].dlen = AS_LEN;
     d->items[i].data = as_floppy_aq_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10000;
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = AS_LEN;
     d->items[i].data = as_floppy_am_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10000;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
@@ -677,9 +731,12 @@ void refresh_add_sw(struct session *ses){
     
 
     switch(asw_type){
-        case SWT_PIPE: /* shell */
+        case SWT_PIPE:/* shell */
             sw = new_subwin(SWT_PIPE, VTEXT(T_SW_SHELL), sw_pipe_enter);
             break;
+        case SWT_KST3:/* KST in tucnak3, Shell in tucnak2 */
+            sw = new_subwin(SWT_KST3, VTEXT(T_SW_SHELL), sw_pipe_enter);
+            break;
         case SWT_QSOS: 
             sw = new_subwin(SWT_QSOS, VTEXT(T_SW_QSOS), NULL);
             break;
@@ -696,7 +753,7 @@ void refresh_add_sw(struct session *ses){
             sw = new_subwin(SWT_UNFI, VTEXT(T_SW_UNFI), NULL);
             break;
         case SWT_DXC: 
-/*            sw = new_subwin(SWT_DXC, VTEXT(T_SW_DXC), NULL);*/
+            sw = new_subwin(SWT_DXC, VTEXT(T_SW_DXC), NULL);
             break;
         case SWT_STAT: 
             sw = new_subwin(SWT_STAT, VTEXT(T_SW_STAT), NULL);
@@ -707,116 +764,28 @@ void refresh_add_sw(struct session *ses){
         case SWT_MAP:
             sw = new_subwin(SWT_MAP, VTEXT(T_SW_MAP), NULL);
             break;
+        case SWT_SCOPE:
+            sw = new_subwin(SWT_SCOPE, VTEXT(T_SW_SCOPE), NULL);
+            break;
+        case SWT_QRV:
+            sw = new_subwin(SWT_QRV, VTEXT(T_SW_QRV), NULL);
+            break;
+        case SWT_HF:
+            sw = new_subwin(SWT_HF, VTEXT(T_SW_HF), NULL);
+            break;
+        case SWT_PLAYER:
+            sw = new_subwin(SWT_PLAYER, VTEXT(T_SW_PLAYER), NULL);
+            break;
+        case SWT_CHART:
+            sw = new_subwin(SWT_CHART, VTEXT(T_SW_CHART), NULL);
+            break;
             
     }
     
 }
 
-char *add_sw_msg[] = {
-    CTEXT(T_SHELL), /* 0 */
-    CTEXT(T_QSOS),
-    CTEXT(T_LOG),
-    CTEXT(T_TALK),
-    CTEXT(T_SKEDS),
-    CTEXT(T_UNFI),          /* 5 */
-    CTEXT(T_DXC),
-    CTEXT(T_STAT),
-    CTEXT(T_SWAP),
-    CTEXT(T_MAP),               
-    CTEXT(T_CMD),  /* 10 */
-    CTEXT(T_RESPAWN),
-    "", /* OK */  /* 12 */
-    "", /* Cancel */
-};
 
-void add_sw_fn(struct dialog_data *dlg)
-{
-    struct terminal *term = dlg->win->term;
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
-
-    max_group_width(term, add_sw_msg + 0, dlg->items + 0, 1, &max);
-    min_group_width(term, add_sw_msg + 0, dlg->items + 0, 1, &min);
-    max_group_width(term, add_sw_msg + 1, dlg->items + 1, 1, &max);
-    min_group_width(term, add_sw_msg + 1, dlg->items + 1, 1, &min);
-    max_group_width(term, add_sw_msg + 2, dlg->items + 2, 1, &max);
-    min_group_width(term, add_sw_msg + 2, dlg->items + 2, 1, &min);
-    max_group_width(term, add_sw_msg + 3, dlg->items + 3, 1, &max);
-    min_group_width(term, add_sw_msg + 3, dlg->items + 3, 1, &min);
-    max_group_width(term, add_sw_msg + 4, dlg->items + 4, 1, &max);
-    min_group_width(term, add_sw_msg + 4, dlg->items + 4, 1, &min);
-    max_group_width(term, add_sw_msg + 5, dlg->items + 5, 1, &max);
-    min_group_width(term, add_sw_msg + 5, dlg->items + 5, 1, &min);
-    max_group_width(term, add_sw_msg + 6, dlg->items + 6, 1, &max);
-    min_group_width(term, add_sw_msg + 6, dlg->items + 6, 1, &min);
-    max_group_width(term, add_sw_msg + 7, dlg->items + 7, 1, &max);
-    min_group_width(term, add_sw_msg + 7, dlg->items + 7, 1, &min);
-    max_group_width(term, add_sw_msg + 8, dlg->items + 8, 1, &max);
-    min_group_width(term, add_sw_msg + 8, dlg->items + 8, 1, &min);
-    max_group_width(term, add_sw_msg + 9, dlg->items + 9, 1, &max);
-    min_group_width(term, add_sw_msg + 9, dlg->items + 9, 1, &min);
-    max_group_width(term, add_sw_msg +10, dlg->items +10, 1, &max);
-    max_group_width(term, add_sw_msg +10, dlg->items +10, 1, &max);
-    min_group_width(term, add_sw_msg +11, dlg->items +11, 1, &min);
-    min_group_width(term, add_sw_msg +11, dlg->items +11, 1, &min);
-    
-    max_buttons_width(term, dlg->items + 12, 2, &max);
-    min_buttons_width(term, dlg->items + 12, 2, &min);
-    
-    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    if (w > dlg->win->term->x - 2 * DIALOG_LB - 8 ) w = dlg->win->term->x - 2 * DIALOG_LB - 8;
-    if (w < 1) w = 1;
-    
-    rw = 0;
-    y ++;
-    dlg_format_group(NULL, term, add_sw_msg + 0, dlg->items + 0, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 1, dlg->items + 1, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 2, dlg->items + 2, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 3, dlg->items + 3, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 4, dlg->items + 4, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 5, dlg->items + 5, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 6, dlg->items + 6, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 7, dlg->items + 7, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 8, dlg->items + 8, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg + 9, dlg->items + 9, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_group(NULL, term, add_sw_msg +10, dlg->items +10, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, add_sw_msg +11, dlg->items +11, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items + 12, 2, 0, &y, w, &rw, AL_LEFT);
-    
-    
-    w = rw;
-    dlg->xw = w + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-
-    
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    y++;
-    dlg_format_group(term, term, add_sw_msg + 0, dlg->items + 0, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 1, dlg->items + 1, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 2, dlg->items + 2, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 3, dlg->items + 3, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 4, dlg->items + 4, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 5, dlg->items + 5, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 6, dlg->items + 6, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 7, dlg->items + 7, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 8, dlg->items + 8, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg + 9, dlg->items + 9, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group(term, term, add_sw_msg +10, dlg->items +10, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, add_sw_msg +11, dlg->items +11, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items + 12, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
-    
-}
-
-void menu_add_subwin(union cba_t cba){
+void menu_add_subwin(cba_t cba){
 
     struct dialog *d;
     int i;
@@ -826,81 +795,145 @@ void menu_add_subwin(union cba_t cba){
     safe_strncpy0(asw_cmd, "/bin/sh", MAX_STR_LEN);
     snprint(asw_respawn_str, MAX_STR_LEN, 10);
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 15 * sizeof(struct dialog_item)))) return;
-    memset(d, 0, sizeof(struct dialog) + 15 * sizeof(struct dialog_item));
+    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
+    memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
     d->title = VTEXT(T_ADD_SUBWIN);
-    d->fn = add_sw_fn;
+    d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_add_sw;
     d->refresh_data = (void *)gses; /*todo */
 
     
-    d->items[i=0].type = D_CHECKBOX;  /* 0 */
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_PIPE;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    
-    d->items[++i].type = D_CHECKBOX;
+    d->items[i=0].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_QSOS;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_LOG;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_TALK;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_QSOS);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_SKED;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SKEDS);
+    d->items[i].wrap = 1;
     
-    d->items[++i].type = D_CHECKBOX; /* 5 */
+    d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_UNFI;
+    d->items[i].gnum = SWT_TALK;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_TALK);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_DXC;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_DXC);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX; /* 5 */
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_UNFI;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_UNFI);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_STAT;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_STAT);
+    d->items[i].wrap = 1;
     
-    d->items[++i].type = D_CHECKBOX;
+    d->items[++i].type = D_CHECKBOX;  /* 0 */
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_SWAP;
+    d->items[i].gnum = SWT_PIPE;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SHELL);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_MAP;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_MAP);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_SCOPE;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SCOPE);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_QRV;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_QRV);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_HF;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_HF);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_LOG;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_LOG);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_PLAYER;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SW_PLAYER);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_CHART;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SW_CHART);
+    d->items[i].wrap = 1;
+    
+    /*d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_SWAP;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&asw_type;
+    d->items[i].msg = CTEXT(T_SWAP);
+    d->items[i].wrap = 1;  */
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = asw_cmd;
+    d->items[i].msg = CTEXT(T_CMD);
+    d->items[i].wrap = 2;
     
-    d->items[++i].type = D_FIELD;
+    /*d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = asw_respawn_str;
+    d->items[i].msg = CTEXT(T_RESPAWN);
+    d->items[i].wrap = 1; */
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
@@ -911,6 +944,8 @@ void menu_add_subwin(union cba_t cba){
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
+    d->items[i].align = AL_BUTTONS;
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_END;
     do_dialog(d, getml(d, NULL));
@@ -919,7 +954,7 @@ void menu_add_subwin(union cba_t cba){
 
 /********************* CLOSE SUBWIN ***********************/
 
-void menu_close_subwin(union cba_t cba){
+void menu_close_subwin(cba_t cba){
     struct subwin *sw;
     
 
@@ -927,11 +962,24 @@ void menu_close_subwin(union cba_t cba){
     if (!sw) return;
 
     free_subwin(sw);
-    sw_totop_next( 0);
+	if (sw == gses->ontop2)
+		sw_totop_next(0, 1);
+    sw_totop_next(0, 0);
     redraw_later();
     
 }
 
+/********************* SPLIT SUBWIN ***********************/
+
+void menu_split_subwin(cba_t cba){
+	cfg->splitheight = 10;
+	resize_terminal(CBA0);
+}
+
+void menu_remove_split(cba_t cba){
+	cfg->splitheight = 0;
+	resize_terminal(CBA0);
+}
 
 /******************* UNFINISHED REMARK *************/
 
@@ -967,23 +1015,29 @@ void do_unfinished(struct session *ses, char *remark){
     if (TMPQ.locator && TMPQ.ulocator){
         g_string_sprintfa(gs, "    %-8s %7d %3d", TMPQ.locator, (int)TMPQ.qrb, TMPQ.qtf);
     }
+#ifdef HAVE_HAMLIB
+    if (trig){
+        g_string_sprintfa(gs, "  %1.1f", trig->qrg/1000.0);
+    }
+#endif
 
     fifo_addf(aband->unfi, gs->str);
     g_string_free(gs, 1);
 
     add_error(aband, remark);/* before clear_tmpqsos! */
     
-    clear_tmpqsos(aband);
+    clear_tmpqsos(aband, 1);
     clear_inputline(aband->il);
     get_cw_qs(aband->il->cdata); /* clears */
     get_band_qs(aband, aband->il->cdata);
+    get_hf_dxc(aband->il->cdata);
     redraw_later();
     
 
 }
 
     
-void menu_unfinished(union cba_t cba){
+void menu_unfinished(cba_t cba){
     static char remark[40];
     int max;
 
@@ -999,7 +1053,7 @@ void menu_unfinished(union cba_t cba){
     input_field(NULL, VTEXT(T_UNFI_QSO), VTEXT(T_N_REMARK), 
                VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
                NULL, max, remark, 0, 0, NULL,
-               (void (*)(void *, char *)) do_unfinished, NULL);
+               (void (*)(void *, char *)) do_unfinished, NULL, 0);
     
 }
 
