@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -107,7 +107,7 @@ void cq_cw_fn(struct dialog_data *dlg)
     
 }
 
-void cq_cw(union cba_t cba){
+void cq_cw(cba_t cba){
 
     struct dialog *d;
     int i;
@@ -115,7 +115,7 @@ void cq_cw(union cba_t cba){
     struct cq *cq;
 
 
-    no = cba.int_;               
+    no = (int)GETCBA(cba, int_);
     cq = g_ptr_array_index(cfg->cqs, no);
     
     safe_strncpy0(cw_str, cq->cw_str, MAX_STR_LEN);
@@ -140,6 +140,9 @@ void cq_cw(union cba_t cba){
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = cw_speed_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = cfg->cwda_maxwpm;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
@@ -150,6 +153,9 @@ void cq_cw(union cba_t cba){
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = cw_ts_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 300;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
@@ -174,24 +180,24 @@ void cq_cw(union cba_t cba){
                                
 }
 
-void menu_cq_cw(union cba_t cba){
+void menu_cq_cw(cba_t cba){
     int i;
     char *c;
     struct menu_item *mi = NULL;
     struct cq *cq;
-    char example[20];
+    char example[40];
     
 
     for (i=0; i<cfg->cqs->len; i++){
         cq = (struct cq *) g_ptr_array_index(cfg->cqs, i);
         
-        safe_strncpy0(example, cq->cw_str, 20);
+        safe_strncpy0(example, cq->cw_str, 40);
         c = g_strdup_printf(VTEXT(T_CW_DSCS), i, keys[i], cq->cw_repeat?'R':' ', example);
         if (!mi) if (!(mi = new_menu(3))) return;
         add_to_menu(&mi, 
                 stracpy(c), 
                 "", "", 
-                MENU_FUNC cq_cw, (union cba_t)i, 0);    
+                MENU_FUNC cq_cw, (cba_t)i, 0);    
         g_free(c);
     }
     if (i) 
@@ -275,7 +281,7 @@ void cq_ssb_fn(struct dialog_data *dlg)
     
 }
 
-void cq_ssb(union cba_t cba){
+void cq_ssb(cba_t cba){
 
     struct dialog *d;
     int i;
@@ -283,7 +289,7 @@ void cq_ssb(union cba_t cba){
     struct cq *cq;
 
 
-    no = cba.int_;            
+    no = (int)GETCBA(cba, int_);            
     cq = g_ptr_array_index(cfg->cqs, no);
     
     safe_strncpy0(ssb_file, cq->ssb_file, MAX_STR_LEN);
@@ -312,6 +318,9 @@ void cq_ssb(union cba_t cba){
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssb_ts_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 300;
     
     d->items[++i].type = D_BUTTON; /* 3 */
     d->items[i].gid = B_ENTER;
@@ -330,7 +339,7 @@ void cq_ssb(union cba_t cba){
 }
 
 
-void menu_cq_ssb(union cba_t cba){
+void menu_cq_ssb(cba_t cba){
     int i;
     char *c;
     struct menu_item *mi = NULL;
@@ -347,7 +356,7 @@ void menu_cq_ssb(union cba_t cba){
         add_to_menu(&mi, 
                 stracpy(c), 
                 "", "", 
-                MENU_FUNC cq_ssb, (union cba_t)i, 0);    
+                MENU_FUNC cq_ssb, (cba_t)i, 0);    
         g_free(c);
     }
     if (i) 
@@ -366,7 +375,10 @@ char cwda_hostname[MAX_STR_LEN];
 char udp_port_str[EQSO_LEN];
 char speed_str[EQSO_LEN];
 char weight_str[EQSO_LEN];
+char minwpm_str[EQSO_LEN], maxwpm_str[EQSO_LEN];
 int cwda_spk;
+char leadin_str[EQSO_LEN];
+char autgive_str[EQSO_LEN];
     
 void refresh_cwda(void *xxx){
     
@@ -376,88 +388,20 @@ void refresh_cwda(void *xxx){
     cfg->cwda_udp_port = atoi(udp_port_str);
     cfg->cwda_speed = atoi(speed_str);
     cfg->cwda_weight = atoi(weight_str);
+    if (cfg->cwda_weight < 30) cfg->cwda_weight = 30;
+    if (cfg->cwda_weight > 70) cfg->cwda_weight = 70;
+    cfg->cwda_minwpm = atoi(minwpm_str);
+    cfg->cwda_maxwpm = atoi(maxwpm_str);
     cfg->cwda_spk = cwda_spk;
+    cfg->cwda_leadin = atoi(leadin_str);
+    cfg->cwda_autgive = atoi(autgive_str);
 	
 	free_cwdaemon(cwda);
 	cwda=init_cwdaemon();
 }
 
-char *cwda_msg[] = {
-	"None",         /* 0 */
-	"parport",
-	"ttys",
-	"Davac4",
-	"cwdaemon",
-    CTEXT(T_DEVICE),   /* 5*/ 
-    CTEXT(T_HOSTNAME),     
-    CTEXT(T_UDPPORT),
-    CTEXT(T_INIT_SPEED),
-    CTEXT(T_INIT_WEIGHT),  
-    CTEXT(T_USE_SPK),      /* 10 */
-    "", /* OK */          
-    "", /* Cancel */       
-};
 
-
-void cwda_fn(struct dialog_data *dlg)
-{
-    struct terminal *term = dlg->win->term;
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
-
-    max_group_width(term, cwda_msg + 0, dlg->items + 0, 5, &max);
-    min_group_width(term, cwda_msg + 0, dlg->items + 0, 5, &min);
-    max_group_width(term, cwda_msg + 5, dlg->items + 5, 1, &max);
-    min_group_width(term, cwda_msg + 5, dlg->items + 5, 1, &min);
-    max_group_width(term, cwda_msg + 6, dlg->items + 6, 2, &max);
-    min_group_width(term, cwda_msg + 6, dlg->items + 6, 2, &min);
-    max_group_width(term, cwda_msg + 8, dlg->items + 8, 2, &max);
-    min_group_width(term, cwda_msg + 8, dlg->items + 8, 2, &min);
-    max_group_width(term, cwda_msg + 10, dlg->items + 10, 1, &max);
-    min_group_width(term, cwda_msg + 10, dlg->items + 10, 1, &min);
-    
-    max_buttons_width(term, dlg->items + 11, 2, &max);
-    min_buttons_width(term, dlg->items + 11, 2, &min);
-    
-    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    if (w > dlg->win->term->x - 2 * DIALOG_LB - 8 ) w = dlg->win->term->x - 2 * DIALOG_LB - 8;
-    if (w < 1) w = 1;
-    
-    rw = 0;
-    y ++;
-    dlg_format_group(NULL, term, cwda_msg +  0, dlg->items +  0, 5, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, cwda_msg +  5, dlg->items +  5, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, cwda_msg +  6, dlg->items +  6, 2, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, cwda_msg +  8, dlg->items +  8, 2, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, cwda_msg + 10, dlg->items + 10, 1, 0, &y, w, &rw);
-    
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items + 11, 2, 0, &y, w, &rw, AL_LEFT);
-    
-    
-    w = rw;
-    dlg->xw = w + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-
-    
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    y++;
-    dlg_format_group(term, term, cwda_msg +  0, dlg->items +  0, 5, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, cwda_msg +  5, dlg->items +  5, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, cwda_msg +  6, dlg->items +  6, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, cwda_msg +  8, dlg->items +  8, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, cwda_msg + 10, dlg->items + 10, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items + 11, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
-    
-}
-
-void menu_cwda(union cba_t cba){
+void menu_cwda(cba_t cba){
 
     struct dialog *d;
     int i;
@@ -467,87 +411,177 @@ void menu_cwda(union cba_t cba){
     safe_strncpy0(cwda_hostname, cfg->cwda_hostname, MAX_STR_LEN);
     g_snprintf(udp_port_str, EQSO_LEN, "%d", cfg->cwda_udp_port);
     g_snprintf(speed_str, EQSO_LEN, "%d", cfg->cwda_speed);
+    if (cfg->cwda_weight < 30 || cfg->cwda_weight > 70) cfg->cwda_weight = 50;
     g_snprintf(weight_str, EQSO_LEN, "%d", cfg->cwda_weight);
+    g_snprintf(minwpm_str, EQSO_LEN, "%d", cfg->cwda_minwpm);
+    g_snprintf(maxwpm_str, EQSO_LEN, "%d", cfg->cwda_maxwpm);
     cwda_spk=cfg->cwda_spk;
+    g_snprintf(leadin_str, EQSO_LEN, "%d", cfg->cwda_leadin);
+    g_snprintf(autgive_str, EQSO_LEN, "%d", cfg->cwda_autgive);
 
     
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_CW_DAEMON);
-    d->fn = cwda_fn;
+//  d->fn = cwda_fn;
+    d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_cwda;
     d->refresh_data = NULL;
+    d->y0 = 1;
     
     d->items[i=0].type = D_CHECKBOX;  /* 0 */
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_NONE;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "None";
 	
     d->items[++i].type = D_CHECKBOX; 
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_PARPORT;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "parport";
 	
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_TTYS;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "ttys";
 	
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_DAVAC4;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "Davac4";
+    d->items[i].wrap = 1;
 	
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_CWD;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "cwdaemon";
 	
-    d->items[++i].type = D_FIELD;       /* 5 */
+    d->items[++i].type = D_CHECKBOX;    /* 5 */
+    d->items[i].gid = 1;
+    d->items[i].gnum = CWD_WINKEY;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "winkey";
+
+    d->items[++i].type = D_CHECKBOX;    /* 6 */
+    d->items[i].gid = 1;
+    d->items[i].gnum = CWD_WINKEY4;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&cwda_type;
+    d->items[i].msg = "winkey+davac4";
+    d->items[i].wrap = 1;
+	
+    d->items[++i].type = D_FIELD;       
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = cwda_device;
     d->items[i].maxl = 20;
+    d->items[i].msg = CTEXT(T_DEVICE);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = cwda_hostname;
     d->items[i].maxl = 20;
+    d->items[i].msg = CTEXT(T_HOSTNAME);
     
     d->items[++i].type = D_FIELD;    
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = udp_port_str;
     d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 65535;
+    d->items[i].msg = CTEXT(T_UDPPORT);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;      
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = speed_str;
     d->items[i].maxl = 3;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = cfg->cwda_maxwpm;
+    d->items[i].msg = CTEXT(T_INIT_SPEED);
     
-    d->items[++i].type = D_FIELD;      
+    d->items[++i].type = D_FIELD;     /* 10 */ 
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = weight_str;
     d->items[i].maxl = 3;
+    d->items[i].msg = CTEXT(T_INIT_WEIGHT);
+    d->items[i].wrap = 1;
     
-    d->items[++i].type = D_CHECKBOX;  /* 10 */
+    d->items[++i].type = D_FIELD;  
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = minwpm_str;
+    d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 99;
+    d->items[i].msg = CTEXT(T_WPMFROM);
+    
+    d->items[++i].type = D_FIELD;  
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = maxwpm_str;
+    d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 99;
+    d->items[i].msg = CTEXT(T_WPMTO);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;  
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&cwda_spk;
+    d->items[i].msg = CTEXT(T_USE_SPK);
     
-    d->items[++i].type = D_BUTTON;  
+    d->items[++i].type = D_FIELD;  
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = leadin_str;
+    d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10000;
+    d->items[i].msg = CTEXT(T_LEADIN);
+    d->items[i].wrap = 1;
+
+    d->items[++i].type = D_FIELD;     /* 15 */
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = autgive_str;
+    d->items[i].maxl = 3;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 9;
+    d->items[i].msg = CTEXT(T_AUTGIVE);
+    d->items[i].wrap = 2;
+
+    d->items[++i].type = D_BUTTON;    
+    d->items[i].gid = 0;
+    d->items[i].fn = dlg_winkey_opts;
+    d->items[i].text = VTEXT(T_WINKEYOPTS);
+    d->items[i].wrap = 2;
+    
+    d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
     
-    d->items[++i].type = D_BUTTON;    /* 12 */
+    d->items[++i].type = D_BUTTON;    
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
+    d->items[i].align = AL_BUTTONS;
+    d->items[i].wrap = 1;
     
     
     d->items[++i].type = D_END;
@@ -558,6 +592,8 @@ void menu_cwda(union cba_t cba){
 
 /*********************** SSBD ****************************/
 
+#ifdef HAVE_SNDFILE
+
 #define MAX_RECORD_FORMAT 7
 int record_format; /* index 0..MAX_RECORD_FORMAT-1 */
 extern char ssbd_format_str[MAX_STR_LEN];
@@ -584,7 +620,7 @@ int record_formats[]={
     SF_FORMAT_WAV | SF_FORMAT_GSM610
 };
 
-void record_format_func (union cba_t cba){
+void record_format_func (cba_t cba){
     int active;
     
     active=cba.int_;
@@ -601,18 +637,18 @@ int dlg_record_format(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 0; i < MAX_RECORD_FORMAT; i++) {
-        add_to_menu(&mi, record_format_msg[i], "", "", MENU_FUNC record_format_func, (union cba_t)i, 0);
+        add_to_menu(&mi, record_format_msg[i], "", "", MENU_FUNC record_format_func, (cba_t)i, 0);
     }
     sel = record_format;
     if (sel < 0) sel = 0;
     if (sel>=MAX_RECORD_FORMAT) sel=0;
-    do_menu_selected(mi, (union cba_t)record_format, sel);
+    do_menu_selected(mi, (cba_t)record_format, sel);
     return 0;
 }
 
 /* ------------------------ */    
 
-void alsa_src_func (union cba_t cba){
+void alsa_src_func (cba_t cba){
     int i, active;
     GString *labels;
     gchar **items;
@@ -655,10 +691,10 @@ int dlg_alsa_src(struct dialog_data *dlg, struct dialog_item_data *di){
     items=g_strsplit(labels->str, ";", 0);
     dbg("dlg_alsa_src() labels='%s'\n", labels->str);
     if (!(mi = new_menu(1))) goto x;
-    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC alsa_src_func, (union cba_t)0, 0);
+    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC alsa_src_func, (cba_t)0, 0);
     sel=0;
     for (i = 0; items[i]!=NULL; i++) {
-        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC alsa_src_func, (union cba_t)(i+1), 0);
+        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC alsa_src_func, (cba_t)(i+1), 0);
         if (strcmp(items[i], ssbd_alsa_src)==0) sel=i+1;
     }
     dbg("  sel=%d\n", sel);
@@ -671,7 +707,7 @@ x:;
     
 /* ------------------------ */    
 
-void oss_src_func (union cba_t cba){
+void oss_src_func (cba_t cba){
     int i, active;
     GString *labels;
     gchar **items;
@@ -679,7 +715,9 @@ void oss_src_func (union cba_t cba){
     active=cba.int_;
 
     labels=g_string_sized_new(100);
+#ifdef HAVE_OSS
     oss_get_sources(labels);
+#endif
     items=g_strsplit(labels->str, ";", 0);
     for (i=0;items[i]!=NULL;i++);/* dbg("items[%d]=%s\n", i, items[i]);*/
     dbg("oss_src_func: active=%d max=%d\n", active, i);
@@ -703,14 +741,16 @@ int dlg_oss_src(struct dialog_data *dlg, struct dialog_item_data *di){
     
     labels=g_string_sized_new(100);
 
+#ifdef HAVE_OSS
     oss_get_sources(labels);
+#endif
     items=g_strsplit(labels->str, ";", 0);
     dbg("labels='%s'\n", labels->str);
     if (!(mi = new_menu(1))) goto x;
-    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC oss_src_func, (union cba_t)0, 0);
+    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC oss_src_func, (cba_t)0, 0);
     sel=0;
     for (i = 0; items[i]!=NULL; i++) {
-        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC oss_src_func, (union cba_t)(i+1), 0);
+        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC oss_src_func, (cba_t)(i+1), 0);
         if (strcmp(items[i], ssbd_oss_src)==0) sel=i+1;
     }
     dbg("sel=%d\n", sel);
@@ -728,6 +768,7 @@ char ssbd_maxmin_str[EQSO_LEN], ssbd_diskfree_str[EQSO_LEN];
 int ssbd_format; /* 0x100xx */
 char ssbd_format_str[MAX_STR_LEN];
 char ssbd_channels_str[EQSO_LEN], ssbd_samplerate_str[EQSO_LEN];
+char ssbd_plev_str[EQSO_LEN], ssbd_rlev_str[EQSO_LEN];
 char ssbd_template[MAX_STR_LEN];
 
 char ssbd_pcm_play[EQSO_LEN], ssbd_pcm_rec[EQSO_LEN];
@@ -756,6 +797,8 @@ void refresh_ssbd(void *xxx){
     dbg("refresh_ssbd: record_formats[%d]=0x%x (%s)\n", record_format, cfg->ssbd_format, _(CTEXT(T_PCM8+record_format)));
     STORE_SINT(cfg, ssbd_channels);
     STORE_SINT(cfg, ssbd_samplerate);
+    STORE_SINT(cfg, ssbd_plev);
+    STORE_SINT(cfg, ssbd_rlev);
     STORE_STR(cfg, ssbd_template);
 
     STORE_STR(cfg, ssbd_pcm_play);
@@ -799,24 +842,26 @@ char *ssbd_msg[] = {
     CTEXT(T_FORMAT),
     CTEXT(T_CHANNELS),      
     CTEXT(T_SAMPLERATE),
+    "Playback volume",
+    "Capture volume",   /* 10 */
     CTEXT(T_TEMPLATE),  
 
-    CTEXT(T_PCM_PLAY),  /* 10 */
+    CTEXT(T_PCM_PLAY),  
     CTEXT(T_PCM_REC),
     CTEXT(T_PERIOD),         
-    CTEXT(T_BUFFER),
+    CTEXT(T_BUFFER),       /* 15 */
     CTEXT(T_ALSA_MIXER),
-    CTEXT(T_CAPTURE),      /* 15 */
+    CTEXT(T_CAPTURE),      
 
     CTEXT(T_DSP), 
     CTEXT(T_MAXFRAG),   
-    CTEXT(T_MIXER),        
+    CTEXT(T_MIXER),         /* 20 */  
     CTEXT(T_RECSRC),
     
-    CTEXT(T_SSBD_HOSTNAME), /* 20 */  
+    CTEXT(T_SSBD_HOSTNAME), 
     CTEXT(T_UDPPORT),
 
-    "", /* OK */       /* 22 */ 
+    "", /* OK */       /* 24 */ 
     "", /* Cancel */
 };
 
@@ -842,26 +887,26 @@ void ssbd_fn(struct dialog_data *dlg)
     min_group_width(term, ssbd_msg + 0, dlg->items + 0, 3, &min);
     max_group_width(term, ssbd_msg + 3, dlg->items + 3, 4, &max);
     min_group_width(term, ssbd_msg + 3, dlg->items + 3, 4, &min);
-    max_group_width(term, ssbd_msg + 7, dlg->items + 7, 2, &max);
-    min_group_width(term, ssbd_msg + 7, dlg->items + 7, 2, &min);
-    max_group_width(term, ssbd_msg + 9, dlg->items + 9, 1, &max);
-    min_group_width(term, ssbd_msg + 9, dlg->items + 9, 1, &min);
+    max_group_width(term, ssbd_msg + 7, dlg->items + 7, 4, &max);
+    min_group_width(term, ssbd_msg + 7, dlg->items + 7, 4, &min);
+    max_group_width(term, ssbd_msg +11, dlg->items +11, 1, &max);
+    min_group_width(term, ssbd_msg +11, dlg->items +11, 1, &min);
 
-    max_group_width(term, ssbd_msg +10, dlg->items +10, 4, &max);
-    min_group_width(term, ssbd_msg +10, dlg->items +10, 4, &min);
-    max_group_width(term, ssbd_msg +14, dlg->items +14, 2, &max);
-    min_group_width(term, ssbd_msg +14, dlg->items +14, 2, &min);
-    
+    max_group_width(term, ssbd_msg +12, dlg->items +12, 4, &max);
+    min_group_width(term, ssbd_msg +12, dlg->items +12, 4, &min);
     max_group_width(term, ssbd_msg +16, dlg->items +16, 2, &max);
     min_group_width(term, ssbd_msg +16, dlg->items +16, 2, &min);
+    
     max_group_width(term, ssbd_msg +18, dlg->items +18, 2, &max);
     min_group_width(term, ssbd_msg +18, dlg->items +18, 2, &min);
-    
     max_group_width(term, ssbd_msg +20, dlg->items +20, 2, &max);
     min_group_width(term, ssbd_msg +20, dlg->items +20, 2, &min);
+    
+    max_group_width(term, ssbd_msg +22, dlg->items +22, 2, &max);
+    min_group_width(term, ssbd_msg +22, dlg->items +22, 2, &min);
 
-    max_buttons_width(term, dlg->items +22, 2, &max);
-    min_buttons_width(term, dlg->items +22, 2, &min);
+    max_buttons_width(term, dlg->items +24, 2, &max);
+    min_buttons_width(term, dlg->items +24, 2, &min);
     
     w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
     if (w > max) w = max;
@@ -873,18 +918,18 @@ void ssbd_fn(struct dialog_data *dlg)
     y ++;
     dlg_format_group1(NULL, term, ssbd_msg + 0, dlg->items + 0, 3, 0, &y, w, &rw);
     dlg_format_group1(NULL, term, ssbd_msg + 3, dlg->items + 3, 4, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg + 7, dlg->items + 7, 2, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg + 9, dlg->items + 9, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_group1(NULL, term, ssbd_msg +10, dlg->items +10, 4, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg +14, dlg->items +14, 2, 0, &y, w, &rw);
+    dlg_format_group1(NULL, term, ssbd_msg + 7, dlg->items + 7, 4, 0, &y, w, &rw);
+    dlg_format_group1(NULL, term, ssbd_msg +11, dlg->items +11, 1, 0, &y, w, &rw);
     y++;
+    dlg_format_group1(NULL, term, ssbd_msg +12, dlg->items +12, 4, 0, &y, w, &rw);
     dlg_format_group1(NULL, term, ssbd_msg +16, dlg->items +16, 2, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg +18, dlg->items +18, 2, 0, &y, w, &rw);
     y++;
+    dlg_format_group1(NULL, term, ssbd_msg +18, dlg->items +18, 2, 0, &y, w, &rw);
     dlg_format_group1(NULL, term, ssbd_msg +20, dlg->items +20, 2, 0, &y, w, &rw);
     y++;
-    dlg_format_buttons(NULL, term, dlg->items +22, 2, 0, &y, w, &rw, AL_LEFT);
+    dlg_format_group1(NULL, term, ssbd_msg +22, dlg->items +22, 2, 0, &y, w, &rw);
+    y++;
+    dlg_format_buttons(NULL, term, dlg->items +24, 2, 0, &y, w, &rw, AL_LEFT);
     
     
     w = rw;
@@ -898,18 +943,18 @@ void ssbd_fn(struct dialog_data *dlg)
     y++;
     dlg_format_group1(term, term, ssbd_msg + 0, dlg->items + 0, 3, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group1(term, term, ssbd_msg + 3, dlg->items + 3, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg + 7, dlg->items + 7, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg + 9, dlg->items + 9, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group1(term, term, ssbd_msg +10, dlg->items +10, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg +14, dlg->items +14, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group1(term, term, ssbd_msg + 7, dlg->items + 7, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group1(term, term, ssbd_msg +11, dlg->items +11, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
+    dlg_format_group1(term, term, ssbd_msg +12, dlg->items +12, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group1(term, term, ssbd_msg +16, dlg->items +16, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg +18, dlg->items +18, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
+    dlg_format_group1(term, term, ssbd_msg +18, dlg->items +18, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group1(term, term, ssbd_msg +20, dlg->items +20, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
-    dlg_format_buttons(term, term, dlg->items + 22, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+    dlg_format_group1(term, term, ssbd_msg +22, dlg->items +22, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    y++;
+    dlg_format_buttons(term, term, dlg->items + 24, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 }
 
 void do_ssbd(void){
@@ -931,6 +976,8 @@ void do_ssbd(void){
     dbg("do_ssbd: ssbd_format=0x%x (%s)  record_format=%d\n", ssbd_format, ssbd_format_str, record_format);
     g_snprintf(ssbd_channels_str, EQSO_LEN, "%d", cfg->ssbd_channels);
     g_snprintf(ssbd_samplerate_str, EQSO_LEN, "%d", cfg->ssbd_samplerate);
+    g_snprintf(ssbd_plev_str, EQSO_LEN, "%d", cfg->ssbd_plev);
+    g_snprintf(ssbd_rlev_str, EQSO_LEN, "%d", cfg->ssbd_rlev);
     safe_strncpy0(ssbd_template, cfg->ssbd_template, MAX_STR_LEN);
 
     safe_strncpy0(ssbd_pcm_play, cfg->ssbd_pcm_play, EQSO_LEN);
@@ -957,7 +1004,10 @@ void do_ssbd(void){
         if (!*cfg->ssbd_oss_src) c=NULL;
         else c=cfg->ssbd_oss_src;
     }else{
+        c=NULL;
+#ifdef HAVE_OSS
         c=oss_recsrc2source(cfg->ssbd_recsrc);
+#endif
     }
     dbg("do_ssbd: cfg->ssbd_recsrc=%d label='%s'\n", cfg->ssbd_recsrc, c);
     if (c){
@@ -1006,11 +1056,17 @@ void do_ssbd(void){
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_maxmin_str;
     d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10000;
 
     d->items[++i].type = D_FIELD;      /* 5 */
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_diskfree_str;
     d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10000;
  
     d->items[++i].type = D_BUTTON;     
     d->items[i].gid  = 0;
@@ -1021,18 +1077,40 @@ void do_ssbd(void){
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_channels_str;
     d->items[i].maxl = 3;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 8;
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_samplerate_str;
     d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 4000;
+    d->items[i].gnum = 180000;
+    
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ssbd_plev_str;
+    d->items[i].maxl = 4;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = -1;
+    d->items[i].gnum = 100;
+    
+    d->items[++i].type = D_FIELD;    /* 10 */
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ssbd_rlev_str;
+    d->items[i].maxl = 4;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = -1;
+    d->items[i].gnum = 100;
     
     d->items[++i].type = D_FIELD;  
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_template;
     d->items[i].maxl = 30;
     
-    d->items[++i].type = D_FIELD;    /* 10 */
+    d->items[++i].type = D_FIELD;    
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_pcm_play;
     d->items[i].maxl = 15;
@@ -1046,18 +1124,24 @@ void do_ssbd(void){
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_period_time_str;
     d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 1000;
     
-    d->items[++i].type = D_FIELD;
+    d->items[++i].type = D_FIELD;   /* 15 */   
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_buffer_time_str;
     d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 1000;
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_alsa_mixer;
     d->items[i].maxl = 15;
     
-    d->items[++i].type = D_BUTTON;  /* 15 */   
+    d->items[++i].type = D_BUTTON;  
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_alsa_src;
     d->items[i].text = ssbd_alsa_src;
@@ -1071,8 +1155,11 @@ void do_ssbd(void){
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_maxfrag_str;
     d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 20;
     
-    d->items[++i].type = D_FIELD;   
+    d->items[++i].type = D_FIELD;    /* 20 */     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_mixer;
     d->items[i].maxl = 20;
@@ -1082,7 +1169,7 @@ void do_ssbd(void){
     d->items[i].fn   = dlg_oss_src;
     d->items[i].text = ssbd_oss_src;
 
-    d->items[++i].type = D_FIELD;    /* 20 */     
+    d->items[++i].type = D_FIELD;    
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_hostname;
     d->items[i].maxl = 20;
@@ -1091,8 +1178,11 @@ void do_ssbd(void){
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_udp_port_str;
     d->items[i].maxl = 6;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 65535;
     
-    d->items[++i].type = D_BUTTON; /* 22 */
+    d->items[++i].type = D_BUTTON; /* 24 */
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -1108,17 +1198,17 @@ void do_ssbd(void){
 }
 
 
-void menu_ssbd(union cba_t cba){
+void menu_ssbd(cba_t cba){
     
     do_ssbd();
 }
-
+#endif //HAVE_SNDFILE
 
 
 /*********************** NETWORK ****************************/
 
 char net_if_ignore[MAX_STR_LEN],net_ip_ignore[MAX_STR_LEN],net_ip_announce[MAX_STR_LEN];
-int trace_bcast, trace_sock, trace_recv, trace_send, trace_qsos ;
+int trace_bcast, trace_sock, trace_recv, trace_send, trace_qsos, trace_sdev;
 
 void refresh_net(void *xxx){
     
@@ -1130,86 +1220,11 @@ void refresh_net(void *xxx){
     STORE_INT(cfg, trace_recv);
     STORE_INT(cfg, trace_send);
     STORE_INT(cfg, trace_qsos);
+    STORE_INT(cfg, trace_sdev);
     free_net_ifaces(net);
     init_net_ifaces(net);
 }
 
-char *net_msg[] = {
-   CTEXT(T_IF_IGNORE),  /* 0 */
-   CTEXT(T_IP_IGNORE),
-   CTEXT(T_IP_ANNOUNCE), 
-   CTEXT(T_TRACE_BCAST), 
-   CTEXT(T_TRACE_SOCK),  
-   CTEXT(T_TRACE_RECV),      /* 5 */    
-   CTEXT(T_TRACE_SEND),      
-   CTEXT(T_TRACE_QSOS), 
-   "", /* OK */       /* 8 */ 
-   "", /* Cancel */
-};
-
-
-void net_fn(struct dialog_data *dlg)
-{
-    struct terminal *term = dlg->win->term;
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
-
-    max_group_width(term, net_msg + 0, dlg->items + 0, 1, &max);
-    min_group_width(term, net_msg + 0, dlg->items + 0, 1, &min);
-    max_group_width(term, net_msg + 1, dlg->items + 1, 1, &max);
-    min_group_width(term, net_msg + 1, dlg->items + 1, 1, &min);
-    max_group_width(term, net_msg + 2, dlg->items + 2, 1, &max);
-    min_group_width(term, net_msg + 2, dlg->items + 2, 1, &min);
-    max_group_width(term, net_msg + 3, dlg->items + 3, 2, &max);
-    min_group_width(term, net_msg + 3, dlg->items + 3, 2, &min);
-    max_group_width(term, net_msg + 5, dlg->items + 5, 2, &max);
-    min_group_width(term, net_msg + 5, dlg->items + 5, 2, &min);
-    max_group_width(term, net_msg + 7, dlg->items + 7, 1, &max);
-    min_group_width(term, net_msg + 7, dlg->items + 7, 1, &min);
-    
-    max_buttons_width(term, dlg->items +8, 2, &max);
-    min_buttons_width(term, dlg->items +8, 2, &min);
-    
-    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    if (w > dlg->win->term->x - 2 * DIALOG_LB - 8 ) w = dlg->win->term->x - 2 * DIALOG_LB - 8;
-    if (w < 1) w = 1;
-    
-    rw = 0;
-    y ++;
-    dlg_format_group1(NULL, term, net_msg + 0, dlg->items + 0, 1, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, net_msg + 1, dlg->items + 1, 1, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, net_msg + 2, dlg->items + 2, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_group1(NULL, term, net_msg + 3, dlg->items + 3, 2, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, net_msg + 5, dlg->items + 5, 2, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, net_msg + 7, dlg->items + 7, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items +8, 2, 0, &y, w, &rw, AL_LEFT);
-    
-    
-    w = rw;
-    dlg->xw = w + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-
-    
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    y++;
-    dlg_format_group1(term, term, net_msg + 0, dlg->items + 0, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, net_msg + 1, dlg->items + 1, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, net_msg + 2, dlg->items + 2, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group1(term, term, net_msg + 3, dlg->items + 3, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, net_msg + 5, dlg->items + 5, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, net_msg + 7, dlg->items + 7, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items + 8, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
-    
-}
 
 void do_net(void){
 
@@ -1229,54 +1244,75 @@ void do_net(void){
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_NETWORK);
-    d->fn = net_fn;
+    d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_net;
+    d->y0 = 1;
     
     d->items[i=0].type = D_FIELD;     /* 0 */
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = net_if_ignore;
-    d->items[i].maxl = 20;
+    d->items[i].maxl = 24;
+    d->items[i].msg = CTEXT(T_IF_IGNORE);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = net_ip_ignore;
-    d->items[i].maxl = 20;
+    d->items[i].maxl = 24;
+    d->items[i].msg = CTEXT(T_IP_IGNORE);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = net_ip_announce;
-    d->items[i].maxl = 20;
+    d->items[i].maxl = 24;
+    d->items[i].msg = CTEXT(T_IP_ANNOUNCE);
+    d->items[i].wrap = 2;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&trace_bcast;
+    d->items[i].msg = CTEXT(T_TRACE_BCAST);
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&trace_sock;
+    d->items[i].msg = CTEXT(T_TRACE_SOCK);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;   /* 5 */
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&trace_recv;
+    d->items[i].msg = CTEXT(T_TRACE_RECV);
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&trace_send;
+    d->items[i].msg = CTEXT(T_TRACE_SEND);
+    d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&trace_qsos;
+    d->items[i].msg = CTEXT(T_TRACE_QSOS);
     
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&trace_sdev;
+    d->items[i].msg = CTEXT(T_TRACE_SDEV);
+    d->items[i].wrap = 2;
     
     d->items[++i].type = D_BUTTON; /* 8 */
     d->items[i].gid = B_ENTER;
@@ -1287,6 +1323,8 @@ void do_net(void){
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
+    d->items[i].align = AL_BUTTONS;
+    d->items[i].wrap = 1;
     
     
     d->items[++i].type = D_END;
@@ -1295,7 +1333,7 @@ void do_net(void){
 }
 
 
-void menu_network(union cba_t cba){
+void menu_network(cba_t cba){
     
     do_net();
 }
