@@ -2,16 +2,14 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-void add_namedb(struct namedb *namedb, gchar *call, gchar *name);
-    
 struct namedb *namedb;
 
 struct namedb *init_namedb(){
@@ -79,8 +77,10 @@ int load_namedb_from_file(struct namedb *namedb, gchar *filename){
 
 void read_namedb_files(struct namedb *namedb){
     gchar *s;
-    
+
+#ifdef UNIX
     load_namedb_from_file(namedb, SHAREDIR"/tucnaknames");
+#endif
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnaknames", NULL); 
     load_namedb_from_file(namedb, s);
     g_free(s);
