@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -76,17 +76,17 @@ int get_e(char *env)
 
 #if defined(UNIX) || defined(BEOS)
 
-void sigwinch(union cba_t cba)
+void sigwinch(cba_t cba)
 {
     ((void (*)(void))cba.void_)();
 }
 
-void handle_terminal_resize(int fd, void (*fn)(union cba_t))
+void handle_terminal_resize(int fd, void (*fn)(cba_t))
 {
 #ifdef HAVE_SDL    
     if (sdl) return;
 #endif
-    install_signal_handler(SIGWINCH, sigwinch, (union cba_t)(void*)fn, 0);
+    install_signal_handler(SIGWINCH, sigwinch, (cba_t)(void*)fn, 0);
 }
 
 void unhandle_terminal_resize(int fd)
@@ -160,7 +160,7 @@ void winch(void *s)
     ((void (*)())s)();
 }
 
-void handle_terminal_resize(int fd, void (*fn)(union cba_t))
+void handle_terminal_resize(int fd, void (*fn)(cba_t))
 {
     if (!is_xterm()) return;
     if (!winch_thread_running) {
@@ -168,7 +168,7 @@ void handle_terminal_resize(int fd, void (*fn)(union cba_t))
         winch_thread_running = 1;
         _beginthread((void (*)(void *))winch_thread, NULL, 0x32000, NULL);
     }
-    set_handlers(winch_pipe[0], winch, NULL, NULL, (union cba_t)(void*)fn);
+    set_handlers(winch_pipe[0], winch, NULL, NULL, (cba_t)(void*)fn);
 }
 
 void unhandle_terminal_resize(int fd)
@@ -242,17 +242,17 @@ int c_pipe(int *fd)
 
 #elif defined(OS2) || defined(WIN32)
 
-void set_bin(int fd)
+/*void set_bin(int fd)
 {
     setmode(fd, O_BINARY);
-}
+} */
 
 int c_pipe(int *fd)
 {
     int r = pipe(fd);
-    if (!r) set_bin(fd[0]), set_bin(fd[1]);
+    //if (!r) set_bin(fd[0]), set_bin(fd[1]);
     return r;
-}
+} 
 
 #endif
 
@@ -1080,23 +1080,24 @@ int start_thread(void (*fn)(void *, int), void *ptr, int l)
 int start_thread(void (*fn)(void *, int), void *ptr, int l)
 {
     int p[2];
-    int f;
+    int f, ret;
+
     if (c_pipe(p) < 0) return -1;
     fcntl(p[0], F_SETFL, O_NONBLOCK);
     fcntl(p[1], F_SETFL, O_NONBLOCK);
     if (!(f = fork())) {
-        close(p[0]);
+        closesocket(p[0]);
         fn(ptr, p[1]);
-        write(p[1], "x", 1);
-        close(p[1]);
+        ret = write(p[1], "x", 1);
+        closesocket(p[1]);
         exit(0);
     }
     if (f == -1) {
-        close(p[0]);
-        close(p[1]);
+        closesocket(p[0]);
+        closesocket(p[1]);
         return -1;
     }
-    close(p[1]);
+    closesocket(p[1]);
     return p[0];
 }
 
@@ -1189,7 +1190,7 @@ struct gpm_mouse_spec {
     void *data;
 };
 
-void gpm_mouse_in(union cba_t cba)
+void gpm_mouse_in(cba_t cba)
 {
     Gpm_Event gev;
     struct event ev;
@@ -1229,7 +1230,7 @@ void *handle_mouse(int cons, void (*fn)(void *, char *, int), void *data)
     gms->h = h;
     gms->fn = fn;
     gms->data = data;
-    set_handlers(h, gpm_mouse_in, NULL, NULL, (union cba_t)gms);
+    set_handlers(h, gpm_mouse_in, NULL, NULL, (cba_t)gms);
     return gms;
 }
 
@@ -1402,6 +1403,9 @@ int can_open_os_shell(int environment)
 
 
 #ifndef HAVE_LOCKF
+#if defined(_MSC_VER) || defined(__MINGW32__)
+// TODO
+#else
 /* lockf is a simplified interface to fcntl's locking facilities.  */
 /* thanx to glibc */
 int
@@ -1448,14 +1452,18 @@ lockf (int fd, int cmd, off_t len)
 
   return fcntl (fd, cmd, &fl);
 }
-
+#endif
 #endif
 
-
+#if defined(_MSC_VER) || defined(__MINGW32__)
+//TODO
+#else
 void disable_screensaver(){
     char ioctlarg[2];
     char *term;
     int vcterm;
+
+    if (cfg->dssaver == 0) return;
     
     term = getenv("TERM");
     if (!term) 
@@ -1483,9 +1491,16 @@ void disable_screensaver(){
     
     /* xset -dpms */
     if (getenv("DISPLAY")){
-        system("xset -dpms ; xset s off");
+    //    system("xset -dpms ; xset s off");
+        pid_t pid=fork();
+        if (pid==0){
+            execlp("xset", "xset", "s", "off", "-dpms", NULL);
+            exit(-1);
+        }
+        waitpid(pid, NULL, 0);
     }
 }
+#endif
 
 int save_screen(){
     if (!is_xterm()) return 1;
@@ -1542,6 +1557,7 @@ int floppy_is_mounted(gchar *floppydev, gchar *path_to_floppy){
     return ret;
 }
 
+#ifdef UNIX
 int is_in_mtab(gchar *mountpoint){
     FILE *f;
     char *filename,*s,*token_ptr;
@@ -1570,6 +1586,12 @@ int is_in_mtab(gchar *mountpoint){
 	/*dbg("return %d\n", ret);*/
     return ret;
 }
+#else
+int is_in_mtab(gchar *mountpoint){
+	return 0;
+}
+#endif
+
 
 int do_mount(gchar *mountcmd){
    	return 0;
@@ -1577,34 +1599,47 @@ int do_mount(gchar *mountcmd){
 }
 
 static int sound_fd=-1;
+int sound_pid=-1;
 
 int init_sound(){
+#if !(defined(_MSC_VER) || defined(__MINGW32__))
     int ret,i;
     int fds[2];
 
-    if (pipe(fds)) internal("Can't create pipe");
+    if (pipe(fds)) internal_("Can't create pipe");
     /*dbg("sound pipe %d->%d\n", fds[1], fds[0]);*/
 
     ret=fork();
     if (ret<0){
-        internal("Can't fork");
+        internal_("Can't fork");
     }
     if (ret) {  /* parent */
-        close(fds[0]);
+        dbg("sound_pid = %d\n", ret);
+        sound_pid = ret;
+        closesocket(fds[0]);
         sound_fd=fds[1];
         return 0; 
     }
     /* child */
-    close(fds[1]);
+    sleep(1);
+    closesocket(fds[1]);
     close(0);
     dup2(fds[0], 0);
-    close(fds[0]);
+    closesocket(fds[0]);
     for (i=3;i<1024;i++) close(i);
+#ifdef _MSC_VER
+    _execlp("soundwrapper", "soundwrapper", "soundwrapper", NULL);
+#else
     execlp("soundwrapper", "soundwrapper", NULL);
+#endif
 	exit(-1);
+#else
+	return 0;
+#endif
 }
 
 int free_sound(){
+#if !(defined(_MSC_VER) || defined(__MINGW32__))
     int cmd, ret;
     
     if (sound_fd<0) return -1;
@@ -1616,6 +1651,14 @@ int free_sound(){
     }
     close(sound_fd);
     sound_fd=-1;
+#endif
+    return 0;
+}
+
+int abort_sound(){
+    if (sound_fd<0) return -1;
+    close(sound_fd);
+    sound_fd=-1;
     return 0;
 }
 
@@ -1630,6 +1673,7 @@ int sound(int freq){
     ret=write(sound_fd, &freq, sizeof(int));
     if (ret!=sizeof(int)){
         char errstr[1030];
+
         
         dbg("error writing sound_fd=%d (error %d) '%s'\n", sound_fd, errno, strerror_r(errno, errstr, sizeof(errstr)));      
         close(sound_fd);
