@@ -4,8 +4,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 */
 
@@ -20,7 +20,9 @@
 #if defined(__EMX__)
 #define OS2
 #elif defined(_WIN32)
+#ifndef WIN32
 #define WIN32
+#endif
 #elif defined(__BEOS__)
 #define BEOS
 #else
@@ -86,6 +88,14 @@ static inline int dir_sep(char x) { return x == '/' || x == '\\'; }
 int lockf(int,int,off_t);
 #endif
 
+#ifdef UNIX
+#define tmkdir(dir, mode) mkdir(dir, mode)
+#define sock_errno errno
+#define sock_strerror_r strerror_r
+#define SO4
+#define closesocket close
+static inline char *wokna(char *file) { return file; }
+#endif
 
 int mkdir_p(const char *s, mode_t mode);
 int save_screen(void);
@@ -96,8 +106,10 @@ int floppy_is_mounted(gchar *floppydev, gchar *path_to_floppy);
 int is_in_mtab(gchar *mp);
 
 
+extern int sound_pid;
 int init_sound(void);
 int free_sound(void);
+int abort_sound(void);
 int sound(int freq);
 double fdf(int fd);
 
