@@ -2,8 +2,8 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -34,9 +34,12 @@ int init_ctest(){
     ctest = g_new0(struct contest, 1);
     ctest->bands = g_ptr_array_new();
     ctest->bystamp = g_hash_table_new(g_str_hash, g_str_equal);
+    ctest->allqsos = g_index_array_new();
+    ctest->spypeers = g_ptr_array_new();
+    ctest->allb_prefs = g_hash_table_new(g_str_hash, g_str_equal);
 
 /*    ctest->serial = 1;*/
-	ctest->minute_timer_id=install_timer(60000, timer_minute_stats_all, CBA0);
+    ctest->minute_timer_id=install_timer(100, timer_minute_stats_all, CBA0);
     return 0;
 }
 
@@ -58,12 +61,12 @@ int new_ctest(char *tdate){
         nowdate_int = (tm.tm_year+1900)*10000 + (tm.tm_mon+1)*100 + tm.tm_mday;
         ctestdate_int = atoi(ctest->cdate);
         if (/*nowdate_int >= ctestdate_int && */nowdate_int <= ctestdate_int + 2) 
-            ctest->recording = 1;
+            ctest->oldcontest = 0;
         else
-            ctest->recording = 0;
+            ctest->oldcontest = 1;
     }else{
         ctest->cdate = g_strdup_printf("%4d%02d%02d", tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday );
-        ctest->recording = 1;
+        ctest->oldcontest = 0;
     }
     
     
@@ -79,7 +82,7 @@ int new_ctest(char *tdate){
                 getenv("HOME"), ctest->cdate, counter);
     }
         
-    if (mkdir_p(ctest->directory,0770)){
+    if (mkdir_p(ctest->directory,0777)){
 
         g_snprintf(s,1000, VTEXT(T_CANT_CREATE_HOME_S),ctest->directory);
         errbox(s,errno);
@@ -133,16 +136,22 @@ int new_ctest(char *tdate){
     
     set_ctest_title();
 #ifdef HAVE_SDL
-	maps_reload();
-#endif	
+    maps_reload();
+#endif  
     
     return 0; 
 }
 
+
+
+
 int load_ctest_from_mem(struct contest *ctest, gchar *datedir, GHashTable *hash){
 
     ctest->directory = g_strdup(datedir); 
     
+/*    if (ctest->tname) g_free((gpointer)ctest->tname);
+    ctest->tname= g_strdup((gchar *)g_hash_table_lookup(hash, "tname")); */
+
     STORE_HASH_STR(ctest, tname);
     STORE_HASH_STR(ctest, tdate);
     STORE_HASH_STR(ctest, pcall);
@@ -161,37 +170,53 @@ int load_ctest_from_mem(struct contest *ctest, gchar *datedir, GHashTable *hash)
     STORE_HASH_STR(ctest, rphon);
     STORE_HASH_STR(ctest, rhbbs);
 
+	{
+    gchar *c; 
+    c = (gchar *)g_hash_table_lookup(hash, "qsoused"); 
+    if (c) ctest->qsoused= atoi(c); 
+    else ctest->qsoused = 0; 
+}
+
+
     STORE_HASH_STR(ctest, default_rs);
     STORE_HASH_STR(ctest, default_rst);
     STORE_HASH_INT(ctest, qsoused);
     STORE_HASH_INT(ctest, qsomult);
+    STORE_HASH_INT(ctest, qsoglob);
+    STORE_HASH_INT(ctest, prefmult);
+    STORE_HASH_INT(ctest, prefglob);
     STORE_HASH_INT(ctest, dxcbonu);
     STORE_HASH_INT(ctest, dxcmult);
     STORE_HASH_INT(ctest, excused);
+    STORE_HASH_INT(ctest, exctype);
+    STORE_HASH_STR(ctest, excname);
     STORE_HASH_INT(ctest, excbonu);
     STORE_HASH_INT(ctest, excmult);
-    STORE_HASH_INT(ctest, qsoused);
-    STORE_HASH_INT(ctest, qsomult);
+    STORE_HASH_INT_DEF(ctest, exccfm, 1);
     STORE_HASH_INT(ctest, qsop_method);
     STORE_HASH_INT(ctest, rstused);
+    STORE_HASH_INT(ctest, defrstr);
     STORE_HASH_INT(ctest, total_method);
     STORE_HASH_INT(ctest, wwltype);
     STORE_HASH_INT(ctest, wwlused);
     STORE_HASH_INT(ctest, wwlbonu);
     STORE_HASH_INT(ctest, wwlmult);
+    STORE_HASH_INT_DEF(ctest, wwlcfm, 1);
+    STORE_HASH_INT_DEF(ctest, tttype, 1);
+    STORE_HASH_INT(ctest, expmode);
     
     set_ctest_title();
+//    dbg("defrstr=%d\n", ctest->defrstr);
 /*#ifdef HAVE_SDL
-	maps_reload();
-#endif	*/
+    maps_reload();
+#endif  */
    
     return 0; 
 }
 
 
 
-struct band *init_band(struct config_band *confb, 
-                       GHashTable *opt_band){
+struct band *init_band(struct config_band *confb, GHashTable *opt_band, struct zstring *zs){
     struct band *b;
     gchar *c;
     static char s[1024]; /* for msg_box */
@@ -199,27 +224,33 @@ struct band *init_band(struct config_band *confb,
     if (init_ctest()) return NULL;
     
     if (confb){
-        b=find_band_by_pband(confb->pband); /*FIXME confb*/
+        b=find_band_by_pband(confb->pband);
         if (b) return b;
     }
 
     b=g_new0(struct band,1);
 
     b->qsos = g_ptr_array_new();
-    b->mode  = MOD_SSB_SSB;
-    b->bandmulti = 1;
-    b->spymode = SM_INPUTLINE;
+//    b->mode  = MOD_SSB_SSB;
+//    b->bandmulti = 1;
+/*    b->spymode = SM_INPUTLINE;*/
     if (confb){   /* usally new contest */
         b->bandchar = confb->bandchar;
         b->pband = g_strdup(confb->pband);
+        if (confb->qrg_min>30000)
+            b->bandname = g_strdup(b->pband);
+        else
+            b->bandname = g_strdup(confb->adifband);
+        b->qrg_min = confb->qrg_min;
+        b->qrg_max = confb->qrg_max;
         
         b->readonly = confb->readonly;
 
         b->psect = confb->psect; /* gint */
-        if (cfg->operator && *cfg->operator){
-            b->operator = fixsemi(g_strdup(cfg->operator));
+        if (cfg->operator_ && *cfg->operator_){
+            b->operator_ = fixsemi(g_strdup(cfg->operator_));
         }else{
-            b->operator = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
+            b->operator_ = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
         }
         b->stxeq = g_strdup(confb->stxeq);
         b->spowe = g_strdup(confb->spowe);
@@ -229,27 +260,45 @@ struct band *init_band(struct config_band *confb,
         b->mope1 = g_strdup(confb->mope1);
         b->mope2 = g_strdup(confb->mope2);
         b->remarks = g_strdup(confb->remarks);
-		b->skedqrg = g_strdup(confb->skedqrg);
+        b->skedqrg = g_strdup(confb->skedqrg);
+        b->band_sw = confb->band_sw;
+
+/*        struct zstring *zs = config_band_format(confb);
+        dbg("zs='%s'\n", zs->str);
+        struct config_band *cb2 = config_band_parse(zs);
+        printf("%p\n", cb2);
+        */
     }
     
     if (opt_band){ /* usually loading contest */
         GHashTable *hash = opt_band;
         gchar *ccc;
+		struct config_band *tmpconfb;
 
         STORE_HASH_STR(b, pband);   
         STORE_HASH_CHR(b, bandchar);
-            if (!b->bandchar){
-                struct config_band *tmpconfb;
-                tmpconfb=get_config_band_by_pband(b->pband);
-                if (tmpconfb) b->bandchar=tmpconfb->bandchar;
-            }
-        STORE_HASH_INT(b, mode);   
-        STORE_HASH_STR(b, operator);
-            if (!b->operator){
-                b->operator = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
+        tmpconfb=get_config_band_by_pband(b->pband);
+        if (tmpconfb){
+            if (!b->bandchar) b->bandchar=tmpconfb->bandchar;
+            if (tmpconfb->qrg_min>30000)
+                b->bandname = g_strdup(b->pband);
+            else
+                b->bandname = g_strdup(tmpconfb->adifband);
+            b->qrg_min = tmpconfb->qrg_min;
+            b->qrg_max = tmpconfb->qrg_max;
+            b->band_sw = tmpconfb->band_sw;
+        }else{
+            // contest created under other band configuration
+            b->bandname = g_strdup(b->pband);
+        }
+//        STORE_HASH_INT(b, mode);   
+        STORE_HASH_INT(gses, mode);   
+        STORE_HASH_STR(b, operator_);
+            if (!b->operator_){
+                b->operator_ = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
             }
         
-        ccc = g_hash_table_lookup(hash, "psect"); 
+        ccc = (gchar *)g_hash_table_lookup(hash, "psect"); 
         /*dbg("c=%s\n",ccc);*/
         if (isdigit(ccc[0]))
             b->psect=atoi(ccc);
@@ -268,17 +317,31 @@ struct band *init_band(struct config_band *confb,
         STORE_HASH_STR(b, mope1);
         STORE_HASH_STR(b, mope2);
  /*        STORE_HASH_STR(b, remarks); NO */
-		STORE_HASH_STR(b, skedqrg);
+        STORE_HASH_STR(b, skedqrg);
         if (!b->skedqrg) b->skedqrg=g_strdup("");
-    }    
+    }   
+    dbg("%s bandname=%s\n",__FUNCTION__, b->bandname);  
+    if (zs){
+        struct config_band *tmpconfb;
+
+        band_parse(zs, b);
+
+        if (cfg->operator_ && *cfg->operator_){
+            b->operator_ = fixsemi(g_strdup(cfg->operator_));
+        }else{
+            b->operator_ = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
+        }
+        tmpconfb=get_config_band_by_pband(b->pband);
+        if (tmpconfb) b->readonly = tmpconfb->readonly;
+    }
 
-    b->spypeers=g_ptr_array_new();
     
     b->il = g_new0(struct inputln,1);
     b->il->term = term;
     b->il->enter = process_input;
     b->il->enterdata = (void *)b;
     b->il->upconvert = 1;
+    b->il->valid_chars = VALID_CHARS; 
     b->il->readonly  = b->readonly;
     b->il->band = b;
 
@@ -293,13 +356,14 @@ struct band *init_band(struct config_band *confb,
     
     c = g_strdup_printf("%s/%c.swp",ctest->directory,b->bandchar);
             
-    clear_tmpqsos(b);
+    clear_tmpqsos(b, 1);
     
     if (!aband){
         aband=b;
         il_set_focus(aband->il);
     }
     
+//    dbg("swap=%s\n", c);
     b->swap = fopen(c,"at");
     if (!b->swap){
         log_addf(VTEXT(T_CANT_APPEND_S),c);
@@ -315,6 +379,7 @@ struct band *init_band(struct config_band *confb,
     b->qs = g_ptr_array_new();
     b->oqs = g_ptr_array_new();
     b->unfi = init_fifo(100);        
+    b->skeds = g_ptr_array_new();        
     b->statsfifo1 = init_fifo(1000);        
     
     g_ptr_array_add(ctest->bands, b);
@@ -323,6 +388,7 @@ struct band *init_band(struct config_band *confb,
     
 /*    init_spypeer(b->spypeers, "127.0.0.1:55555");
     init_spypeer(b->spypeers, "127.0.0.1:55556");*/
+    //dbg("bandname='%s'\n", b->bandname);
     
     return b;
 }
@@ -355,8 +421,20 @@ struct band *find_band_by_bandchar(char bandchar){
         }
     }
     return NULL;
-}
+    }
+        
+struct band *find_band_by_qrg(double freq){
+    struct band *b;
+    int i;
 
+    for (i=0;i<ctest->bands->len; i++){
+        b = g_ptr_array_index(ctest->bands,i);
+        if (freq < b->qrg_min * 1000.0) continue;
+        if (freq > b->qrg_max * 1000.0) continue;
+        return b;
+    }
+    return NULL;
+}
 
 struct band *init_qrv_bands(){
     int i;
@@ -367,7 +445,7 @@ struct band *init_qrv_bands(){
         confb = g_ptr_array_index(cfg->bands,i);
         if (!confb) continue;
         if (confb->qrvnow){
-            init_band(confb, NULL);
+            init_band(confb, NULL, NULL);
         }
     }
     return NULL;
@@ -375,13 +453,13 @@ struct band *init_qrv_bands(){
 
 
 
-void clear_tmpqsos(struct band *b){
+void clear_tmpqsos(struct band *b, int set_defaults){
     int i;
     char *c;
 
     if (!b) return;
 
-    b->dupe_in_tmpqso = 0;
+    b->dupe_in_tmpqso_index = -1;
     b->last_item = LI_NONE;
 
     for(i=0;i<TMP_QSOS;i++){
@@ -411,7 +489,14 @@ void clear_tmpqsos(struct band *b){
     } 
     CONDGFREE(b->unres);
     CONDGFREE(b->qrv_str);
-    wkd_tmpqso(aband, WT_CLEAR, "");
+    CONDGFREE(b->wkd_str);
+    wkd_tmpqso(b, WT_CLEAR, "");
+    CONDGFREE(b->call_played);
+    b->agcall = 0;
+    //dbg("clear_tmpqso: call_played='%s' agcall=%d\n", b->call_played, b->agcall);
+    
+    if (!set_defaults) return;
+
     if (ctest->qsoused){
         c=g_strdup_printf("%03d",b->qsos->len+1);
         ADD_TMPQSO_STRING(b,qsonrs,c,1,udummy);
@@ -420,7 +505,10 @@ void clear_tmpqsos(struct band *b){
     }
 
     default_rst_to_tmpqsos(b);
+#ifdef HAVE_SNDFILE
     ssbd_callsign(ssbd, "");
+    ssbd->code = 0;
+#endif
 
 }
 
@@ -428,18 +516,25 @@ void default_rst_to_tmpqsos(struct band *b){
     gchar *c;
     int i;
     
-	if (!ctest) return;
+    if (!ctest) return;
     for(i=0;i<TMP_QSOS;i++)  CLEAR_TMPQSO_STRING(b,rsts);
 
     /*if (!ctest->qsoused) return;*/
     
-    if (b->mode == MOD_CW_CW)
+    if (gses->mode == MOD_CW_CW)
         c=g_strdup(cfg->default_rst);
     else
         c=g_strdup(cfg->default_rs);
        
     ADD_TMPQSO_STRING(b,rsts,c,1,udummy);
     wkd_tmpqso(b, WT_RSTS, c);
+
+    if (ctest->defrstr){
+      //  for(i=0;i<TMP_QSOS;i++)  CLEAR_TMPQSO_STRING(b,rstr);
+        ADD_TMPQSO_STRING(b,rstr,c,1,udummy);
+        wkd_tmpqso(b, WT_RSTR, c);
+    }
+    
     g_free(c);
 }
        
@@ -479,6 +574,17 @@ struct qso *get_qso(struct band *b, gint i){
     return g_ptr_array_index(b->qsos, i);
 }
 
+struct qso *get_gqso(struct band *b, gint i){
+    
+    if (ctest->qsoglob){
+        if (i<0 || i >=ctest->allqsos->len) return NULL;
+        return g_ptr_array_index(ctest->allqsos, i);
+    }else{
+        if (i<0 || i >=b->qsos->len) return NULL;
+        return g_ptr_array_index(b->qsos, i);
+    }
+}
+
 struct qso *get_qso_by_callsign(struct band *b, gchar *callsign){
     struct qso *qso;
     int i;
@@ -498,22 +604,41 @@ struct qso *get_qso_by_callsign(struct band *b, gchar *callsign){
     return NULL;
 }
 
+int get_qso_index_by_callsign(struct band *b, gchar *callsign){
+    struct qso *qso;
+    int i;
+    char raw[25], qraw[25];
+
+    get_raw_call(raw, callsign);
+    
+              /* TODO callsigns in hash */
+    for(i=0;i<b->qsos->len; i++){
+        qso = g_ptr_array_index(b->qsos, i);
+
+        if (qso->error) continue;
+        
+        get_raw_call(qraw, qso->callsign);
+        if (strcasecmp(raw, qraw)==0) return i;
+    }
+    return -1;
+}
+
 struct qso *get_qso_by_id(struct band *b, gchar *source, gint ser_id){
     struct qso *qso;
     int i;
             
-	/* TODO id's in hash */
-	/*dbg("get_qso_by_id(src=%s, ser_id=%d) \n", source, ser_id);*/
+    /* TODO id's in hash */
+    /*dbg("get_qso_by_id(src=%s, ser_id=%d) \n", source, ser_id);*/
     for(i=0;i<b->qsos->len; i++){
-        qso = g_ptr_array_index(b->qsos, i);
+        qso = (struct qso *)g_ptr_array_index(b->qsos, i);
 
-		/*dbg("		%s %s %d ",qso->callsign, qso->source, qso->ser_id);*/
+        /*dbg("     %s %s %d ",qso->callsign, qso->source, qso->ser_id);*/
         if (ser_id != qso->ser_id) continue;
         if (strcasecmp(source, qso->source)!=0) continue;
-		/*dbg("		OK\n");*/
+        /*dbg("     OK\n");*/
         return qso;
     }
-	/*dbg("		NULL\n");*/
+    /*dbg("     NULL\n");*/
     return NULL;
 }
 
@@ -522,7 +647,7 @@ struct qso *get_qso_by_qsonr(struct band *b, gint qsonrs){
     int i;
               /* TODO id's in hash */
     for(i=0;i<b->qsos->len; i++){
-        qso = g_ptr_array_index(b->qsos, i);
+        qso = (struct qso *)g_ptr_array_index(b->qsos, i);
 
         if (qsonrs != atoi(qso->qsonrs)) continue;
         return qso;
@@ -544,8 +669,8 @@ void add_error(struct band *b, gchar *remark){
     gmtime_r(&now, &utc);
            
     eq = g_new0(struct qso, 1);
-    eq->source   = g_strdup(net->myid);
-    eq->operator = g_strdup(b->operator);
+    eq->source    = g_strdup(net->myid);
+    eq->operator_ = g_strdup(b->operator_);
     if (b->tmpqsos[0].date_str){
         eq->date_str = g_strdup(b->tmpqsos[0].date_str);
         eq->time_str = g_strdup(b->tmpqsos[0].time_str);
@@ -562,12 +687,14 @@ void add_error(struct band *b, gchar *remark){
     eq->locator  = g_strdup(b->tmpqsos[0].locator&&b->tmpqsos[0].ulocator?b->tmpqsos[0].locator:"");    
     eq->error    = 1;
     eq->remark   = g_strdup(remark?remark:"");
+    eq->qrg      = 0.0;
     eq->ser_id   = -1;  /* computed by add_qso_to_index */
     
     add_qso(b, eq);
     stats_thread_join(b);
-    update_stats(b, b->stats, eq);
-	minute_stats(b);
+    update_stats(b, b->stats, eq);  
+    recalc_allb_stats();
+    minute_stats(b);
     recalc_statsfifo(b);
     
     if (b->qsos->len+1 != atoi(b->tmpqsos[0].qsonrs)){
@@ -613,7 +740,7 @@ int add_swap(struct band *band, gchar *s){
 }
 
 
-
+//tern int *pnn;
 void activate_band(struct band *b){
     struct event ev={EV_RESIZE, 0, 0, 0};
     
@@ -622,7 +749,7 @@ void activate_band(struct band *b){
     if (!b) return;
 
 
-    
+ // dbg("pnn=%d\n", *pnn);
     il_unset_focus(aband->il);
     aband=b;
     sw_all_func(&ev, 1);
@@ -633,12 +760,37 @@ void activate_band(struct band *b){
     get_band_qs(aband, aband->il->cdata);
     get_oband_qs(aband, aband->il->cdata);
     get_cw_qs(aband->il->cdata);
-    
-    if (gses->ontop && gses->ontop->type == SWT_STAT)
-        recalc_statsfifo(b);
+    get_hf_dxc(aband->il->cdata);
+    if (gses->ontop){
+        switch(gses->ontop->type){
+            case SWT_STAT:
+                recalc_statsfifo(b);
+                break;
+            case SWT_QRV:
+                sw_qrv_sort(qrvdb);
+                break;
+            default:
+                break;
+        }
+    }
 
+
+    cwdaemon_band(cwda, b->band_sw);
+    default_rst_to_tmpqsos(aband);
+
+#ifdef HAVE_HAMLIB
+    if (trig){
+        if (cfg->rig_qrg_t2r){
+            if (aband->qrg != 0){
+                trig_set_qrg(trig, aband->qrg);
+            }
+            trig_resend_freq(trig);
+        }
+    }
+#endif
     redraw_later();
-	maps_reload();
+    maps_reload();
+    chart_reload();
 }
 
 
@@ -764,21 +916,21 @@ int compare_stamp(const void *a, const void *b){
     if (!*qa && !*qb) return 0;
     if (!*qa) return -1;
     if (!*qb) return +1;
-    return (*qa)->stamp - (*qb)->stamp;
+    return (int)((*qa)->stamp - (*qb)->stamp);
 }
 
 /* only for backward compatibility with version without ser_id in edi file */
 void foreach_source_recalc_ser_id(gpointer key, gpointer value, gpointer data){
-	int i;
-	struct qso *q;
+    int i;
+    struct qso *q;
     GIndexArray *ia;
 
     ia = (GIndexArray *)value;
 
-	for (i=0; i<ia->len; i++){
-		q=(struct qso *)g_index_array_index(ia, i);
-		q->ser_id=i;
-	}
+    for (i=0; i<ia->len; i++){
+        q=(struct qso *)g_index_array_index(ia, i);
+        q->ser_id=i;
+    }
 } 
 
 void foreach_source_qsort_by_stamp(gpointer key, gpointer value, gpointer data){
@@ -794,7 +946,7 @@ void foreach_source_print(gpointer key, gpointer value, gpointer data){
     int i;
 
     ia = (GIndexArray *)value;
-    dbg_qsos("source: %s", key);
+    trace(cfg->trace_qsos, "source: %s", key);
 
     for (i=0; i<ia->len; i++){
         q = (struct qso *) g_index_array_index(ia, i);   
@@ -803,12 +955,21 @@ void foreach_source_print(gpointer key, gpointer value, gpointer data){
 }
 
 void dump_all_sources(struct contest *ctest){
+    int i;
+    struct qso *q;
+
+    return;
     if (!ctest){
-        dbg_qsos("dump_all_sources(NULL)\n");
+        trace(cfg->trace_qsos, "dump_all_sources(NULL)\n");
         return;
     }
     g_hash_table_foreach(ctest->bystamp, foreach_source_print, NULL);
-    dbg_qsos("---end---");
+    dbg("ctest->allqsos:\n");
+    for (i=0; i<ctest->allqsos->len; i++){
+        q = (struct qso *) g_index_array_index(ctest->allqsos, i);   
+        dump_qso(q, "foreach_source_print");
+    }
+    trace(cfg->trace_qsos, "---end---");
 }
 
 void add_qso_to_index(struct qso *q, int qsort_if_needed){
@@ -823,15 +984,18 @@ void add_qso_to_index(struct qso *q, int qsort_if_needed){
         g_hash_table_insert(ctest->bystamp, g_strdup(q->source), ia);
     }
         
-	if (q->ser_id==-1) q->ser_id=ia->len; /* ia is NOT sorted by ser_id! */
+    if (q->ser_id==-1) q->ser_id=ia->len; /* ia is NOT sorted by ser_id! */
     g_index_array_add(ia, q);
 
     if (qsort_if_needed && ia->len >= 2){
         ql = (struct qso *) g_index_array_index(ia, ia->len-2);
         if (ql->stamp > q->stamp){ /* q is not latest */
             g_index_array_qsort(ia, compare_stamp);
-		}
+        }
     }
+    g_index_array_add(ctest->allqsos, q);
+
+    qrv_set_wkd(qrvdb, q);
 }
 
 void remove_qso_from_index(struct qso *q){
@@ -840,6 +1004,7 @@ void remove_qso_from_index(struct qso *q){
     
     ia = (GIndexArray *) g_hash_table_lookup(ctest->bystamp, q->source);
     ret=g_index_array_remove(ia, q);
+    ret=g_index_array_remove(ctest->allqsos, q);
     dbg("remove: ret=%d\n", ret);
 }
 
@@ -855,8 +1020,8 @@ void compute_qrbqtf(struct qso *q){
         q->qtf = 0;
     }
     qtfrad=qtf*MY_PI/180.0;
-    q->kx =   q->qrb*sin(qtfrad);
-    q->ky = - q->qrb*cos(qtfrad);
+    q->kx = (int)(  q->qrb*sin(qtfrad));
+    q->ky = (int)(- q->qrb*cos(qtfrad));
     /*dbg("compute_qrbqtf(%s) %s->%s qrb=%f qtf=%d kx=%d ky=%d\n", 
             q->callsign, ctest->pwwlo, q->locator, q->qrb, q->qtf, q->kx, q->ky); */
 }
@@ -1042,7 +1207,7 @@ GPtrArray *get_band_qs(struct band *band, gchar *str){
     for (i=0;i<band->qsos->len;i++){
         qso = (struct qso *)g_ptr_array_index(band->qsos, i);
         if (qso->error || qso->dupe) continue;
-        if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - band->spypeers->len - 1) break;
+        if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - ctest->spypeers->len - 1) break;
         
         if (qso->callsign && my_strstr(qso->callsign, qs->str)){
             if (qso->locator)
@@ -1100,7 +1265,7 @@ GPtrArray *get_oband_qs(struct band *oband, gchar *str){
         for (i=0;i<band->qsos->len;i++){
             qso = (struct qso *)g_ptr_array_index(band->qsos, i);
             if (qso->error || qso->dupe) continue;
-            if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - oband->spypeers->len - 1) break;
+            if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - ctest->spypeers->len - 1) break;
             /* FIXME contest without locator */
             if ((qso->callsign && my_strstr(qso->callsign, qs->str))||
                 (qso->locator  && my_strstr(qso->locator, qs->str))){    
@@ -1111,7 +1276,7 @@ GPtrArray *get_oband_qs(struct band *oband, gchar *str){
                     char d[20], *pd, *line;
                     
                     safe_strncpy(d, line=g_ptr_array_index(qs->result1,j), 10);
-                    pd = index(d, ' ');
+                    pd = strchr(d, ' ');
                     if (pd) *pd='\0';
                     
                     if (strcmp(d, qso->callsign)==0){ 
@@ -1148,7 +1313,6 @@ gchar *find_wwl_by_oband(struct band *oband, gchar *call){
     if (ctest->bands->len<2) return NULL;
     if (!ctest->wwlused) return NULL;
     
-    ST_START;
     for (j=0;j<ctest->bands->len;j++){
         struct band *band;
         
@@ -1163,12 +1327,39 @@ gchar *find_wwl_by_oband(struct band *oband, gchar *call){
             return qso->locator; /* can be NULL */
         }
     }
+    return NULL;
+}
+
+
+gchar *find_exc_by_oband(struct band *oband, gchar *call){
+    int i, j;
+    struct qso *qso;
+    static char raw[256];
+
+    if (ctest->bands->len<2) return NULL;
+    if (ctest->excused != 1) return NULL;
+    
+    ST_START;
+    for (j=0;j<ctest->bands->len;j++){
+        struct band *band;
+        
+        band = (struct band *)g_ptr_array_index(ctest->bands, j);
+        if (band==oband) continue;
+        
+        for (i=0;i<band->qsos->len;i++){
+            qso = (struct qso *)g_ptr_array_index(band->qsos, i);
+            if (qso->error || qso->dupe) continue;
+            get_raw_call(raw, qso->callsign);
+            if (strcasecmp(raw,call)) continue;
+            return qso->exc; /* can be NULL */
+        }
+    }
     ST_STOP;
     return NULL;
 }
 
 
-char *get_raw_call(char *buf, char *call){
+char *get_raw_call(char *buf, const char *call){
     char s[20], *c;
     char *token_ptr;
 
@@ -1185,6 +1376,13 @@ char *get_raw_call(char *buf, char *call){
     return buf;
 }
 
+int is_rover(const char *callsign){
+    int len = strlen(callsign);
+    if (len < 2) return 0;
+    if (callsign[len - 2] != '/') return 0;
+    if (toupper(callsign[len - 1]) != 'R') return 0;
+    return 1;
+}
 
 int get_psect_int(char *psect){
     if (strcasecmp(psect, "single")==0) return 1;
@@ -1194,11 +1392,11 @@ int get_psect_int(char *psect){
 
 void dump_qso(struct qso *q, char *desc){
     if (!q){
-        dbg_qsos("  NULL");
+        trace(cfg->trace_qsos, "  NULL");
         return;
     }
-   /* dbg_qsos("  %8s %c%03d %s:%d.%05d  %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, desc);*/
-    dbg_qsos("  %8s %c%03d %s:%d.%05d  (qsop=%d error=%d dupe=%d) %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, q->qsop, q->error, q->dupe,  desc);
+   /* trace(cfg->trace_qsos, "  %8s %c%03d %s:%d.%05d  %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, desc);*/
+    trace(cfg->trace_qsos, "  %8s %c%03d %s:%d.%05d  (qsop=%d error=%d dupe=%d) %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, q->qsop, q->error, q->dupe,  desc);
 }
 
 void invalidate_tmpqso(struct band *b, struct qso *q){
@@ -1231,8 +1429,12 @@ void free_ctest(){
     if (!ctest) return;
 
 
-	kill_timer(ctest->minute_timer_id);
-	
+    kill_timer(ctest->minute_timer_id);
+    if (gses->skedw_timer_id > 0) {
+        kill_timer(gses->skedw_timer_id); 
+        gses->skedw_timer_id = 0;
+    }
+
     dump_all_sources(ctest);
     
     if (gses->last_cq_timer_id) {
@@ -1240,9 +1442,11 @@ void free_ctest(){
         gses->last_cq_timer_id = 0;
     }
     
+    clear_qrv(qrvdb);
+    chart_clear_all();
 
     for (i=0;i<ctest->bands->len; i++){
-        free_band(g_ptr_array_index(ctest->bands,i));
+        free_band((struct band*)g_ptr_array_index(ctest->bands,i));
     }
 
     if (ctest->descfile)  fclose(ctest->descfile);
@@ -1259,6 +1463,7 @@ void free_ctest(){
     CONDGFREE(ctest->pexch);
     CONDGFREE(ctest->default_rs);
     CONDGFREE(ctest->default_rst);
+    CONDGFREE(ctest->excname);
     
     CONDGFREE(ctest->rname);
     CONDGFREE(ctest->rcall);
@@ -1277,16 +1482,19 @@ void free_ctest(){
     CONDGFREE(ctest->);
   */
 
+    g_hash_table_foreach_remove(ctest->allb_prefs, free_gpointer_item, NULL);
     g_hash_table_foreach(ctest->bystamp, foreach_free_bystamp, NULL); 
+    g_index_array_free(ctest->allqsos, TRUE);
     g_ptr_array_free(ctest->bands, TRUE);
-    if (ctest->redraw_timer_id) kill_timer(ctest->redraw_timer_id);
+    free_spypeers(ctest->spypeers);
     g_free(ctest);
     ctest=NULL;
     aband=NULL;
     
     set_ctest_title();
-    cq_abort(0);
+    cq_abort(1);
     if (!terminate) maps_reload();
+    chart_reload();
 }
 
 
@@ -1297,18 +1505,17 @@ void free_band(struct band *b){
     struct event ev = {EV_ABORT, 0,0,0};
 
     stats_thread_kill(b);
-    clear_tmpqsos(b);
-    if (b->tmpqsos[0].qsonrs) g_free(b->tmpqsos[0].qsonrs);
-    if (b->tmpqsos[0].rsts)   g_free(b->tmpqsos[0].rsts);
+    clear_tmpqsos(b, 0);
 
     for (i=0;i<b->qsos->len;i++){
         struct qso *qso;
-        qso=g_ptr_array_index(b->qsos, i);
+        qso=(struct qso *)g_ptr_array_index(b->qsos, i);
         free_qso(qso);
     }
     
     CONDGFREE(b->pband);
-    CONDGFREE(b->operator);
+    CONDGFREE(b->bandname);
+    CONDGFREE(b->operator_);
     CONDGFREE(b->stxeq);
     CONDGFREE(b->spowe);
     CONDGFREE(b->srxeq);
@@ -1317,13 +1524,21 @@ void free_band(struct band *b){
     CONDGFREE(b->mope1);
     CONDGFREE(b->mope2);
     CONDGFREE(b->remarks);
-	CONDGFREE(b->skedqrg);
+    CONDGFREE(b->skedqrg);
     /*CONDGFREE(b->);
     CONDGFREE(b->);
     CONDGFREE(b->);
     CONDGFREE(b->);*/
     
     
+    if (b->skeds)     {
+        for (i=0;i<b->skeds->len;i++){
+            struct sked *sked;
+            sked=(struct sked*)g_ptr_array_index(b->skeds, i);
+            free_sked(sked);
+        }
+        g_ptr_array_free(b->skeds, TRUE);
+    }
     
     inputln_func(b->il,&ev);
     g_free(b->il);
@@ -1338,7 +1553,6 @@ void free_band(struct band *b){
     if (b->unfi)      free_fifo(b->unfi);
     if (b->statsfifo1) free_fifo(b->statsfifo1);
     if (b->ctrlsp)    g_free(b->ctrlsp);
-    free_spypeers(b->spypeers);
     CONDGFREE(b->tmplocqso.locator);
     g_free(b);
 }
@@ -1346,7 +1560,7 @@ void free_band(struct band *b){
 
 void free_qso(struct qso *qso){
     CONDGFREE(qso->source);
-    CONDGFREE(qso->operator);
+    CONDGFREE(qso->operator_);
     CONDGFREE(qso->date_str);
     CONDGFREE(qso->time_str);
     CONDGFREE(qso->callsign);
@@ -1363,18 +1577,183 @@ void free_qso(struct qso *qso){
 
 
 void dirty_band(struct band *band) {
-    dbg_qsos("DIRTY_BAND %c",band->bandchar);
+    trace(cfg->trace_qsos, "DIRTY_BAND %c",band->bandchar);
     band->dirty_save = 1;
     band->dirty_stats = 1;
     band->dirty_statsf = 1;
 };
 
 enum modes get_mode(void){
-	if (aband) return aband->mode;
-	else return gses->mode;
+    
+   /* if (aband) return aband->mode;
+    else*/ return gses->mode;
 }
 
 void set_mode(enum modes mode){
-	if (aband) aband->mode=mode;
-	else gses->mode=mode;
+    /*if (aband) aband->mode=mode;
+    else*/ gses->mode=mode;
+
+    dbg("set_mode(%d)\n", mode);
+#ifdef HAVE_HAMLIB
+    if (!trig) return;
+    if (!cfg->rig_mode_t2r) return;
+    switch (mode){
+        case MOD_SSB_SSB:
+            //trig_set_mode(trig, RIG_MODE_USB);
+            //trig_set_qrg(trig, trig->qrg - cfg->rig_ssbcw_shift);
+            trig_set_mode_qrg(trig, RIG_MODE_USB, trig->qrg - cfg->rig_ssbcw_shift);
+            break;
+        case MOD_CW_CW:
+            //trig_set_mode(trig, RIG_MODE_CW);
+            //trig_set_qrg(trig, trig->qrg + cfg->rig_ssbcw_shift);
+            trig_set_mode_qrg(trig, RIG_MODE_CW, trig->qrg + cfg->rig_ssbcw_shift);
+            break;
+        default:
+            /* nic jineho se nanastavuje */
+            break;
+                    
+    }
+#endif    
 }
+
+struct zstring *contest_format(struct contest *ctest){
+    gchar *ints;
+    struct zstring *zs;
+
+    ints = g_strdup_printf("%d;%d;%d;%d;"
+                           "%d;%d;%d;%d;"
+                           "%d;%d;%d;%d;"
+                           "%d;%d;%d;%d;"
+                           "%d;%d;%d;%d;"
+                           "%d;%d",
+        ctest->qsoused, ctest->qsomult, ctest->qsoglob, ctest->wwlused, 
+        ctest->wwlcfm,  ctest->wwlbonu, ctest->wwlmult, ctest->excused,
+        ctest->exctype, ctest->excbonu, ctest->excmult, ctest->exccfm, 
+        ctest->dxcbonu, ctest->dxcmult, ctest->rstused, ctest->defrstr, 
+        ctest->qsop_method, ctest->total_method, ctest->tttype, ctest->prefmult, 
+        ctest->prefglob, ctest->expmode);
+            
+            
+    zs = zconcatesc(ints, ctest->tname, ctest->cdate, ctest->pcall, 
+        ctest->pclub, ctest->pwwlo, ctest->pexch, ctest->default_rst, 
+        ctest->default_rs, ctest->rname, ctest->rcall, ctest->radr1, 
+        ctest->radr2, ctest->rpoco, ctest->rcity, ctest->rcoun, 
+        ctest->rphon, ctest->rhbbs, ctest->excname, NULL);
+    return zs;
+}
+
+void ctest_parse(struct zstring *zstr, struct contest *ctest){
+    gchar *c;
+    struct zstring *zsint;
+
+    c = ztokenize(zstr, 1);
+    zsint = zstrdup(c);
+    ctest->qsoused      = atoi(ztokenize(zsint, 1));
+    ctest->qsomult      = atoi(ztokenize(zsint, 0));
+    ctest->qsoglob     = atoi(ztokenize(zsint, 0));
+    ctest->wwlused      = atoi(ztokenize(zsint, 0));
+    ctest->wwlcfm       = atoi(ztokenize(zsint, 0));
+    ctest->wwlbonu      = atoi(ztokenize(zsint, 0));
+    ctest->wwlmult      = atoi(ztokenize(zsint, 0));
+    ctest->excused      = atoi(ztokenize(zsint, 0));
+    ctest->exctype      = atoi(ztokenize(zsint, 0));
+    ctest->excbonu      = atoi(ztokenize(zsint, 0));
+    ctest->excmult      = atoi(ztokenize(zsint, 0));
+    ctest->exccfm       = atoi(ztokenize(zsint, 0));
+    ctest->dxcbonu      = atoi(ztokenize(zsint, 0));
+    ctest->dxcmult      = atoi(ztokenize(zsint, 0));
+    ctest->rstused      = atoi(ztokenize(zsint, 0));
+    ctest->defrstr      = atoi(ztokenize(zsint, 0));
+    ctest->qsop_method  = atoi(ztokenize(zsint, 0));
+    ctest->total_method = atoi(ztokenize(zsint, 0));
+    ctest->tttype       = atoi(ztokenize(zsint, 0));
+    ctest->prefmult     = atoi(ztokenize(zsint, 0));
+    ctest->prefglob     = atoi(ztokenize(zsint, 0));
+    ctest->expmode      = atoi(ztokenize(zsint, 0));
+
+    STORE_STR3(ctest, tname, zstr, 0);
+    STORE_STR3(ctest, cdate, zstr, 0);
+    STORE_STR3(ctest, pcall, zstr, 0);
+    STORE_STR3(ctest, pclub, zstr, 0);
+    STORE_STR3(ctest, pwwlo, zstr, 0);
+    STORE_STR3(ctest, pexch, zstr, 0);
+    STORE_STR3(ctest, default_rst, zstr, 0);
+    STORE_STR3(ctest, default_rs, zstr, 0);
+    STORE_STR3(ctest, rname, zstr, 0);
+    STORE_STR3(ctest, rcall, zstr, 0);
+    STORE_STR3(ctest, radr1, zstr, 0);
+    STORE_STR3(ctest, radr2, zstr, 0);
+    STORE_STR3(ctest, rpoco, zstr, 0);
+    STORE_STR3(ctest, rcity, zstr, 0);
+    STORE_STR3(ctest, rcoun, zstr, 0);
+    STORE_STR3(ctest, rphon, zstr, 0);
+    STORE_STR3(ctest, rhbbs, zstr, 0);
+    STORE_STR3(ctest, excname, zstr, 0);
+    
+    zfree(zsint);
+}
+struct zstring *band_format(struct band *b){
+    gchar *ints;
+    struct zstring *zs;
+    
+    ints = g_strdup_printf("%c;%d;%d",
+            b->bandchar, b->psect, b->readonly);
+    
+    zs = zconcatesc(ints, b->pband, b->bandname, b->stxeq, 
+                    b->spowe, b->srxeq, b->sante, b->santh, 
+                    b->mope1, b->mope2, b->remarks, b->skedqrg, 
+                    NULL);
+    
+    g_free(ints);
+    return zs;
+}
+
+
+
+struct band *band_parse(struct zstring *zstr, struct band *b){
+    gchar *c;
+    struct zstring *zsint;
+
+    c = ztokenize(zstr, 1);
+    zsint = zstrdup(c);
+    c = ztokenize(zsint, 1);
+    b->bandchar    = c[0];
+    b->psect       = atoi(ztokenize(zsint, 0));
+    b->readonly    = atoi(ztokenize(zsint, 0));
+
+    STORE_STR3(b, pband, zstr, 0);
+    STORE_STR3(b, bandname, zstr, 0);
+    STORE_STR3(b, stxeq, zstr, 0);
+    STORE_STR3(b, spowe, zstr, 0);
+    STORE_STR3(b, srxeq, zstr, 0);
+    STORE_STR3(b, sante, zstr, 0);
+    STORE_STR3(b, santh, zstr, 0);
+    STORE_STR3(b, mope1, zstr, 0);
+    STORE_STR3(b, mope2, zstr, 0);
+    STORE_STR3(b, remarks, zstr, 0);
+    STORE_STR3(b, skedqrg, zstr, 0);
+    
+    zfree(zsint);
+    return b;
+}
+
+int compare_date_time_qsonrs(const void *a, const void *b){
+    struct qso **qa, **qb;
+    int ret;
+
+    qa = (struct qso **)a;
+    qb = (struct qso **)b;
+    
+    if (!*qa && !*qb) return 0;
+    if (!*qa) return -1;
+    if (!*qb) return +1;
+
+    ret = strcmp((*qa)->date_str, (*qb)->date_str);
+    if (ret) return ret;
+    
+    ret = strcmp((*qa)->time_str, (*qb)->time_str);
+    if (ret) return ret;
+    
+    return atoi((*qa)->qsonrs) - atoi((*qb)->qsonrs);
+}
+
