@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,10 +14,18 @@ struct rotar *rota,*rotb;
 int hdkeyb_disp_ofs[HDKEYB_ROTARS] = {0, 8, 64, 72};
 struct hdkeyb *hdkeyb;
 
-GPtrArray *rotars2;
-MUTEX_DEFINE(rotars2);
+GPtrArray *rotars;
+MUTEX_DEFINE(rotars);
 struct rotar *rotar;
 
+gpointer rot_hamlib_main(gpointer xxx);
+
+static int rot_print_model_list(const struct rot_caps *caps, void *data)
+{
+    log_addf("%-6d %-14s  %-20s  %s\n", caps->rot_model, caps->mfg_name, caps->model_name, caps->version);
+    return 1;  /* !=0, we want them all ! */
+}
+
 
 struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
     struct rotar *rot;
@@ -56,10 +64,27 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 #ifdef HAVE_HAMLIB
         case ROT_HAMLIB:
             rot->rot = rot_init(crot->rot_model);
-            safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, FILPATHLEN);
+			if (!rot->rot){
+				log_addf("Unknown rotarot number %d or initialization error", crot->rot_model);
+				g_free(rot);
+
+				log_addf("Registrated rotators:");
+				rot_load_all_backends();
+				rot_list_foreach(rot_print_model_list, NULL);
+
+				return NULL;
+			}
+			if (crot->rot_model > 0){
+				safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, FILPATHLEN);
+				//rot->rot->state.rotport.timeout = 2000;
+			}
 
             rot->hl_azim = -1;
             rot->hl_model = crot->rot_model;
+			rot->hl_thread = g_thread_create(rot_hamlib_main, (gpointer)rot, TRUE, NULL); 
+			if (!rot->hl_thread) {
+				log_addf("Can't create hamlib rotar thread\n");
+			}
             break;
 #endif            
         default:
@@ -92,6 +117,13 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 }
 
 void free_rotar(struct rotar *rot){
+	if (rot->hl_thread){
+        rot->hl_thread_break = 1;
+        dbg("join hamlib rotar...\n");
+        g_thread_join(rot->hl_thread);
+        dbg("done\n");
+        rot->hl_thread=0;
+    }
     g_free(rot->desc);
     g_free(rot->netid);
     g_free(rot->rem_rotstr);
@@ -105,17 +137,17 @@ int init_rotars(){
     int i;
     char rotchar='A';
 
-    
-    MUTEX_INIT(rotars2);
-    rotars2=g_ptr_array_new();
+
+    MUTEX_INIT(rotars);
+    rotars=g_ptr_array_new();
     for (i=0;i<cfg->crotars->len;i++){
         cfgrot=(struct config_rotar*)g_ptr_array_index(cfg->crotars, i);
         if (!cfgrot) continue;
         rot=init_rotar(cfgrot, rotchar++);
         if (!rot) continue;
-        MUTEX_LOCK(rotars2);
-        g_ptr_array_add(rotars2, rot); 
-        MUTEX_UNLOCK(rotars2);
+        MUTEX_LOCK(rotars);
+        g_ptr_array_add(rotars, rot); 
+        MUTEX_UNLOCK(rotars);
     }
 #ifdef HAVE_SDL
     rot_update_colors();
@@ -127,14 +159,14 @@ int free_rotars(void){
     int i;
     struct rotar *rot;
 
-    MUTEX_LOCK(rotars2);
-    for (i=0;i<rotars2->len;i++){
-        rot=g_ptr_array_index(rotars2, i);
+    MUTEX_LOCK(rotars);
+    for (i=0;i<rotars->len;i++){
+        rot = (struct rotar *)g_ptr_array_index(rotars, i);
         free_rotar(rot);
     }
-    MUTEX_UNLOCK(rotars2);
-    g_ptr_array_free(rotars2,1);
-    rotars2=NULL;
+    MUTEX_UNLOCK(rotars);
+    g_ptr_array_free(rotars,1);
+    rotars=NULL;
     return 0;
 }
 
@@ -143,10 +175,10 @@ int rot_update_colors(){
 	int i;
 	struct rotar *rot;
 	
-    if (!sdl || !rotars2) return -1;
-    MUTEX_LOCK(rotars2);
-    for (i=0;i<rotars2->len;i++){
-        rot=g_ptr_array_index(rotars2, i);
+    if (!sdl || !rotars) return -1;
+    MUTEX_LOCK(rotars);
+    for (i=0;i<rotars->len;i++){
+        rot = (struct rotar *)g_ptr_array_index(rotars, i);
 	
 		switch(i%4){
 			case 0:
@@ -167,7 +199,7 @@ int rot_update_colors(){
 				break;
 		}
 	}
-    MUTEX_UNLOCK(rotars2);
+    MUTEX_UNLOCK(rotars);
 	return 0;
 }
 #endif
@@ -216,12 +248,12 @@ struct config_rotar *get_config_rotar_by_number(GPtrArray *crots, int nr){
 struct rotar *get_rotar(int nr){
     struct rotar *rot;
 
-    if (!rotars2) return NULL;
-    if (nr>=rotars2->len) return NULL;
+    if (!rotars) return NULL;
+    if (nr>=rotars->len) return NULL;
 
-    MUTEX_LOCK(rotars2);
-    rot = (struct rotar*) g_ptr_array_index(rotars2, nr);
-    MUTEX_UNLOCK(rotars2);
+    MUTEX_LOCK(rotars);
+    rot = (struct rotar*) g_ptr_array_index(rotars, nr);
+    MUTEX_UNLOCK(rotars);
     return rot;
 }
 
@@ -229,14 +261,14 @@ struct rotar *get_rotar_by_rotstr(char *rotstr){
     int i;
     struct rotar *rot = NULL;
 
-    if (!rotars2) return NULL;
-    MUTEX_LOCK(rotars2);
-    for (i=0;i<rotars2->len;i++){
-        rot=(struct rotar *)g_ptr_array_index(rotars2, i);
+    if (!rotars) return NULL;
+    MUTEX_LOCK(rotars);
+    for (i=0;i<rotars->len;i++){
+        rot=(struct rotar *)g_ptr_array_index(rotars, i);
         if (rot->rotchar!=rotstr[0]) continue;
         break;
     }
-    MUTEX_UNLOCK(rotars2);
+    MUTEX_UNLOCK(rotars);
     return rot;
 }
 
@@ -244,17 +276,17 @@ struct rotar *get_rotar_by_netid_rotstr(char *netid, char *rotstr){
     int i;
     struct rotar *rot = NULL;
 
-    if (!rotars2) return NULL;
-    MUTEX_LOCK(rotars2);
-    for (i=0;i<rotars2->len;i++){
-        rot=(struct rotar *)g_ptr_array_index(rotars2, i);
+    if (!rotars) return NULL;
+    MUTEX_LOCK(rotars);
+    for (i=0;i<rotars->len;i++){
+        rot=(struct rotar *)g_ptr_array_index(rotars, i);
         //dbg("  rot->type=%d rotchar=%s netid=%s\n", rot->type, rot->rem_rotstr, rot->netid);
         if (rot->type != ROT_REMOTE) continue;
         if (strcmp(rot->netid, netid)!=0) continue;
         if (strcmp(rot->rem_rotstr, rotstr)!=0) continue;
         break;
     }
-    MUTEX_UNLOCK(rotars2);
+    MUTEX_UNLOCK(rotars);
     return rot;
 }
 
@@ -488,7 +520,7 @@ int rotar_func(struct dialog_data *data, struct event *ev){
             if (rotchar<'A' || rotchar>'Z') {
                 return -1;
             }
-            if (rotchar<'A' || rotchar-'A'>=rotars2->len) {
+            if (rotchar<'A' || rotchar-'A'>=rotars->len) {
                 return EVENT_PROCESSED;
             }
             /*dbg("rotar %c\n", rotchar);*/
@@ -554,7 +586,7 @@ void menu_rotar(cba_t cba){
 }
 
 #ifdef HAVE_HAMLIB
-gpointer rot_haml_main(gpointer xxx){
+gpointer rot_hamlib_main(gpointer xxx){
     int ret;
     struct rotar *rot = (struct rotar*)xxx;
     azimuth_t azimuth;
@@ -578,7 +610,10 @@ gpointer rot_haml_main(gpointer xxx){
         }
         
         if (rot->hl_azim >= 0){
-            rot_move(rot->rot, rot->hl_azim, 100);
+            azimuth_t az = rot->hl_azim;
+            elevation_t el = 0;
+
+            ret = rot_set_position(rot->rot, az, el);
             rot->hl_azim = -1;
         }
 
