@@ -412,7 +412,7 @@ void sdl_redraw_screen(void){
 
     if (term->last_screen!=DUMMY && (term->cx!=term->lcx || term->cy!=term->lcy)){
         term->last_screen[term->cx+term->cy*term->x]=0;
-        if (term->lcx >= 0 && term->lcy >= 0) term->last_screen[term->lcx+term->lcy*term->x]=0;
+        if (term->lcx >= 0 && term->lcy >= 0 && term->lcx < term->x && term->lcy < term->y) term->last_screen[term->lcx+term->lcy*term->x]=0;
         term->lcx=term->cx;
         term->lcy=term->cy;
     }
