@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -14,13 +14,16 @@
 struct timeval start;
 
 struct thread {
-    void (*read_func)(union cba_t);
-    void (*write_func)(union cba_t);
-    void (*error_func)(union cba_t);
-    union cba_t cba;
+    void (*read_func)(cba_t);
+    void (*write_func)(cba_t);
+    void (*error_func)(cba_t);
+    cba_t cba;
 #ifdef LEAK_DEBUG_LIST
 	char *file;
 	int line;
+    char *read_fname;
+    char *write_fname;
+    char *error_fname;
 #endif	
 };
 
@@ -38,14 +41,20 @@ int w_max;
 
 int timer_id = 1;
 
+/*int timer_redraw_screen_id = 0;
+int screen_drawed = 0;
+*/
 struct timer {
     struct timer *next;
     struct timer *prev;
     ttime interval;
 
-    void (*func)(union cba_t);
-    union cba_t cba;
+    void (*func)(cba_t);
+    cba_t cba;
     int id;
+#ifdef LEAK_DEBUG_LIST
+    char *fname;    
+#endif
 };
 
 /*struct list_head timers = {&timers, &timers};*/
@@ -62,19 +71,19 @@ ttime get_time(void)
 struct bottom_half {
     struct bottom_half *next;
     struct bottom_half *prev;
-    void (*fn)(union cba_t);
-    union cba_t cba;
+    void (*fn)(cba_t);
+    cba_t cba;
 };
 
 /*struct list_head bottom_halves = { &bottom_halves, &bottom_halves };*/
 struct bottom_half bottom_halves = { &bottom_halves, &bottom_halves };
 
-int register_bottom_half(void (*fn)(union cba_t), union cba_t cba)
+int register_bottom_half(void (*fn)(cba_t), cba_t cba)
 {
     struct bottom_half *bh;
-    foreach(bh, bottom_halves) if (bh->fn == fn && bh->cba.int_ == cba.int_) return 0;
+    foreach(bh, bottom_halves) if (bh->fn == fn && GETCBA(bh->cba, void_) == GETCBA(cba, void_)) return 0;
 
-    if (!(bh = mem_alloc(sizeof(struct bottom_half)))) return -1;
+    if (!(bh = (struct bottom_half *)mem_alloc(sizeof(struct bottom_half)))) return -1;
 	/*dbg("register_bottom_half(0x%x,0x%x)=0x%x\n", fn, data, bh);*/
     bh->fn = fn;
     bh->cba = cba;
@@ -85,8 +94,8 @@ int register_bottom_half(void (*fn)(union cba_t), union cba_t cba)
 void check_bottom_halves()
 {
     struct bottom_half *bh;
-    void (*fn)(union cba_t);
-    union cba_t cba;
+    void (*fn)(cba_t);
+    cba_t cba;
     rep:
     if (list_empty(bottom_halves)) return;
     bh = bottom_halves.prev;
@@ -107,11 +116,15 @@ void check_timers(void)
 {
     ttime interval = get_time() - last_time;
     struct timer *t;
+
     foreach(t, timers) t->interval -= interval;
     ch:
     foreach(t, timers) if (t->interval <= 0) {
         struct timer *tt = t;
         del_from_list(tt);
+#ifdef LEAK_DEBUG_LIST        
+        //dbg("---%s\n", tt->fname);
+#endif
         tt->func(tt->cba);
         mem_free(tt);
         CHK_BH;
@@ -120,15 +133,33 @@ void check_timers(void)
     last_time += interval;
 }
 
+//void timer_redraw_screen(cba_t cba){
+    //dbg("timer_redraw_screen\n");
+    //ST_STOP;
+//#ifdef HAVE_SDL        
+//   if (sdl) {
+//        sdl_redraw_screen();
+//    }
+//    else
+//#endif
+//    {
+//        term_redraw_screen();
+//    }
+   // screen_drawed = 1;
+   // dbg("drawed=1\n");
+   // timer_redraw_screen_id = 0;
+//}
+
 #ifdef LEAK_DEBUG_LIST
-int debug_install_timer(char *file, int line, ttime t, void (*func)(union cba_t), union cba_t cba)
+int debug_install_timer(char *file, int line, ttime t, void (*func)(cba_t), cba_t cba, char *fname)
 #else    
-int install_timer(ttime t, void (*func)(union cba_t), union cba_t cba)
+int install_timer(ttime t, void (*func)(cba_t), cba_t cba)
 #endif    
 {
     struct timer *tm, *tt;
 #ifdef LEAK_DEBUG_LIST
     tm = debug_mem_alloc(file, line, sizeof(struct timer));
+    tm->fname = fname;
 #else            
     tm = mem_alloc(sizeof(struct timer));
 #endif        
@@ -177,8 +208,8 @@ void kill_timer(int id)
 #endif        
         k++;
     }
-    if (!k) internal("trying to kill nonexisting timer");
-    if (k >= 2) internal("more timers with same id");
+    if (!k) internal_("trying to kill nonexisting timer");
+    if (k >= 2) internal_("more timers with same id");
 }
 
 
@@ -195,32 +226,37 @@ ttime get_timer_time(int id)
 void *get_handler(int fd, int tp)
 {
     if (fd < 0 || fd >= FD_SETSIZE) {
-        internal("get_handler: handle %d >= FD_SETSIZE %d", fd, FD_SETSIZE);
+        internal_("get_handler: handle %d >= FD_SETSIZE %d", fd, FD_SETSIZE);
         return NULL;
     }
     switch (tp) {
         case H_READ:    return threads[fd].read_func;
         case H_WRITE:   return threads[fd].write_func;
         case H_ERROR:   return threads[fd].error_func;
-        case H_DATA:    return threads[fd].cba.void_; /* todo */
+        case H_DATA:    return GETCBA(threads[fd].cba, void_); 
     }
-    internal("get_handler: bad type %d", tp);
+    internal_("get_handler: bad type %d", tp);
     return NULL;
 }
 
 #ifdef LEAK_DEBUG_LIST
-void debug_set_handlers(char *file, int line, int fd, void (*read_func)(union cba_t), void (*write_func)(union cba_t), void (*error_func)(union cba_t), union cba_t cba)
+void debug_set_handlers(char *file, int line, int fd, 
+        void (*read_func)(cba_t), void (*write_func)(cba_t), void (*error_func)(cba_t), 
+        cba_t cba, 
+        char *read_fname, char *write_fname, char *error_fname)
 #else
-void set_handlers(int fd, void (*read_func)(union cba_t), void (*write_func)(union cba_t), void (*error_func)(union cba_t), union cba_t cba)
+void set_handlers(int fd, void (*read_func)(cba_t), void (*write_func)(cba_t), void (*error_func)(cba_t), cba_t cba)
 #endif
 {
     /*dbg("set_handlers(%d,%p,%p,%p,%p)\n",fd,read_func,write_func,error_func,cba.void_);*/
     
     if (fd!=0) sock_debug(fd, "set_handlers %p,%p,%p %p",read_func,write_func,error_func,cba);
+#if !defined(_MSC_VER) && !defined(__MINGW32__)
     if (fd < 0 || fd >= FD_SETSIZE) {
-        internal("set_handlers: handle %d >= FD_SETSIZE %d", fd, FD_SETSIZE);
+        internal_("set_handlers: handle %d >= FD_SETSIZE %d", fd, FD_SETSIZE);
         return;
     }
+#endif
     threads[fd].read_func = read_func;
     threads[fd].write_func = write_func;
     threads[fd].error_func = error_func;
@@ -252,14 +288,17 @@ void set_handlers(int fd, void (*read_func)(union cba_t), void (*write_func)(uni
 #ifdef LEAK_DEBUG_LIST
 	threads[fd].file=file;
 	threads[fd].line=line;
+    threads[fd].read_fname = read_fname;
+    threads[fd].write_fname = write_fname;
+    threads[fd].error_fname = error_fname;
 #endif	
 }
 
 #define NUM_SIGNALS 32
 
 struct signal_handler {
-    void (*fn)(union cba_t);
-    union cba_t cba;
+    void (*fn)(cba_t);
+    cba_t cba;
     int critical;
 };
 
@@ -290,11 +329,11 @@ void got_signal(int sig)
     check_for_select_race();
 }
 
-void install_signal_handler(int sig, void (*fn)(union cba_t), union cba_t cba, int critical)
+void install_signal_handler(int sig, void (*fn)(cba_t), cba_t cba, int critical)
 {
     struct sigaction sa;
     if (sig >= NUM_SIGNALS || sig < 0) {
-        internal("bad signal number: %d", sig);
+        internal_("bad signal number: %d", sig);
         return;
     }
     memset(&sa, 0, sizeof sa);
@@ -311,7 +350,7 @@ void install_signal_handler(int sig, void (*fn)(union cba_t), union cba_t cba, i
 
 int pending_alarm = 0;
 
-void alarm_handler(union cba_t cba)
+void alarm_handler(cba_t cba)
 {
     pending_alarm = 0;
     check_for_select_race();
@@ -351,10 +390,15 @@ int check_signals(void)
     return r;
 }
 
-void sigchld(union cba_t cba)
+void sigchld(cba_t cba)
 {
     /*wait(NULL);*/
-    waitpid(-1, NULL, WNOHANG);
+    int pid;
+    
+    pid = waitpid(-1, NULL, WNOHANG);
+//    dbg("sigchld  pid=%d\n", pid);
+    if (pid == sound_pid) abort_sound(); 
+    
 }
 
 void set_sigcld()
@@ -382,17 +426,23 @@ void select_loop(void (*init)(void))
         int n, i;
         struct timeval tv;
         struct timeval *tm = NULL;
+
+/*        screen_drawed = 0; 
+        printf("drawed=0\n");*/
+
         check_signals();
         check_timers();
 
-        check_sdevlist(sdevlist);
-        if (!list_empty(timers)) {
-            ttime tt = ((struct timer *)&timers)->next->interval + 1;
-            if (tt < 0) tt = 0;
-            tv.tv_sec = tt / 1000;
-            tv.tv_usec = (tt % 1000) * 1000;
-            tm = &tv;
+       // printf("drawed==%d\n", screen_drawed);
+#if 0
+        if (!timer_redraw_screen_id && !screen_drawed){
+            ST_START;
+            timer_redraw_screen_id = install_timer(80, timer_redraw_screen, CBA0);
+         //   dbg("\ninstall\n");
         }
+        screen_drawed = 0;
+        //printf("drawed=0\n");
+#endif
 #ifdef HAVE_SDL        
         if (sdl) {
             sdl_redraw_screen();
@@ -402,6 +452,15 @@ void select_loop(void (*init)(void))
         {
             term_redraw_screen();
         }
+
+        if (!list_empty(timers)) {
+            ttime tt = ((struct timer *)&timers)->next->interval + 1;
+            if (tt < 0) tt = 0;
+            tv.tv_sec = tt / 1000;
+            tv.tv_usec = (tt % 1000) * 1000;
+            tm = &tv;
+        }
+
         memcpy(&x_read, &w_read, sizeof(fd_set));
         memcpy(&x_write, &w_write, sizeof(fd_set));
         memcpy(&x_error, &w_error, sizeof(fd_set));
@@ -445,7 +504,7 @@ void select_loop(void (*init)(void))
        /* sound(800);
         ST_START;      */
         if (n < 0) {
-			err=errno;
+			err=sock_errno;
             critical_section = 0;
             uninstall_alarm();
             if (err != EINTR) {
@@ -480,7 +539,7 @@ void select_loop(void (*init)(void))
 #endif							
                     }
                     
-                    raise(SIGSEGV);
+                    internal_("select_loop() select failed");
                     /*dbg("DIE\n");
                     
                     sleep(1000000);
@@ -496,6 +555,15 @@ void select_loop(void (*init)(void))
 /*      printf("sel: %d\n", n);*/
 /*      dbg(".\n");*/
         check_timers();
+
+                    /*dbg("R:");
+                    for (i = 0; i < 32; i++) if (FD_ISSET(i, &x_read)) {dbg("%d,", i); }
+                    dbg("  W:");
+                    for (i = 0; i < 32; i++) if (FD_ISSET(i, &x_write)) {dbg("%d,", i); }
+                    dbg("  E:");
+                    for (i = 0; i < 32; i++) if (FD_ISSET(i, &x_error)) {dbg("%d,", i); }
+                    dbg("\n");*/
+
         i = -1;
         while (n > 0 && ++i < w_max) {
             int k = 0;
@@ -503,21 +571,36 @@ void select_loop(void (*init)(void))
             printf("A %d : %d,%d,%d\n",i,FD_ISSET(i, &x_read),FD_ISSET(i, &x_write),FD_ISSET(i, &x_error));*/
             if (FD_ISSET(i, &x_read)) {
                 if (threads[i].read_func) {
+                    //dbg("R%d\n", i);
+#ifdef LEAK_DEBUG_LIST                    
+                    //dbg("---%s\n", threads[i].read_fname);
+#endif
                     threads[i].read_func(threads[i].cba);
+                    //dbg("end R%d\n", i);
                     CHK_BH;
                 }
                 k = 1;
             }
             if (FD_ISSET(i, &x_write)) {
                 if (threads[i].write_func) {
+                    //dbg("W%d\n", i);
+#ifdef LEAK_DEBUG_LIST                    
+                    //dbg("---%s\n", threads[i].write_fname);
+#endif
                     threads[i].write_func(threads[i].cba);
+                    //dbg("end W%d\n", i);
                     CHK_BH;
                 }
                 k = 1;
             }
             if (FD_ISSET(i, &x_error)) {
                 if (threads[i].error_func) {
+                    //dbg("X%d\n", i);
+#ifdef LEAK_DEBUG_LIST                    
+                    //dbg("---%s\n", threads[i].error_fname);
+#endif
                     threads[i].error_func(threads[i].cba);
+                    //dbg("end X%d\n", i);
                     CHK_BH;
                 }
                 k = 1;
