@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -57,9 +57,23 @@
 #define COL_BG         COL(0x07)
 #define COL_NORM       COL(0x07)
 #define COL_INV        COL(0x38)
-#define COL_YELLOW     COL(0x43) 
-#define COL_DARKYELLOW COL(0x03) 
-#define COL_RED        COL(0x41) 
+
+#define COL_DARKRED     COL(0x01) 
+#define COL_DARKGREEN   COL(0x02) 
+#define COL_DARKYELLOW  COL(0x03) 
+#define COL_DARKBLUE    COL(0x04) 
+#define COL_DARKMAGENTA COL(0x05)
+#define COL_DARKCYAN    COL(0x06)
+#define COL_GREY        COL(0x07)
+
+#define COL_RED         COL(0x41) 
+#define COL_GREEN       COL(0x42) 
+#define COL_YELLOW      COL(0x43) 
+#define COL_BLUE        COL(0x44) 
+#define COL_MAGENTA     COL(0x45)
+#define COL_CYAN        COL(0x46)
+#define COL_WHITE       COL(0x47)
+
 
 #define ESC_TIMEOUT         200
 
@@ -76,6 +90,7 @@
 #define ACK_TIMEOUT 20000
 #define REMOVE_TIMEOUT 10000
 #define DELAY_AFTER_REPLICATION 1000
+#define DELAY_AFTER_UPDATE_ROTAR 500
 
 /* sec */
 #define NET_GLOBAL_EXPIRE 60
