@@ -39,7 +39,7 @@ int main(int argc, char *argv[]){
         return 1;
     }
 
-    if (argc >= 1){
+    if (argc > 1){
         freq = atoi(argv[1]);
         if (freq){ 
             arg=1193180/freq;
