@@ -2,15 +2,14 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
-
 char *get_wwl(char *buf, char *wwl){
     safe_strncpy(buf, wwl,5);
     return buf;
@@ -88,21 +87,15 @@ void draw_bigdigit(int x, int y, int num){
     
 }
 
-
-
-
-
-
-
-
 struct stats *init_stats(){
     struct stats *st;
 
     st = (struct stats *)g_new0(struct stats, 1); 
 
-    st->wwls = g_hash_table_new(g_str_hash, g_str_equal);
-    st->dxcs = g_hash_table_new(g_str_hash, g_str_equal);
-    st->excs = g_hash_table_new(g_str_hash, g_str_equal);
+    st->wwls  = g_hash_table_new(g_str_hash, g_str_equal);
+    st->dxcs  = g_hash_table_new(g_str_hash, g_str_equal);
+    st->prefs = g_hash_table_new(g_str_hash, g_str_equal);
+    st->excs  = g_hash_table_new(g_str_hash, g_str_equal);
     st->calls = g_hash_table_new(g_str_hash, g_str_equal);
     st->hours = g_hash_table_new(g_str_hash, g_str_equal);
     
@@ -118,17 +111,22 @@ void free_stats(struct stats *st){
     
     g_hash_table_destroy(st->wwls);
     g_hash_table_destroy(st->dxcs);
+    g_hash_table_destroy(st->prefs);
     g_hash_table_destroy(st->excs);
     g_hash_table_destroy(st->calls);
     g_hash_table_destroy(st->hours);
+    CONDGFREE(st->mycontinent);
+    CONDGFREE(st->mydxcc);
     g_free(st);
     
 }
 
 
 void clear_stats(struct stats *st){
+    struct dw_item *dxi;
+    char s[120];
+
  /*   dbg("clear_stats(%p)\n", st);*/
-    
     st->first_date = G_MAXINT;
     st->last_date  = 0;
     
@@ -137,11 +135,13 @@ void clear_stats(struct stats *st){
     st->nwwlp = 0;
     st->nexcp = 0;
     st->ndxcp = 0;
+    st->nprefs = 0;
     st->ntotal = 0;
     
-    g_hash_table_foreach_remove(st->wwls, free_gpointer_item, NULL);
-    g_hash_table_foreach_remove(st->dxcs, free_gpointer_item, NULL);
-    g_hash_table_foreach_remove(st->excs, free_gpointer_item, NULL);
+    g_hash_table_foreach_remove(st->wwls,  free_gpointer_item, NULL);
+    g_hash_table_foreach_remove(st->dxcs,  free_gpointer_item, NULL);
+    g_hash_table_foreach_remove(st->prefs, free_gpointer_item, NULL);
+    g_hash_table_foreach_remove(st->excs,  free_gpointer_item, NULL);
     g_hash_table_foreach_remove(st->calls, free_gpointer_item, NULL);
     g_hash_table_foreach_remove(st->hours, free_gpointer_item, NULL);
     
@@ -149,15 +149,24 @@ void clear_stats(struct stats *st){
     if (st->odxwwl)      {g_free(st->odxwwl);      st->odxwwl       = NULL;}
     if (st->odxoperator) {g_free(st->odxoperator); st->odxoperator  = NULL;}
     st->odxqrb_int = 0;
+
+    dxi = get_dw_item_by_call(dw, ctest->pcall);
+    if (!dxi) internal_("unknown dxcc of my callsign");
+
+    CONDGFREE(st->mycontinent);
+    st->mycontinent = g_strdup(dxi->continent);
+    CONDGFREE(st->mydxcc);
+    st->mydxcc = g_strdup(get_dxcc(dw, s, ctest->pcall));
 }
 
 int find_preferred_dxc(int type, char *dxc){
 	char *prefs9[]  = {"ES","LA","LY","OH","OH0","OJ0","OY","OZ","SM","TF","YL",NULL};
 	char *prefs10[] = {"G","GD","GI","GJ","GM","GU","GW",NULL};
     char *prefs11[] = {"F",NULL}; 
+    char *prefs16[] = {"F","TK", "TM",NULL}; 
 	char **prefs;
 	int multi=1;
-	int i;
+	int i;  
 	
 	switch(type){
 		case 9: 
@@ -172,6 +181,11 @@ int find_preferred_dxc(int type, char *dxc){
 			prefs=prefs11;
 			multi=3;
 			break;
+        case 16:
+            prefs=prefs16;
+            multi=4;
+            break;
+
 		default:
 			return 1;	
 	}
@@ -186,11 +200,13 @@ int find_preferred_dxc(int type, char *dxc){
 #define SAD_INT(item,value) {if (q->item!=value) band->dirty_save=1;q->item=value;}
 
 void update_stats(struct band *band, struct stats *st, struct qso *q){
-    gint date,qsop,anew;
-    gchar s[120];
+    gint date, qsop, anew;
+    gchar s[120], *c;
     gpointer orig_key;
     struct cntpts *orig_value;
-    gint wwls, dxcs, excs;
+    gint wwls, dxcs, prefs, excs;
+    struct dw_item *dwi;
+    int oldmult;
 
     qsop=0;
     if (band->stats_thread && band->stats_thread!=g_thread_self()){
@@ -203,7 +219,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         stats_thread_create(band);
         return;
     }
-    if (band->stats==st) LOCK(band->stats);
+    if (band->stats==st) MUTEX_LOCK(band->stats);
     
 /*    dbg("update_stats(%p, %c, %p)\n", st, b->bandchar, q);*/
     if (!q->error){
@@ -213,7 +229,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     }
     
     if (q->error) {
-        SAD_INT(new,0);
+        SAD_INT(new_,0);
         SAD_INT(qsop,0);
         SAD_INT(susploc,0);
         SAD_INT(unkcall,0);
@@ -226,6 +242,13 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     dbg("   dupe=%d  qsop=%d \n", q->dupe, q->qsop);*/
     
     get_raw_call(s, q->callsign);
+    if (is_rover(q->callsign) && q->locator){
+        strcat(s, "_");
+        safe_strncpy0(s + strlen(s), q->locator, 4+1);
+    } 
+
+    //dbg("update_stats(%s)\n", s);    
+
     if (g_hash_table_lookup_extended(st->calls, s, &orig_key, (gpointer) &orig_value)){
         SAD_INT(dupe,1);
     }else{
@@ -241,23 +264,23 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
             qsop = ctest->qsomult;
             break;
 		case 3: /* FIXME! it's really correct? */
-			qsop = ((q->qrb/50)*2+1 ) * ctest->qsomult;
+			qsop = (int)(((q->qrb/50)*2+1 ) * ctest->qsomult);
 			break;	
         case 4: /* WWL ring */
             qsop = qsopwr(ctest->pwwlo, q->locator) * ctest->qsomult;
             break;
 		case 5: /* statute miles (anglicka mile) */
-			qsop = q->qrb/1.6094 * ctest->qsomult;	
+			qsop = (int)(q->qrb/1.6094 * ctest->qsomult);
 			break;
 		case 6: /* 1 * qso-points multi + 1 point for CW QSO FIXME! bug in taclog? */
 			qsop = ctest->qsomult;
 			if (q->mode==MOD_CW_CW) q->qsop++;
 			break;
 		case 7: /* distance(min 50) */	
-			qsop = (q->qrb>50 ? q->qrb : 50) * ctest->qsomult;
+			qsop = (int)((q->qrb>50 ? q->qrb : 50) * ctest->qsomult);
 			break;
 		case 8: /* distance(min 10) */	
-			qsop = (q->qrb>10 ? q->qrb : 10) * ctest->qsomult;
+			qsop = (int)( (q->qrb>10 ? q->qrb : 10) * ctest->qsomult );
 			break;
 		case 9:
 		case 10:
@@ -266,8 +289,76 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 				find_preferred_dxc(ctest->qsop_method,
 						get_dxcc(dw,s,q->callsign));
 			break;
-
-            
+        case 12:
+            qsop = iaru_round(q->qrb) * ctest->qsomult;
+            break;
+        case 13:
+//            dbg("call=%s \n", q->callsign);
+            dwi = get_dw_item_by_call(dw, q->callsign);
+            if (!dwi){
+//                dbg("Don't know dxcc of %s\n", ctest->pcall);
+                qsop=0;              
+                break;
+            }
+            if (strcasecmp(dwi->continent, st->mycontinent)==0){ // same continent
+                c = get_dxcc(dw, s, q->callsign);
+//                dbg("  dxcc=%s  mydxcc=%s\n", c, st->mydxcc);
+                if (c && strcasecmp(c, st->mydxcc)==0){  // same DXCC
+                    qsop = 0;               // same dxcc 0
+                }else{
+//                    dbg("  cont=%s\n", dwi->continent);
+                    if (strcasecmp(dwi->continent, "NA")==0)
+                        qsop = 2;           // NA has 2 points
+                    else
+                        qsop = 1;           // others only 1
+                }
+            }else{
+                qsop = 3;                   // other continent
+            }
+            break;
+        case 14:
+//            dbg("call=%s \n", q->callsign);
+            dwi = get_dw_item_by_call(dw, q->callsign);
+            if (!dwi){
+//                dbg("Don't know dxcc of %s\n", ctest->pcall);
+                qsop=0;              
+                break;
+            }
+            qsop = 1;
+            if (band->qrg_min<10000) qsop = 2; // bellow 10 MHz
+            if (strcasecmp(dwi->continent, st->mycontinent)==0){ // same continent
+                c = get_dxcc(dw, s, q->callsign);
+//                dbg("  dxcc=%s  mydxcc=%s\n", c, st->mydxcc);
+                if (c && strcasecmp(c, st->mydxcc)==0){  // same DXCC
+                    qsop = 1;               // same dxcc 0
+                }else{
+//                    dbg("  cont=%s\n", dwi->continent);
+                    if (strcasecmp(dwi->continent, "NA")==0)
+                        qsop *= 2;           // NA has 2 points
+                    else
+                        qsop *= 1;           // others only 1
+                }
+            }else{
+                qsop *= 3;                   // other continent
+            }
+            break;
+        case 15:    // ARRL VHF contests http://www.arrl.org/contests/rules/2008/june-vhf.html
+            dbg("%d\n", band->qrg_min);
+            if (band->qrg_min > 2000000)  
+                qsop = 4;   // 2,3 GHz and above
+            else if (band->qrg_min > 500000)
+                qsop = 3;   // 902 MHz, 1,3 GHz 
+            else if (band->qrg_min > 200000)
+                qsop = 2;   // 222 MHz, 432 MHz
+            else 
+                qsop = 1;    // 144 MHz and below
+            break;
+        case 16:
+            qsop = iaru_round(q->qrb) * ctest->qsomult *
+				find_preferred_dxc(ctest->qsop_method,
+						get_dxcc(dw,s,q->callsign));
+            qsop *= qsomult16(band);
+            break;
     }
     if (q->dupe) qsop=0;
     SAD_INT(qsop,qsop);
@@ -282,16 +373,17 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         if (st->odxwwl)      { g_free(st->odxwwl);      st->odxwwl  = NULL; }
         if (st->odxoperator) { g_free(st->odxoperator); st->odxoperator  = NULL;}
          
+        st->odx         = q;
         st->odxqrb_int  = (int)q->qrb;
         st->odxcall     = g_strdup(q->callsign);
         st->odxwwl      = g_strdup(q->locator);  
-        st->odxoperator = g_strdup(q->operator);
+        st->odxoperator = g_strdup(q->operator_);
     }
 
     q->unkcall=!find_wwl_by_call(cw, q->callsign);
     q->susploc = get_susp_loc(cw, dw, q->callsign, q->locator) & 0xff;
     
-    anew=q->new;
+    anew=q->new_;
 
     if (ctest->wwlused){    
         get_wwl(s, q->locator);
@@ -321,21 +413,53 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         g_hash_table_insert(st->dxcs, g_strdup(s), orig_value);
         anew |= NEW_DXC;
     }
-        
 
-    if (ctest->excused){
-        safe_strncpy0(s, q->exc, 20);
-        if (g_hash_table_lookup_extended(st->excs, s, &orig_key, (gpointer) &orig_value)){
-	        orig_value->count++; 
-			orig_value->points+=qsop;
-            anew &= ~NEW_EXC;
+    if (ctest->prefmult>0){
+        get_pref(s, q->callsign);
+        if (g_hash_table_lookup_extended(st->prefs, s, &orig_key, (gpointer) &orig_value)){
+            orig_value->count++; 
+            orig_value->points+=qsop;
+            anew &= ~NEW_PREF;
         }else{
             orig_value = g_new0(struct cntpts, 1);
             orig_value->count = 1;
             orig_value->points = qsop;
-            g_hash_table_insert(st->excs, g_strdup(s), orig_value);
-            anew |= NEW_EXC;
+            g_hash_table_insert(st->prefs, g_strdup(s), orig_value);
+            anew |= NEW_PREF;
         }
+    }
+        
+        
+
+    if (ctest->excused>0){
+        safe_strncpy0(s, q->exc, 20);
+        if (strcasecmp(ctest->excname, "USACA")==0){
+            usaca_multiple_exc(s);
+        }
+        if (*s){
+            if (g_hash_table_lookup_extended(st->excs, s, &orig_key, (gpointer) &orig_value)){
+                orig_value->count++; 
+                orig_value->points+=qsop;
+                anew &= ~NEW_EXC;
+            }else{
+                orig_value = g_new0(struct cntpts, 1);
+                orig_value->count = 1;
+                orig_value->points = qsop;
+                g_hash_table_insert(st->excs, g_strdup(s), orig_value);
+                anew |= NEW_EXC;
+            }
+            if (ctest->qsop_method == 12){ // RSGB DC
+                int max = rsgbdc_maxcount(q->exc);
+                if (orig_value->count <= max) {
+                    st->nexcp++;
+                    anew |= NEW_EXC;
+                    //dbg("protekce\n");
+                }
+                //dbg("%-12s exc %s can be counted %d times (after this qso %d times) st->nexcp=%d\n", q->callsign, q->exc, max, orig_value->count, st->nexcp); 
+            }
+        }
+
+        
     } 
 
     safe_strncpy0(s, q->time_str, 3);
@@ -353,38 +477,53 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         }
     }
     
-    SAD_INT(new,anew);
-    
-    wwls = g_hash_table_size(st->wwls);
-    dxcs = g_hash_table_size(st->dxcs);
-    excs = g_hash_table_size(st->excs);
+    wwls  = g_hash_table_size(st->wwls);
+    dxcs  = g_hash_table_size(st->dxcs);
+    prefs = g_hash_table_size(st->prefs);
 
+    st->nwwlp = wwls;
+    if (ctest->qsop_method != 12)
+        st->nexcp = g_hash_table_size(st->excs);
+    st->ndxcp = dxcs;
+    
+    excs = st->nexcp;
+    oldmult = st->nmult;
     switch (ctest->total_method){
         case 1:      
+            st->nmult = (ctest->wwlmult  ? wwls  * ctest->wwlmult  : 1 ) *
+                        (ctest->prefmult ? prefs * ctest->prefmult : 1 ) *
+                        (ctest->dxcmult  ? dxcs  * ctest->dxcmult  : 1 ) *
+                        (ctest->excmult  ? excs  * ctest->excmult  : 1 );
+            ;  
             st->ntotal = ( st->nqsop +
                            wwls * ctest->wwlbonu + 
                            dxcs * ctest->dxcbonu +
                            excs * ctest->excbonu 
-                         ) *
-                           (ctest->wwlmult ? wwls * ctest->wwlmult : 1 ) *
-                           (ctest->dxcmult ? dxcs * ctest->dxcmult : 1 ) *
-                           (ctest->excmult ? excs * ctest->excmult : 1 )
-                         ;  
+                         ) * st->nmult;
+
             break;              
         case 2:      
+        case 3:
+            st->nmult = wwls  * ctest->wwlmult  +
+                        prefs * ctest->prefmult +
+                        dxcs  * ctest->dxcmult  +
+                        excs  * ctest->excmult;  
             st->ntotal = ( st->nqsop +
                            wwls * ctest->wwlbonu + 
                            dxcs * ctest->dxcbonu +
                            excs * ctest->excbonu 
-                         ) *
-                         ( wwls * ctest->wwlmult +
-                           dxcs * ctest->dxcmult +
-                           excs * ctest->excmult 
-                         );  
+                         ) * st->nmult;
             break;              
     }
+    if (st->nmult != oldmult)
+        anew |= NEW_MULT;
+    else
+        anew &= ~NEW_MULT;
+    
+    SAD_INT(new_,anew);
+    
 x:;    
-    if (band->stats==st) UNLOCK(band->stats);
+    if (band->stats==st) MUTEX_UNLOCK(band->stats);
 }
 
 void recalc_stats(struct band *band){
@@ -446,8 +585,30 @@ void recalc_all_qrbqtf(struct contest *ctest){
 void redraw_stats(struct band *band){
     gchar *c;
     struct stats *st=band->stats;    
+    int nqsos, nqsop, nwwlp, nexcp, ndxcp, ntotal;
+    
+    MUTEX_LOCK(band->stats);
+
+    nqsos  = st->nqsos;
+    nqsop  = st->nqsop;
+    nwwlp  = st->nwwlp;
+    nexcp  = st->nexcp;
+    ndxcp  = st->ndxcp;
+    ntotal = st->ntotal;
+    if (ctest->qsoglob){
+        nqsos  = ctest->allb_nqsos;
+        nqsop  = ctest->allb_nqsop;
+        nwwlp  = ctest->allb_nwwlp;
+        nexcp  = ctest->allb_nexcp;
+        ndxcp  = ctest->allb_ndxcp;
+        ntotal = ctest->allb_ntotal;
+    }
+    if (ctest->total_method == 3){
+        ntotal = ctest->allb_nqsop * (ctest->allb_ndxcp + ctest->allb_nexcp);
+        ctest->allb_ntotal = ntotal; // for HF subwin
+    }
+
     
-    LOCK(band->stats);
     print_text(STATS_X,STATS_Y,-1,VTEXT(T_SES_ODX),COL_NORM);
     if (st->odxcall && st->odxwwl){
         c = g_strdup_printf(VTEXT(T_SES_ODX_DATA), st->odxcall, st->odxwwl, st->odxqrb_int, st->odxoperator);
@@ -455,21 +616,20 @@ void redraw_stats(struct band *band){
         g_free(c);
     }    
     print_text(STATS_X,STATS_Y+2,term->x-QSONR_WIDTH-BAND_WIDTH-3,VTEXT(T_SES_STAT_TIT),COL_NORM);
-    c = g_strdup_printf("%4d  %6d  %4d  %4d   %4d  %7d",
-            st->nqsos, st->nqsop, 
-            g_hash_table_size(st->excs), 
-            g_hash_table_size(st->wwls), 
-            g_hash_table_size(st->dxcs),
-            st->ntotal);
+    c = g_strdup_printf("%4d  %6d  %4d  %4d   %4d  %7d %6.1f",
+            nqsos, nqsop, 
+            nexcp, nwwlp, ndxcp,
+            ntotal,
+            (double)ntotal/(double)nqsos);
     print_text(STATS_X,STATS_Y+3,term->x-QSONR_WIDTH-BAND_WIDTH-3,c,COL_NORM);
     g_free(c);
 
-    if (st->nqsos){
+/*    if (st->nqsos){
         c = g_strdup_printf(VTEXT(T_SES_AVG), (double)st->ntotal/(double)st->nqsos);
         print_text(term->x-QSONR_WIDTH-BAND_WIDTH,ORIG_Y+1,BAND_WIDTH-1, c, COL_NORM); 
         g_free(c);
-    }
-    UNLOCK(band->stats);
+    }*/
+    MUTEX_UNLOCK(band->stats);
     
 }
 
@@ -490,6 +650,22 @@ void sf_wwl_func(gpointer key, gpointer value, gpointer user_data){
 }
 
 
+void sf_miss_exc_func(gpointer key, gpointer value, gpointer user_data){
+    gchar       *missexc;
+    struct miss_struct *ms;
+    int maxlen = 0;
+    
+    missexc   = (gchar *) key;  
+    ms        = (struct miss_struct *) user_data;
+
+    if (g_hash_table_lookup_extended(ms->st->excs, missexc, NULL, NULL)) return;
+
+    maxlen = strlen(missexc);
+    if (maxlen > ms->maxlen) ms->maxlen = maxlen;
+    g_index_array_add(ms->ia, g_strdup(missexc));
+}
+
+
 gint compare_colon_int (gconstpointer a, gconstpointer b){
 	gchar **ca, **cb, *c;
 	int ia, ib;
@@ -497,16 +673,25 @@ gint compare_colon_int (gconstpointer a, gconstpointer b){
 	ca=(gchar **)a;
 	cb=(gchar **)b;
 
-	c=index(*ca, ':');
+	c=strchr(*ca, ':');
 	if (c) ia=atoi(c+1);
 	else ia=0;
-	c=index(*cb, ':');
+	c=strchr(*cb, ':');
 	if (c) ib=atoi(c+1);
 	else ib=0;
 
 	return ib-ia;
 }
 
+gint compare_string (gconstpointer a, gconstpointer b){
+	gchar **ca, **cb;
+
+	ca=(gchar **)a;
+	cb=(gchar **)b;
+
+	return strcmp(*ca, *cb);
+}
+
 #define COLS 4
 void add_sf_wwls(struct band *b){
     GString *gs;
@@ -542,6 +727,100 @@ void add_sf_wwls(struct band *b){
     g_index_array_free_all(ia);
 }
 
+////////////
+
+void sf_exc_func(gpointer key, gpointer value, gpointer user_data){
+    gchar       *exc;
+    struct cntpts *n;
+    GIndexArray *ia;
+    
+    
+    exc   = (gchar *)       key;  
+	n     = (struct cntpts *) value;
+    ia    = (GIndexArray *) user_data;
+
+    g_index_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", exc, n->points, n->count));
+}
+
+void add_sf_excs(struct band *b){
+    GString *gs;
+    GIndexArray *ia;
+    int i,j,k,lines;
+    gchar *c;
+    
+    gs = g_string_new("");
+    fifo_addf(F, "");
+    fifo_addf(F, VTEXT(T_SF_EXCS), g_hash_table_size(b->stats->excs));
+    fifo_addf(F, VTEXT(T_SF_EXCS_UNDER));
+   /* for (i=0;i<COLS;i++) g_string_append(gs, " EXC  QSO-p cnt    ");
+	fifo_addf(F, gs->str);*/
+    
+    ia = g_index_array_new();
+    g_hash_table_foreach(b->stats->excs, sf_wwl_func, ia);
+    g_index_array_qsort(ia, compare_colon_int);
+    g_string_truncate(gs, 0);
+    j = 0;
+    lines=(ia->len+COLS-1)/COLS;
+    
+    for (i=0; i<lines ; i++){
+        for (j=0;j<COLS; j++){
+            k = i+j*lines;
+            if (k>=ia->len) continue;
+            c = (gchar *)g_index_array_index(ia, k);
+            g_string_append(gs, c);        
+        }
+        fifo_addf(F, "%s", gs->str);
+        g_string_truncate(gs, 0);
+    }
+    g_string_free(gs,1);
+    g_index_array_free_all(ia);
+}
+
+#define MISS_EXC_COLS 10
+
+void miss_compute_ms(struct miss_struct *ms, struct band *b){
+    ms->ia = g_index_array_new();
+    ms->st = b->stats;
+    ms->maxlen = 0;
+    g_hash_table_foreach(excdb->vexc, sf_miss_exc_func, ms);
+    g_index_array_qsort(ms->ia, compare_string);
+}
+
+void add_sf_miss_excs(struct band *b){
+    GString *gs;
+//    GIndexArray *ia;
+    int i,j,k,lines;
+    gchar *c;
+    struct miss_struct ms;
+    
+    if (!excdb) return;
+    gs = g_string_new("");
+    fifo_addf(F, "");
+    fifo_addf(F, VTEXT(T_SF_MISS_EXCS), g_hash_table_size(excdb->vexc) - g_hash_table_size(b->stats->excs));
+    fifo_addf(F, VTEXT(T_SF_MISS_EXCS_UNDER));
+    
+    miss_compute_ms(&ms, b);
+
+    g_string_truncate(gs, 0);
+    j = 0;
+    lines=(ms.ia->len+MISS_EXC_COLS-1)/MISS_EXC_COLS;
+    
+    dbg("lines=%d\n", lines);
+    for (i=0; i<lines ; i++){
+        for (j=0;j<MISS_EXC_COLS; j++){
+            k = i+j*lines;
+            if (k>=ms.ia->len) continue;
+            c = (gchar *)g_index_array_index(ms.ia, k);
+            g_string_sprintfa(gs, "%-*s", ms.maxlen+1, c);        
+        }
+        fifo_addf(F, "%s", gs->str);
+        g_string_truncate(gs, 0);
+    }
+    g_string_free(gs,1);
+    g_index_array_free_all(ms.ia);
+}
+
+////////////////////
 void sf_dxc_func(gpointer key, gpointer value, gpointer user_data){
     gchar       *dxc;
     struct cntpts *n;
@@ -590,6 +869,46 @@ void add_sf_dxcs(struct band *b){
     g_index_array_free_all(ia);
 }
 
+void add_sf_prefs(struct band *b){
+    GString *gs;
+    GIndexArray *ia;
+    int i,j,k,lines;
+    gchar *c;
+    GHashTable *hash;
+
+    if (ctest->prefglob)
+        hash = ctest->allb_prefs;
+    else
+        hash = b->stats->prefs;
+
+    
+    gs = g_string_new("");
+    fifo_addf(F, "");
+    fifo_addf(F, VTEXT(T_SF_PREFS), g_hash_table_size(hash));
+    fifo_addf(F, VTEXT(T_SF_PREFS_UNDER));
+/*	for (i=0;i<COLS;i++) g_string_append(gs, " DXCC QSO-p cnt    ");
+	fifo_addf(F, gs->str);*/
+    
+    ia = g_index_array_new();
+    g_hash_table_foreach(hash, sf_dxc_func, ia);
+    g_index_array_qsort(ia, compare_colon_int);
+    g_string_truncate(gs, 0);
+    j = 0;
+    lines = (ia->len+COLS-1)/COLS;
+    for (i=0; i<lines ; i++){
+        for (j=0;j<COLS; j++){
+            k = i+j*lines;
+            if (k>=ia->len) continue;
+            c = (gchar *)g_index_array_index(ia, k);
+            g_string_append(gs, c);        
+        }
+        fifo_addf(F, "%s", gs->str);
+        g_string_truncate(gs, 0);
+    }
+    g_string_free(gs,1);
+    g_index_array_free_all(ia);
+}
+
 void add_sf_wkd_wwls(struct band *b){
     int w,len, lines, i, j, myw, myh;
     GString *gs;
@@ -732,7 +1051,7 @@ void add_sf_top_dx(struct band *b)
 	fifo_addf(F, VTEXT(T_SF_TOP_DX_UNDER));
 	for (i = 0; i < TOPS_NUM && i < b->qsos->len && top[i]; i++) {
 		q = top[i];
-		fifo_addf(F, "%-12s  %-6s  %u km by %-6s %s", q->callsign, q->locator, (int) q->qrb, q->operator, mode_msg[q->mode]);
+		fifo_addf(F, "%-12s  %-6s %4u km by %-6s %s", q->callsign, q->locator, (int) q->qrb, q->operator_, mode_msg[q->mode]);
 	}
 }
 
@@ -823,6 +1142,7 @@ void add_sf_hours(struct band *b){
 
 void recalc_statsfifo(struct band *b){
     
+    //dbg("recalc_statsfifo(%c, %d)\n", b->bandchar, b->dirty_statsf);
     if (!b || !b->dirty_statsf) return;
 
     stats_thread_join(b);
@@ -832,9 +1152,19 @@ void recalc_statsfifo(struct band *b){
     F->withouttime = 1;
 
     fifo_addf(F, "");
-    fifo_addf(F, VTEXT(T_SF_STATS_BAND), b->pband);
+    fifo_addf(F, VTEXT(T_SF_STATS_BAND), b->bandname);
     fifo_addf(F, VTEXT(T_SF_STATS_BAND_UNDER));
     fifo_addf(F, "");
+    fifo_addf(F, VTEXT(T_SF_CONTEST), ctest->tname);
+    if (ctest->total_method == 3 || ctest->qsoglob){
+        fifo_addf(F, VTEXT(T_SF_ALLBQSOS), ctest->allb_nqsos); 
+        fifo_addf(F, VTEXT(T_SF_ALLBQSOP), ctest->allb_nqsop);
+        fifo_addf(F, VTEXT(T_SF_ALLBPTS), ctest->allb_ntotal);
+    }
+    fifo_addf(F, VTEXT(T_SF_QSOS), b->stats->nqsos);  
+    fifo_addf(F, VTEXT(T_SF_QSOP), b->stats->nqsop);
+    fifo_addf(F, VTEXT(T_SF_PTS), b->stats->ntotal);
+    
 
     if (b->stats->nqsos>0){
         fifo_addf(F, VTEXT(T_SF_AVG_PTS), ((double)b->stats->ntotal) / ((double)b->stats->nqsos));
@@ -842,7 +1172,6 @@ void recalc_statsfifo(struct band *b){
 		fifo_addf(F, VTEXT(T_SF_QSO_HOUR), b->stats->qso_per_hour);
 		fifo_addf(F, VTEXT(T_SF_QSOP_HOUR), b->stats->pts_per_hour);
 		fifo_addf(F, VTEXT(T_SF_QSOP_50QSO), b->stats->pts_per_50qso);
-
     }
     fifo_addf(F, "");
 
@@ -850,11 +1179,16 @@ void recalc_statsfifo(struct band *b){
     	add_sf_wkd_wwls(b);
     	add_sf_cnt_in_wwls(b);
 	}
-	add_sf_wwls(b);
+    if (ctest->wwlused > 0) add_sf_wwls(b);
+	if (ctest->excused > 0) {
+        add_sf_excs(b);   
+        add_sf_miss_excs(b);
+    }
     add_sf_dxcs(b);
+    if (ctest->prefmult > 0) add_sf_prefs(b);
     add_sf_hours(b);
-	add_sf_top_dx(b);
-
+	if (ctest->wwlused > 0) add_sf_top_dx(b);
+   
     fifo_addf(F, "");
     b->dirty_statsf = 0;
     /*ST_STOP; */
@@ -885,7 +1219,7 @@ void export_stats_fifo(void) {
 
 gpointer stats_thread_func(gpointer data){
     struct qso *q;
-    int i;
+    int i, ret;
     struct band *band=(struct band*)data;
     struct stats *tmp;
     char s[256];
@@ -898,18 +1232,18 @@ gpointer stats_thread_func(gpointer data){
         q = get_qso(band, i);
         update_stats(band, band->tmpstats, q);
     }
-    LOCK(band->stats);
+    MUTEX_LOCK(band->stats);
     tmp=band->stats;
     band->stats=band->tmpstats;
     band->tmpstats=tmp;
-    UNLOCK(band->stats);
+    MUTEX_UNLOCK(band->stats);
 
 	minute_stats(band);
     
     band->dirty_save = 1;
     
     sprintf(s, "ST\n");
-    write(tpipe->threadpipe_write, s, strlen(s));
+    ret = write(tpipe->threadpipe_write, s, strlen(s));
     
     band->dirty_stats = 0;
    /* dbg("stats done %c\n", band->bandchar);*/
@@ -917,18 +1251,18 @@ gpointer stats_thread_func(gpointer data){
 }
 
 void stats_thread_create(struct band *band){
-    if (band->stats_thread) raise(SIGSEGV);
+    if (band->stats_thread) internal_("stats_threadcreate band->stats_thread");
     /*dbg("stats_thread_create(%c)\n", band->bandchar);*/
     band->stats_break=0;
     band->stats_thread=g_thread_create(stats_thread_func, (gpointer)band, TRUE, NULL);
-    if (!band->stats_thread) raise(SIGSEGV);
+    if (!band->stats_thread) internal_("stats_threadcreate !band->stats_thread");
 }
 
 void stats_thread_join(struct band *band){
     if (!band->stats_thread) return;
-   /* dbg("stats_thread_join(%c)...\n", band->bandchar);*/
+    dbg("join stats ...\n");
     g_thread_join(band->stats_thread);
- /*   dbg("done\n");*/
+    dbg("done\n");
     band->stats_thread=NULL;
     
 }
@@ -937,7 +1271,9 @@ void stats_thread_kill(struct band *band){
     if (!band->stats_thread) return;
 /*    dbg("stats_thread_kill(%c)\n", band->bandchar);*/
     band->stats_break=1;
+    dbg("join stats ...\n");
     g_thread_join(band->stats_thread);
+    dbg("done\n");
     band->stats_thread=NULL;
 }
 
@@ -959,6 +1295,7 @@ void minute_stats(struct band *band){
 	qso_per_hour=pts_per_hour=pts_per_50qso=0;
 	cnt50=50;
 	brk=0;
+    ctest->lastmultqso = NULL;
 	for (j=band->qsos->len-1;j>=0;j--){
 		if (band->stats_break) break;
 		qso = (struct qso *)g_ptr_array_index(band->qsos, j);
@@ -982,9 +1319,14 @@ void minute_stats(struct band *band){
 			brk|=0x02;
 		}
 
-		if (brk==0x03) break;
+        if ((qso->new_ & NEW_MULT) && !ctest->lastmultqso){
+            brk |= 0x04;
+            ctest->lastmultqso = qso;
+        }
+
+		if (brk==0x07) break;
 	}
-	LOCK(band->stats);
+	MUTEX_LOCK(band->stats);
 	if (band->stats->qso_per_hour!=qso_per_hour ||
 		band->stats->pts_per_hour!=pts_per_hour ||
 		band->stats->pts_per_50qso!=pts_per_50qso){
@@ -994,12 +1336,17 @@ void minute_stats(struct band *band){
 		band->stats->pts_per_50qso=pts_per_50qso;
 		band->dirty_statsf = 1;
 	}
-	UNLOCK(band->stats);
+	MUTEX_UNLOCK(band->stats);
 }
 
 void minute_stats_all(void){
     struct band *band;
-	int i;
+    time_t hb;
+    struct tm utc;
+	int i, brk;
+	char s[100], hourbefore[100], tenbefore[100];
+    int qso_per_10, qso_per_60;
+    struct qso *qso;
 
 	if (!aband) return;
 	
@@ -1010,9 +1357,129 @@ void minute_stats_all(void){
 		minute_stats(band);
 	}
 	if (aband->dirty_statsf) recalc_statsfifo(aband);
+
+	hb=time(NULL)-3600;
+    gmtime_r(&hb, &utc);
+    sprintf(hourbefore, "%4d%02d%02d%02d%02d",1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday, utc.tm_hour, utc.tm_min);  
+
+	hb=time(NULL)-600;
+    gmtime_r(&hb, &utc);
+    sprintf(tenbefore, "%4d%02d%02d%02d%02d",1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday, utc.tm_hour, utc.tm_min);  
+
+	if (ctest->allqsos->len==0) return;
+    qso_per_10 = qso_per_60 = 0;
+	brk=0;
+    for (i=ctest->allqsos->len-1;i>=0;i--){
+		qso = (struct qso *)g_ptr_array_index(ctest->allqsos, i);
+		if (qso->error || qso->dupe) continue;
+		
+        strcpy(s, qso->date_str);
+		strcat(s, qso->time_str);
+		
+        if (strcmp(s, hourbefore)>0)
+			qso_per_60++;
+		else
+			brk|=0x01;
+		
+        if (strcmp(s, tenbefore)>0)
+			qso_per_10++;
+		else
+			brk|=0x02;
+		
+		if (brk==0x03) break;
+    }    
+    if (ctest->qso_per_60 != qso_per_60 ||
+        ctest->qso_per_10 != qso_per_10){
+
+        ctest->qso_per_60 = qso_per_60;
+        ctest->qso_per_10 = qso_per_10;
+        redraw_later();
+    }
 }
 
-void timer_minute_stats_all(union cba_t cba){
+void timer_minute_stats_all(cba_t cba){
 	minute_stats_all();
 	ctest->minute_timer_id=install_timer(60000, timer_minute_stats_all, CBA0);
 }
+
+void stats_merge_prefs_fn(gpointer key, gpointer value, gpointer data){
+    gchar *pref = (gchar *)key;
+    struct cntpts *cp = (struct cntpts *)value;
+    gpointer orig_key;
+    struct cntpts *orig_value;
+
+    dbg("stats_merge_prefs_fn('%s') cp=%d %d\n", key, cp->count, cp->points); 
+    if (!g_hash_table_lookup_extended(ctest->allb_prefs, pref, &orig_key, (gpointer) &orig_value)){
+        orig_value = g_new0(struct cntpts, 1);
+        g_hash_table_insert(ctest->allb_prefs, g_strdup(pref), orig_value);
+    } 
+    orig_value->count += cp->count; 
+    orig_value->points += cp->points;
+}
+
+// called from main thread, no locking needed
+void recalc_allb_stats(){
+    int i;
+    struct band *band;
+
+    //dbg("recalc_allb_stats\n");
+    if (ctest->total_method != 3 && ctest->qsoglob == 0) return;
+//    dbg("do!\n");
+
+    ctest->allb_nqsos  = 0;
+    ctest->allb_nqsop  = 0;
+    ctest->allb_nwwlp  = 0;
+    ctest->allb_nexcp  = 0;
+    ctest->allb_nprefp = 0;
+    ctest->allb_ndxcp  = 0;
+    ctest->allb_nmult = 0;
+    ctest->allb_ntotal = 0;
+
+    if (ctest->prefmult > 0){
+        g_hash_table_foreach_remove(ctest->allb_prefs, free_gpointer_item, NULL);
+    }
+
+    for (i=0; i<ctest->bands->len; i++){
+        band = (struct band *)g_ptr_array_index(ctest->bands, i);
+        
+        ctest->allb_nqsos  += band->stats->nqsos;
+        ctest->allb_nqsop  += band->stats->nqsop;
+        ctest->allb_nwwlp  += band->stats->nwwlp;
+        ctest->allb_nexcp  += band->stats->nexcp;
+        ctest->allb_ndxcp  += band->stats->ndxcp;
+        ctest->allb_nmult  += band->stats->nmult;
+        ctest->allb_ntotal += band->stats->ntotal;
+
+        if (ctest->prefmult > 0){
+            g_hash_table_foreach(band->stats->prefs, stats_merge_prefs_fn, NULL);
+            ctest->allb_nprefp = g_hash_table_size(band->stats->prefs);
+        }
+    }
+    if (ctest->total_method == 4){
+        ctest->allb_nmult = ( ctest->wwlmult  ? ctest->allb_nwwlp * ctest->wwlmult : 1 ) *
+                            ( ctest->dxcmult  ? ctest->allb_ndxcp * ctest->dxcmult : 1 ) *
+                            ( ctest->excmult  ? ctest->allb_nexcp * ctest->excmult : 1 );
+
+        ctest->allb_ntotal = ( ctest->allb_nqsop + 
+                               ctest->allb_nwwlp * ctest->wwlbonu + 
+                               ctest->allb_ndxcp * ctest->dxcbonu + 
+                               ctest->allb_nexcp * ctest->excbonu 
+                             ) * ctest->allb_nmult;
+    }
+
+}
+
+int qsomult16(struct band *band){
+    int qsomult = 1;
+
+    if (band->qrg_min > 2000000)
+        qsomult = 10;     // 2,3 GHz and above
+    else if (band->qrg_min > 1000000)
+        qsomult = 5;      // 1,3 GHz
+    else if (band->qrg_min > 400000)
+        qsomult = 3;      // 432 MHz
+    else
+        qsomult = 1;      // 144 MHz
+
+    return qsomult;
+}
