@@ -3,8 +3,8 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -13,13 +13,15 @@
 #include "header.h"
 
 struct terminal *term;
+struct window *gseswin;
 
 int hard_write(int fd, char *p, int l)
 {
+	int w, t;
     if (fd<0) return -1;
     
-    int w = 1;
-    int t = 0;
+    w = 1;
+    t = 0;
     while (l > 0 && w) {
         if ((w = write(fd, p, l)) < 0) {
             if (errno == EINTR) continue;
@@ -85,9 +87,20 @@ void alloc_term_screen(struct terminal *term, int x, int y)
             term->dirty = 1;
         }
     }
+#ifdef FALL
+    int i;
+    if (term->fall != NULL) {
+        term->fall = mem_realloc(term->fall, x * y * sizeof(int));
+        for (i = 0; i < x * y; i++) term->fall[i] = i;
+    }else{
+        term->fall = mem_alloc(x * y * sizeof(int));
+        for (i = 0; i < x * y; i++) term->fall[i] = i;
+    } 
+#endif
+        
 }
 
-void in_term(union cba_t);
+void in_term(cba_t);
 void check_if_no_terminal(void);
 
 void clear_terminal(void)
@@ -110,12 +123,49 @@ void redraw_terminal_ev(struct terminal *term, int e)
     term->redrawing = 2;
     foreachback(win, term->windows) win->handler(win, &ev, 0);
     term->redrawing = 0;
-    
+#ifdef FALL
+    if (term->fall){
+        int i;
+        for (i = term->x * term->y; i >= 0; i--){
+            if (term->fall[i] < 0)
+                term->screen[i] = ' ';
+            else
+                term->screen[i] = term->screen[term->fall[i]];
+        }
+    }
+#endif
 }
 
-void redraw_terminal(union cba_t cba)
+#define ISSPACE(idx) ((term->screen[idx] & 0xff) == ' ' || (term->screen[idx] & 0xff) == '\0')
+
+#ifdef FALL
+void fall_func(union cba_t cba){
+    int s, d, j;
+//    static iii = 0;
+
+    if (term && term->fall){
+        for (j = 0; j < 1000; j++){
+            s = rand() % (term->x * term->y);
+            if (ISSPACE(s)) continue;
+            for (d = s + term->x; ISSPACE(d) && d < (term->x * term->y); d += term->x);
+            d -= term->x;
+            if (!ISSPACE(d)) continue;
+            term->fall[d] = term->fall[s];
+            term->fall[s] = -1;
+            break;
+        }
+
+        
+    }
+    gses->fall_id = install_timer(10, fall_func, CBA0);
+    redraw_later();
+}
+#endif
+
+void redraw_terminal(cba_t cba)
 {
     /*if (!term) return;*/
+    //dbg("redraw_terminal()\n");
     redraw_terminal_ev(term, EV_REDRAW);
 }
 
@@ -125,10 +175,24 @@ void redraw_terminal_all(void)
     redraw_terminal_ev(term, EV_RESIZE);
 }
 
-void redraw_later(void)
-{
-    if (!term) return;
+/*void timer_redraw(cba_t cba){
     register_bottom_half(redraw_terminal, CBA0);
+    gses->redraw_timer_id = 0;
+} */
+
+void redraw_later(void)
+{       
+    if (!term) return;
+    //dbg("redraw_later()\n");
+    register_bottom_half(redraw_terminal, CBA0);
+
+    /*if (!gses){
+        register_bottom_half(redraw_terminal, CBA0);
+        return;
+    }
+    
+    if (!gses->redraw_timer_id)
+        gses->redraw_timer_id = install_timer(100, timer_redraw, CBA0);*/
 }
 
 void erase_screen(void)
@@ -247,7 +311,7 @@ void get_parent_ptr(struct window *win, int *x, int *y)
 struct window *get_root_window()
 {
     if (list_empty(term->windows)) {
-        internal("terminal has no windows");
+        internal_("terminal has no windows");
         return NULL;
     }
     return (struct window *)term->windows.prev;
@@ -297,10 +361,15 @@ void empty_window_handler(struct window *win, struct event *ev, int fwd)
 
 void free_term_specs()
 {
-    free_list(term_specs);
+/*    free_list(term_specs);*/
+    while (term_specs.next != &(term_specs)) {
+        struct term_spec *a=term_specs.next; 
+        del_from_list(a); 
+        mem_free(a); 
+    }
 }
 
-struct list_head term_specs = {&term_specs, &term_specs};
+struct term_spec term_specs = {&term_specs, &term_specs};
 
 struct term_spec dumb_term = { NULL, NULL, "", 0, 0, 0, 0, 0, 0 };
 
@@ -372,6 +441,7 @@ struct terminal *init_term(int fdin, int fdout,
     if (fdin!=-1){
         set_handlers(fdin, in_term, NULL, destroy_terminal, CBA0);
     }
+    gseswin = win;
     return term;
 }
 
@@ -401,27 +471,35 @@ static void term_send_ucs(struct terminal *term, struct event *ev, unicode_val u
 
 #define get_bool_opt(s, opt) (s->opt == 1 ? 1 : 0)
 
-void in_term(union cba_t cba)
+void in_term(cba_t cba)
 {
     struct event *ev;
-    int r;
+    int r, err;
     char *iq, *c;
-    if (!(iq = mem_realloc(term->input_queue, term->qlen + ALLOC_GR))) {
+    if (!(iq = (char*)mem_realloc(term->input_queue, term->qlen + ALLOC_GR))) {
         destroy_terminal(CBA0);
         return;
     }
     term->input_queue = iq;
-    if ((r = read(term->fdin, iq + term->qlen, ALLOC_GR)) <= 0) {
-        if (r == -1 && errno != ECONNRESET) error("ERROR: error %d on terminal: could not read event", errno);
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	r = recv(term->fdin, iq + term->qlen, ALLOC_GR, 0); 
+    err = sock_errno;
+#else
+	r = read(term->fdin, iq + term->qlen, ALLOC_GR);
+    err = errno;
+#endif
+
+    if (r <= 0) {
+        if (r == -1/* && errno != ECONNRESET*/) internal_("ERROR: error %d on terminal: could not read event", err);
         destroy_terminal(CBA0);
         return;
     }
 #ifdef HAVE_SDL    
     if (sdl){
-        LOCK(sdl->eventpipestate);
+        MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate-=r;
 /*        dbg("r %d\n", sdl->eventpipestate);*/
-        UNLOCK(sdl->eventpipestate);
+        MUTEX_UNLOCK(sdl->eventpipestate);
     }
 #endif
 
@@ -431,7 +509,7 @@ void in_term(union cba_t cba)
     if (term->qlen < sizeof(struct event)) return;
     ev = (struct event *)iq;
     
-  /*  dbg("in_term [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);*/
+//    dbg("in_term [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
 /*    dbg("in_term qlen=%d\n", term->qlen);*/
 
     if (term->qlen > sizeof(struct event) &&    /* more than one event */
@@ -459,6 +537,7 @@ void in_term(union cba_t cba)
         
         xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
         if (!(ts = getenv("TERM"))) ts = "";
+//        dbg("in_term ts=%s\n", ts);
         strcpy(term->term, ts);
     
         if (!(ts = get_cwd()) && !(ts = stracpy(""))) goto neni_pamet;
@@ -470,7 +549,7 @@ void in_term(union cba_t cba)
         sync_term_specs();
     }
     if (ev->ev == EV_SDLRESIZE){
-        dbg("EV_SDLRESIZE(%d,%d) (%d,%d)\n", ev->x, ev->y, ev->mx, ev->my);
+//        dbg("EV_SDLRESIZE(%d,%d) (%d,%d)\n", ev->x, ev->y, ev->mx, ev->my);
 #ifdef HAVE_SDL        
         if (sdl) {
 			cfg->gfx_x=ev->mx;
@@ -497,7 +576,7 @@ void in_term(union cba_t cba)
 	if (ev->ev == EV_MOUSE && preferred_func(ev))
 		term_send_event(term, ev);
 	if (ev->ev == EV_KBD) {
-		if (upcase(ev->x) == 'L' && ev->y == KBD_CTRL) {
+		if (upcase(ev->x) == (int)'L' && ev->y == KBD_CTRL) {
 			ev->ev = EV_REDRAW;
 			ev->x = term->x;
 			ev->y = term->y;
@@ -559,8 +638,8 @@ inline int getcompcode(int c)
     return (c<<1 | (c&4)>>2) & 7;
 }
 
-char frame_dumb[48] =  "   ||||++||++++++--|-+||++--|-+----++++++++     ";
-char frame_vt100[48] = "aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjla    ";
+char frame_dumb[49] =  "   ||||++||++++++--|-+||++--|-+----++++++++     ";
+char frame_vt100[49] = "aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjla    ";
 
 /* For UTF8 I/O */
 static char frame_vt100_u[48] = {
@@ -727,7 +806,7 @@ void term_redraw_screen()
     term->dirty = 0;
 }
 
-void destroy_terminal(union cba_t cba)
+void destroy_terminal(cba_t cba)
 {
     if (!term) return;
     terminate=1;
@@ -778,6 +857,11 @@ void set_char(int x, int y, unsigned c)
     if (x >= 0 && x < term->x && y >= 0 && y < term->y) term->screen[x + term->x * y] = c;
 }
 
+void set_last_char(int x, int y, unsigned c)
+{
+    if (x >= 0 && x < term->x && y >= 0 && y < term->y) term->last_screen[x + term->x * y] = c;
+}
+
 unsigned get_char(int x, int y)
 {
     if (x >= term->x) x = term->x - 1;
@@ -856,6 +940,7 @@ void print_text(int x, int y, int l, char *text, unsigned c)
 
 void set_cursor(int x, int y, int altx, int alty)
 {
+    //dbg("set_cursor(%d,%d, %d,%d)\n", x, y, altx, alty);
     term->dirty = 1;
     if (term->spec->block_cursor) x = altx, y = alty;
     if (x >= term->x) x = term->x - 1;
@@ -876,16 +961,16 @@ void exec_thread(char *path, int p)
     if (path[1 + strlen(path + 1) + 1]) unlink(path + 1 + strlen(path + 1) + 1);
 }
 
-void close_handle(union cba_t cba)
+void close_handle(cba_t cba)
 {
-    int h = cba.int_;
+    int h = (int)GETCBA(cba, int_);
     close(h);
     set_handlers(h, NULL, NULL, NULL, CBA0);
 }
 
-void unblock_terminal(union cba_t cba)
+void unblock_terminal(cba_t cba)
 {
-    close_handle((union cba_t)term->blocked);
+    close_handle((cba_t)term->blocked);
     term->blocked = -1;
     set_handlers(term->fdin, in_term, NULL, destroy_terminal, CBA0);
     unblock_itrm(term->fdin);
@@ -925,7 +1010,7 @@ void exec_on_terminal(char *path, char *delete, int fg)
                 set_handlers(term->fdin, NULL, NULL, destroy_terminal, CBA0);
                 /*block_itrm(term->fdin);*/
             } else {
-                set_handlers(blockh, close_handle, NULL, close_handle, (union cba_t)blockh);
+                set_handlers(blockh, close_handle, NULL, close_handle, (cba_t)blockh);
             }
         }
     } else {
@@ -950,7 +1035,7 @@ void exec_on_terminal(char *path, char *delete, int fg)
 void do_terminal_function(char code, char *data)
 {
     char *x_data;
-    if (!(x_data = mem_alloc(strlen(data) + 2))) return;
+    if (!(x_data = (char *)mem_alloc(strlen(data) + 2))) return;
     x_data[0] = code;
     strcpy(x_data + 1, data);
     exec_on_terminal(NULL, x_data, 0);
