@@ -1,9 +1,9 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2011  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This program is free software; you can redistribute it and/or                                                        
-    modify it under the terms of the GNU General Public License                                                          
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
@@ -20,7 +20,7 @@ int export_all_bands_titlpage(){
     FILE *f;
     char callbuf[20];
     int ignoreerror=0;
-    gchar *template;
+    gchar *template_;
     gchar s[4100];
     struct stat st;
     char *c;
@@ -46,8 +46,8 @@ int export_all_bands_titlpage(){
         goto mem;
     }
    
-    template=g_new0(gchar, st.st_size+1);
-    if (fread(template, st.st_size, 1, f)!=1){
+    template_=g_new0(gchar, st.st_size+1);
+    if (fread(template_, st.st_size, 1, f)!=1){
         fclose(f);
   /*      log_addf(VTEXT(T_CANT_READ_S), SHAREDIR"/titlpage.html");*/
         goto mem;
@@ -56,7 +56,7 @@ int export_all_bands_titlpage(){
     filetemplate=1;
 mem:;
 
-    if (!filetemplate) template=g_strdup(txt_titlpage);
+    if (!filetemplate) template_=g_strdup(txt_titlpage);
             
     
     gs=g_string_new("");
@@ -82,13 +82,13 @@ mem:;
         }
         
         ptr=0;
-        while((c=index(template+ptr, '$'))!=NULL){
-            fwrite(template+ptr, c-template-ptr, 1, f);
+        while((c=strchr(template_+ptr, '$'))!=NULL){
+            fwrite(template_+ptr, c-template_-ptr, 1, f);
             idlen=strspn(c+1, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_");
             
             safe_strncpy0(s, c+1, idlen+1);
             log_addf("id=%s idlen=%d\n", s, idlen);
-            ptr=c-template+1+idlen;
+            ptr=c-template_+1+idlen;
 
             if (strcasecmp(s, "PCall")==0){
                 fprintf(f, "%s", qh(gs, ctest->pcall));
@@ -162,7 +162,7 @@ mem:;
             }
             
         }
-        fprintf(f, "%s", template+ptr);
+        fprintf(f, "%s", template_+ptr);
         
         
 	    fclose(f);
