@@ -10,6 +10,17 @@
 
 #include "header.h"
 
+#define STORE_WIZZ_INT(name) \
+    else if (strcasecmp(k, #name)==0){ \
+        wi->name = atoi(item); \
+    } \
+
+#define STORE_WIZZ_STR(name) \
+    else if (strcasecmp(k, #name)==0){ \
+        if (wi->name) g_free(wi->name); \
+        wi->name = g_strdup(item); \
+    } \
+
 struct wizz *wizz;
 
 struct wizz *init_wizz(void){
@@ -24,7 +35,8 @@ void free_wizz_item(struct wizz_item *wi){
     
     if (wi->tname) g_free(wi->tname);
     if (wi->pexch) g_free(wi->pexch);
-    if (wi->pexchtype) g_free(wi->pexchtype);
+    if (wi->excname) g_free(wi->excname);
+    if (wi->bands) g_free(wi->bands);
     
     g_free(wi);
 }
@@ -59,10 +71,9 @@ int load_wizz_from_file(struct wizz *wizz, gchar *filename){
     FILE *f;
     GString *gs;
     struct wizz_item *wi=NULL;
-    gchar **items, *k, *v;
+    gchar **items, *k, *item;
     long int pos;
 
-    gs = g_string_sized_new(100);
     
     f=NULL;
     pos=0;
@@ -75,6 +86,8 @@ int load_wizz_from_file(struct wizz *wizz, gchar *filename){
         }
     }
     
+    gs = g_string_sized_new(100);
+    
     while(1){
         if (filename){
             if (!safe_fgets(gs, f, 1)) break;
@@ -95,11 +108,12 @@ int load_wizz_from_file(struct wizz *wizz, gchar *filename){
             if (!wi) {
                 wi = g_new0(struct wizz_item,1);
                 g_ptr_array_add(wizz->items, wi);
-                
+                wi->wwlcfm = 1;
+                wi->exccfm = 1;
             }
 
             /*dbg("BEGIN '%s'\n", gs->str+1);*/
-            if (wi->tname) g_free(wi->tname);
+            CONDGFREE(wi->tname)
             wi->tname = tname;  
 /*            dbg("  set tname='%s' \n", tname);*/
             continue;
@@ -111,61 +125,38 @@ int load_wizz_from_file(struct wizz *wizz, gchar *filename){
             continue;
         }
         k=items[0];
-        v=items[1];
+        item=items[1];
         
-        /*dbg("   '%s'='%s'\n", k, v);*/
-
-        if      (g_strcasecmp(k, "rstused")==0)
-            wi->rstused=atoi(v);
-        else if (g_strcasecmp(k, "qsoused")==0)
-            wi->qsoused=atoi(v); 
-        else if (g_strcasecmp(k, "wwlused")==0)
-            wi->wwlused=atoi(v); 
-        else if (g_strcasecmp(k, "excused")==0)
-            wi->excused=atoi(v); 
-        else if (g_strcasecmp(k, "qsopointsmultiplier")==0)
-            wi->qsomult=atoi(v); 
-        else if (g_strcasecmp(k, "minqsopoints")==0)
-            wi->minqsop=atoi(v); 
-        else if (g_strcasecmp(k, "maxqsopoints")==0)
-            wi->maxqsop=atoi(v); 
-        else if (g_strcasecmp(k, "qsocalcmethod")==0)
-            wi->qsop_method=atoi(v); 
-        else if (g_strcasecmp(k, "wwlbonus")==0)
-            wi->wwlbonu=atoi(v); 
-        else if (g_strcasecmp(k, "wwlmultiplier")==0)
-            wi->wwlmult=atoi(v); 
-        else if (g_strcasecmp(k, "dxccbonus")==0)
-            wi->dxcbonu=atoi(v); 
-        else if (g_strcasecmp(k, "dxccmultiplier")==0)
-            wi->dxcmult=atoi(v); 
-        else if (g_strcasecmp(k, "optionalexchangebonus")==0)
-            wi->excbonu=atoi(v); 
-        else if (g_strcasecmp(k, "optionalexchangemultiplier")==0)
-            wi->excmult=atoi(v); 
-        else if (g_strcasecmp(k, "totalcalcmethod")==0)
-            wi->total_method=atoi(v); 
-        
-        else  if (g_strcasecmp(k, "wwltype")==0){
-            wi->wwltype=0;
-            if (g_strcasecmp(v, "normal")==0) wi->wwltype=1;
-            if (g_strcasecmp(v, "extended")==0) wi->wwltype=2;
-        }else if (g_strcasecmp(k, "shorttonormal")==0){
-            wi->shorttonormal=0;
-            if (g_strcasecmp(v, "yes")==0) wi->shorttonormal=1;
-        }else if (g_strcasecmp(k, "insertoptionalexchange")==0){
-            wi->insertpexch=0;
-            if (g_strcasecmp(v, "yes")==0) wi->insertpexch=1;
-        }else if (g_strcasecmp(k, "optionalexchangetype")==0){
-
-            if (wi->pexchtype) g_free(wi->pexchtype);
-            wi->pexchtype = g_strdup(v);
-        }else if (g_strcasecmp(k, "ownoptionalexchange")==0){
-            /* no action */
-        }else 
+        /*dbg("   '%s'='%s'\n", k, item);*/
+        if (0){}
+        STORE_WIZZ_STR(pexch)
+        STORE_WIZZ_INT(rstused)
+        STORE_WIZZ_INT(qsoused)
+        STORE_WIZZ_INT(qsomult)
+        STORE_WIZZ_INT(qsoglob)
+        STORE_WIZZ_INT(tttype)
+        STORE_WIZZ_INT(wwlused)
+        STORE_WIZZ_INT(wwlbonu)
+        STORE_WIZZ_INT(wwlmult)
+        STORE_WIZZ_INT(wwlcfm)
+        STORE_WIZZ_INT(excused)
+        STORE_WIZZ_INT(excbonu)
+        STORE_WIZZ_INT(excmult)
+        STORE_WIZZ_INT(exccfm)
+        STORE_WIZZ_INT(exctype)
+        STORE_WIZZ_STR(excname)
+        STORE_WIZZ_INT(rstused)
+        STORE_WIZZ_INT(defrstr)
+        STORE_WIZZ_INT(prefmult)
+        STORE_WIZZ_INT(prefglob)
+        STORE_WIZZ_INT(dxcmult)
+        STORE_WIZZ_INT(dxcbonu)
+        STORE_WIZZ_INT(qsop_method)
+        STORE_WIZZ_INT(total_method)
+        STORE_WIZZ_STR(bands)
+        else{
             dbg("!!! unresolved key '%s'\n", k);
-        
-        
+        }
         g_strfreev(items);
     }
 
